/****************************************************************************/
// RT-Sample program to demonstrate how to make a Profibus DP Slave out of 
// a X-Bus board.
// In this sample all inputs and outputs of a X-DAD-4/U module are brougth
// onto the bus through the X-DPS-2 module.
// This is done in 2 steps:
// 1. Configuration: 
//    1.1. Configure the X-DPS-2 module
//    1.2. Open the profibus channels to the X-DPS-2 MDD that should bring 
//         the inputs and outputs to the bus.
//    1.3. Open channels to all inputs and outputs of the X-DAD-4/U
//         (DIO-8 and DIO-17 are not used)
// 2. Operation: 
//    2.1  Read inputs of the X-DAD-4 and write their data to the bus. 
//    2.2  Read output data from the bus and write them to the X-DAD-4. 
//    The communication function is called cyclically under the control of a 
//    Windows timer. For control purposes it display all values.
// 
// This sample uses channel 0 of the X-DPS-2.
// The X-DAD-4/U is configured as following:
// All analog inputs and outputs in +/-10V range. 
// Analog data is transfered in mV on the bus.
// Digital pins 0..7 as inputs, 9..16 as outputs, 8 and 17 not used.
// Slave configuration IDs: 99, 83, 32, 16	
//
// Copyright SORCUS Computer GmbH 2004                                     
/****************************************************************************/

#pragma option -r-

#define _RTLIB_                        // must be defined in every real time
// program before including max_lib.h
#include "max_lib.h"
#include "x_error.h"
#include "x_mdd.h"
#include "dos.h"
#include "stdlib.h"

#define PROGRAMM_NUMBER 0x9007	//set this to your needs
#define VERSION         '1'		//version of program
#define REVISION        'A'		//revision of program

// PDT structure
struct pdt_type
{
	MAX_PDT_HEAD    rcHead;
	void* MAXEXPORT main_proc;
	void* MAXEXPORT auto_init;
	void* MAXEXPORT user_init;
	
} rcPDT;

// parameter range structure
struct parameter_type
{
	MAX_TDT_TYPE rcTDT;                 // always before the first parameter !
	// offset 0
	MAX_ERROR    Error;                 // Errorcode
	USHORT		 usSlotXDAD4;				// slot where the X-DAD-4 is
	USHORT		 usLayerXDAD4;				// layer where the X-DAD-4 is
	USHORT		 usSlotXDPS2;				// slot where the X-DPS-2 is
	USHORT		 usLayerXDPS2;				// layer where the X-DPS-2 is
	USHORT		 usChannel;
	UCHAR			 ucStation;					// profibus address of this slave
	UCHAR		 	 aucSlaveConfiguration[4]; // IDs for profibus slave configuration
	MAXMODHND    hMyself;               // Handle for the CPU module
	MAX_OS_INFO  rcOsX;                 // Version info about OsX
	MAXMDDHND	 hXDAD4;						// MDD handle for X-DAD-4
	MAXMDDHND	 hXDPS2;						// MDD handle for X-DPS-2
	MAXCHLHND	 hBusCtrl;					// Channel handle for bus control
	MAXCHLHND	 hWatchdog;					// Channel handle for watchdog on X-DPS-2
	MAXCHLHND	 hAin;						// Channel handle analog inputs on X-DAD-4
	MAXCHLHND	 hDin;						// Channel handle digital inputs on X-DAD-4
	MAXCHLHND	 hAout;						// Channel handle analog outputs on X-DAD-4
	MAXCHLHND	 hDout;						// Channel handle digital outputs on X-DAD-4
	MAXCHLHND	 hAinBus;					// Channel handle for bus out of the AIN values on X-DPS-2
	MAXCHLHND	 hAoutBus;					// Channel handle for bus in of the AOUT values on X-DPS-2
	MAXCHLHND	 hDinBus;					// Channel handle for bus out of the DIN values on X-DPS-2
	MAXCHLHND	 hDoutBus;					// Channel handle for bus in of the DOUT values on X-DPS-2
} parameter;

/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
void MAXEXPORT MainProc(void)
{
   ULONG aulAin[4];
   USHORT ausAinBus[4];
   ULONG aulAout[4];
   USHORT ausAoutBus[4];
   ULONG ulSize;
   USHORT usDigVal, i;
    
	max_entry_proc();

   // trigger the watchdog
   parameter.Error = max_trigger_channel(parameter.hWatchdog);

   // acquire analog inputs
   ulSize = sizeof(aulAin);
   parameter.Error = max_read_channel_block (parameter.hAin, &ulSize, aulAin);
   // the AIN values are encoded: 1 = 1mV
   for(i=0;i<4;i++)
      ausAinBus[i] = aulAin[i] / 1000;
   // write analog input values to profibus
   ulSize = sizeof(ausAinBus);
   parameter.Error = max_write_channel_block (parameter.hAinBus, &ulSize, ausAinBus);

   // acquire digital inputs
   parameter.Error = max_read_channel_ushort (parameter.hDin, &usDigVal);
   // write digital input values to profibus
   ulSize = sizeof(UCHAR);
   parameter.Error = max_write_channel_block (parameter.hDinBus, &ulSize, &usDigVal);

   // read values for AOUT from the profibus
   ulSize = sizeof(ausAoutBus);
   parameter.Error = max_read_channel_block (parameter.hAoutBus, &ulSize, ausAoutBus);
   // write AOUT values to X-DAD-4
   // the AOUT values are encoded: 1 = 1mV
   for(i=0;i<4;i++)
      aulAout[i] = (ULONG)ausAoutBus[i] * 1000;
   ulSize = sizeof(aulAout);
   parameter.Error = max_write_channel_block (parameter.hAout, &ulSize, aulAout);

   // read values for DOUT from the profibus
   ulSize = sizeof(UCHAR);
   usDigVal = 0;
   parameter.Error = max_read_channel_block (parameter.hDoutBus, &ulSize, &usDigVal);
   // write DOUT values to X-DAD-4
   parameter.Error = max_write_channel_ushort (parameter.hDout, usDigVal);

	max_exit_proc();
}

/****************************************************************************/
// Basic Init task parameter
void MAXEXPORT AutoInit(void)
{
	max_entry_proc();
	
	max_init_lib(0);
	// get a handle for the X-MAX-1
	parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,&parameter.rcOsX, &parameter.hMyself);
	
	// bus configuration
	parameter.aucSlaveConfiguration[0] = 99; // ID for 4 analog bus out 
	parameter.aucSlaveConfiguration[1] = 83; // ID for 4 analog bus in
	parameter.aucSlaveConfiguration[2] = 32; // ID for 4 digital bus out 
	parameter.aucSlaveConfiguration[3] = 16; // ID for 4 digital bus in

	// default values for module loacations, channel and profibus address
	// the values may be changed before calling the UserInit procedure
	parameter.usSlotXDAD4 = 5;					// slot where the X-DAD-4 is
	parameter.usLayerXDAD4 = 0;				// layer where the X-DAD-4 is
	parameter.usSlotXDPS2 = 4;					// slot where the X-DPS-2 is
	parameter.usLayerXDPS2 = 0;				// layer where the X-DPS-2 is
	parameter.usChannel = 0;					// channel on the X-DPS-2 (0 or 1)	
	parameter.ucStation = 3;					// profibus address of this slave

	max_exit_proc();
};


/****************************************************************************/
// open all channels and do configuration
void MAXEXPORT UserInit(void)
{
	ULONG		   ulCfgSize;
	CPS_XDPS		rcCpsDps;
	CPS_XDAD4	rcCpsDad;
   USHORT      usMyTaskNo;
   MAX_TI_TYPE rcSchedule;
	
	usMyTaskNo = max_entry_proc();

	// get handles for the MDDs	
	parameter.Error = max_load_mdd (parameter.hMyself, 
											  parameter.usSlotXDAD4, 
	                      			  parameter.usLayerXDAD4, 0,
				          	 			  MDD_X_DAD_4U, NULL, 
											  &parameter.hXDAD4);
	
	parameter.Error = max_load_mdd (parameter.hMyself,
											  parameter.usSlotXDPS2, 
	                      			  parameter.usLayerXDPS2, 0,
				          	 			  MDD_X_DPS_2, NULL, 
											  &parameter.hXDPS2);

	// open control channel to configure slave
	rcCpsDps.usDevice  = DEVICE_CTRL;
   rcCpsDps.usAddress = parameter.ucStation;
   rcCpsDps.usChannel = parameter.usChannel;
   rcCpsDps.usFlags   = _CP_EXCLUSIVE;
   rcCpsDps.usMode    = 0;
   rcCpsDps.usTimeout = 0;
   rcCpsDps.usIndexFirst = 0;
   rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
	                                   &rcCpsDps, NULL, 0, &parameter.hBusCtrl);
	
   // download profibus configuration
  	ulCfgSize = sizeof(parameter.aucSlaveConfiguration);
	parameter.Error = max_write_channel_block (parameter.hBusCtrl,
	                                           &ulCfgSize, parameter.aucSlaveConfiguration);

	// start profibus
	parameter.Error = max_channel_control(parameter.hBusCtrl, CMD_START, 0, NULL);

    // open a watchdog channel
    rcCpsDps.usDevice = DEVICE_WATCHDOG;
    rcCpsDps.usFlags = 0;
    rcCpsDps.usIndexFirst = 0;
    rcCpsDps.usIndexLast = rcCpsDps.usIndexFirst;
    rcCpsDps.usTimeout = 1000;	
    parameter.Error = max_open_channel (parameter.hXDPS2, sizeof(rcCpsDps),
        											 &rcCpsDps, NULL, 0, &parameter.hWatchdog);

    // open a channel to all AOUTs
    rcCpsDad.usDevice	    = DEVICE_AOUT;
    rcCpsDad.usIndexFirst = 0;
    rcCpsDad.usIndexLast  = 3;
    rcCpsDad.usFlags		 = _CP_EXCLUSIVE + _CP_UNCORRECTED;
    rcCpsDad.usReadMode	 = IO_MODE_RAM;
    rcCpsDad.usWriteMode  = IO_MODE_DIRECT;
    rcCpsDad.usRange		 = RANGE_BIP_10V;
    parameter.Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
        											 &rcCpsDad, NULL, 0, &parameter.hAout);
    
    // open bus channel that receives the AOUT values from the bus
    rcCpsDps.usDevice  = DEVICE_BUS_OUT;
    rcCpsDps.usFlags   = _CP_EXCLUSIVE;
    rcCpsDps.usMode    = 0;
    rcCpsDps.usTimeout = 0;
    rcCpsDps.usIndexFirst = 0;             // AOUT are configuration element 0
    rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
    parameter.Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
        											&rcCpsDps, NULL, 0, &parameter.hAoutBus);
    /* DEVICE_BUS_OUT channel can be opened with callback, 
       but this increases the CPU load. Use it only for debugging! 
       parameter.Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
        												&rcCpsDps, DataCallback, 0, &parameter.hAoutBus);
    */
   
    // open a channel to all AINs
    rcCpsDad.usDevice	    = DEVICE_AIN_DIFF;
    rcCpsDad.usIndexFirst = 0;
    rcCpsDad.usIndexLast  = 3;
    rcCpsDad.usFlags		 = _CP_UNCORRECTED;
    rcCpsDad.usReadMode	 = IO_MODE_DIRECT;
    rcCpsDad.usWriteMode  = 0;
    rcCpsDad.usRange		 = RANGE_BIP_10V;
    parameter.Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
        											&rcCpsDad, NULL, 0, &parameter.hAin);
    
    // open bus channel that transmits the AIN values on the bus
    rcCpsDps.usDevice  = DEVICE_BUS_IN;
    rcCpsDps.usFlags   = _CP_EXCLUSIVE;
    rcCpsDps.usMode    = 0;
    rcCpsDps.usTimeout = 0;
    rcCpsDps.usIndexFirst = 1;             // AIN are configuration element 1
    rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
    parameter.Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
        											&rcCpsDps, NULL, 0, &parameter.hAinBus);
    
    // open a channel to DOUTs
    rcCpsDad.usDevice	    = DEVICE_DOUT;
    rcCpsDad.usIndexFirst = 9;
    rcCpsDad.usIndexLast  = 16;
    rcCpsDad.usFlags		 = _CP_EXCLUSIVE;
    rcCpsDad.usReadMode	 = IO_MODE_DIRECT;
    rcCpsDad.usWriteMode  = IO_MODE_DIRECT;
    parameter.Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
        											&rcCpsDad, NULL, 0, &parameter.hDout);
    
    // open bus in channel that receives the DOUT values from the bus
    rcCpsDps.usDevice  = DEVICE_BUS_OUT;
    rcCpsDps.usFlags   = _CP_EXCLUSIVE;
    rcCpsDps.usMode    = 0;
    rcCpsDps.usTimeout = 0;
    rcCpsDps.usIndexFirst = 2;             // DOUT are configuration element 2
    rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
    parameter.Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
        											&rcCpsDps, NULL, 0, &parameter.hDoutBus);
    
    // open a channel to DINs
    rcCpsDad.usDevice	    = DEVICE_DIN;
    rcCpsDad.usIndexFirst = 0;
    rcCpsDad.usIndexLast  = 7;
    rcCpsDad.usFlags		 = _CP_EXCLUSIVE;
    rcCpsDad.usReadMode	 = IO_MODE_DIRECT;
    rcCpsDad.usWriteMode  = 0;
    parameter.Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
        											&rcCpsDad, NULL, 0, &parameter.hDin);
    
    // open bus out channel that transmits the DIN values on the bus
    rcCpsDps.usDevice  = DEVICE_BUS_IN;
    rcCpsDps.usFlags   = _CP_EXCLUSIVE;
    rcCpsDps.usMode    = 0;
    rcCpsDps.usTimeout = 0;
    rcCpsDps.usIndexFirst = 3;             // DIN are configuration element 3
    rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
    parameter.Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
        											&rcCpsDps, NULL, 0, &parameter.hDinBus);

	// activate the task   
   rcSchedule.usPriority = 0;
   rcSchedule.ulHoldOff = 1;
   rcSchedule.ulIntervall = 1;   // 1 timer tic intervall (OsX default timer tic is 1 ms) 
   rcSchedule.ulCalls = 0;
   parameter.Error = max_wakeup_ti_task(parameter.hMyself, usMyTaskNo, &rcSchedule);
   
	max_exit_proc();
};


/****************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
	// the PDT-flags determine the task-type and
	// information how the data and parameter range are allocated
	
	//set flags for task type (here NI-task) etc. You may want to change this.
	rcPDT.rcHead.usFlags         = MAX_NI_TASK +	MAXPDT_PARAMETER_IN_PROG;
	
	// interrupt source (0 for NI-tasks)
	rcPDT.rcHead.usInterrupt     = 0;
	
	// the program number identifies the program uniquely
	rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;
	
	// number of global task procedures in the PDT
	rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;
	
	// the following lines are just for documentation and information purpose
	rcPDT.rcHead.ucType          = 1;   // always !
	rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
	rcPDT.rcHead.cVersion        = VERSION;
	rcPDT.rcHead.cRevision       = REVISION;
	rcPDT.rcHead.ucCPU           = MAX_CPU486;
	rcPDT.rcHead.ucCoProc        = _NOCOPROC;
	rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
	rcPDT.rcHead.ucProgType      = 0;   // not used
	rcPDT.rcHead.ulRes1          = 0;
	rcPDT.rcHead.ulRes2          = 0;
	rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet
	
	// address and size of the tasks data range
	// only when MAXPDT_DATA_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulDataAdr       = 0;
	rcPDT.rcHead.ulDataSize      = 0;
	
	// address and size of the tasks parameter range
	// only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
	rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);
	
	// initialize global procedure addresses in PDT
	rcPDT.main_proc        = MainProc;
	rcPDT.auto_init        = AutoInit;
	rcPDT.user_init        = UserInit;
	
	// transfer PDT information to the OsX
	max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
