/****************************************************************************/
// Sample program to demonstrate how to make a Profibus DP Slave out of 
// a X-Bus board.
// In this sample all inputs and outputs of a X-DAD-4/U module are brougth
// onto the bus through the X-DPS-2 module.
// This is done in 2 steps:
// 1. Configuration: 
//    1.1. Configure the X-DPS-2 module
//    1.2. Open the profibus channels to the X-DPS-2 MDD that should bring 
//         the inputs and outputs to the bus.
//    1.3. Open channels to all inputs and outputs of the X-DAD-4/U
//         (DIO-8 and DIO-17 are not used)
// 2. Operation: 
//    2.1  Read inputs of the X-DAD-4 and write their data to the bus. 
//    2.2  Read output data from the bus and write them to the X-DAD-4. 
//    The communication function is called cyclically under the control of a 
//    Windows timer. For control purposes it display all values.
// 
// This sample uses channel 0 of the X-DPS-2.
// The X-DAD-4/U is configured as following:
// All analog inputs and outputs in +/-10V range. 
// Analog data is transfered in mV on the bus.
// Digital pins 0..7 as inputs, 9..16 as outputs, 8 and 17 not used.
// Slave configuration IDs: 99, 83, 32, 16	
//
// Copyright SORCUS Computer GmbH 2002                                      
/****************************************************************************/

// History
// 17.10.2003: Callback functions and useage of INFO_DEVICE service added

#include "stdafx.h"
#include "dad4slave.h"
#include "dad4slaveDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BOARD 1           // Board ID as in control panel - SORCUS boards

// global variables are joined in a struct to make adaption
// into a realtime program easier (this could be used as task parameter than).
struct parameter_type
{
	USHORT		 usSlotXDAD4;				// slot where the X-DAD-4 is
	USHORT		 usLayerXDAD4;				// layer where the X-DAD-4 is
	USHORT		 usSlotXDPS2;				// slot where the X-DPS-2 is
	USHORT		 usLayerXDPS2;				// layer where the X-DPS-2 is
   USHORT       usChannel;             // used channel of the X-DPS-2 
	UCHAR			 ucStation;					// profibus address of this slave
	UCHAR		 	 aucSlaveConfiguration[4]; // IDs for profibus slave configuration
	MAXMODHND    hMyself;               // Handle for the CPU 
	MAXMDDHND	 hXDAD4;						// MDD handle for X-DAD-4
	MAXMDDHND	 hXDPS2;						// MDD handle for X-DPS-2
	MAXCHLHND	 hBusCtrl;					// Channel handle for bus control
	MAXCHLHND	 hWatchdog;					// Channel handle for watchdog on X-DPS-2
	MAXCHLHND	 hAin;						// Channel handle analog inputs on X-DAD-4
	MAXCHLHND	 hDin;						// Channel handle digital inputs on X-DAD-4
	MAXCHLHND	 hAout;						// Channel handle analog outputs on X-DAD-4
	MAXCHLHND	 hDout;						// Channel handle digital outputs on X-DAD-4
	MAXCHLHND	 hAinBus;					// Channel handle for bus out of the AIN values on X-DPS-2
	MAXCHLHND	 hAoutBus;					// Channel handle for bus in of the AOUT values on X-DPS-2
	MAXCHLHND	 hDinBus;					// Channel handle for bus out of the DIN values on X-DPS-2
	MAXCHLHND	 hDoutBus;					// Channel handle for bus in of the DOUT values on X-DPS-2
} parameter;

CDad4slaveDlg *prcDlg;

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg Dialogfeld

CDad4slaveDlg::CDad4slaveDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDad4slaveDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDad4slaveDlg)
	m_Din0 = FALSE;
	m_Din1 = FALSE;
	m_Din2 = FALSE;
	m_Din3 = FALSE;
	m_Din4 = FALSE;
	m_Din5 = FALSE;
	m_Din6 = FALSE;
	m_Din7 = FALSE;
	m_Dout0 = FALSE;
	m_Dout1 = FALSE;
	m_Dout2 = FALSE;
	m_Dout3 = FALSE;
	m_Dout4 = FALSE;
	m_Dout5 = FALSE;
	m_Dout6 = FALSE;
	m_Dout7 = FALSE;
	m_Aout0 = 0;
	m_Aout1 = 0;
	m_Aout2 = 0;
	m_Aout3 = 0;
	m_Ain0 = 0;
	m_Ain1 = 0;
	m_Ain2 = 0;
	m_Ain3 = 0;
	m_Board = 0;
	m_SlotDAD4 = 0;
	m_SlotDPS2 = 0;
	m_Station = 0;
	m_Channel = 0;
	//}}AFX_DATA_INIT
	// Beachten Sie, dass LoadIcon unter Win32 keinen nachfolgenden DestroyIcon-Aufruf bentigt
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CDad4slaveDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDad4slaveDlg)
	DDX_Control(pDX, IDC_EDIT_EVENTS, m_Events);
	DDX_Check(pDX, IDC_CHECK_DIN0, m_Din0);
	DDX_Check(pDX, IDC_CHECK_DIN1, m_Din1);
	DDX_Check(pDX, IDC_CHECK_DIN2, m_Din2);
	DDX_Check(pDX, IDC_CHECK_DIN3, m_Din3);
	DDX_Check(pDX, IDC_CHECK_DIN4, m_Din4);
	DDX_Check(pDX, IDC_CHECK_DIN5, m_Din5);
	DDX_Check(pDX, IDC_CHECK_DIN6, m_Din6);
	DDX_Check(pDX, IDC_CHECK_DIN7, m_Din7);
	DDX_Check(pDX, IDC_CHECK_DOUT0, m_Dout0);
	DDX_Check(pDX, IDC_CHECK_DOUT1, m_Dout1);
	DDX_Check(pDX, IDC_CHECK_DOUT2, m_Dout2);
	DDX_Check(pDX, IDC_CHECK_DOUT3, m_Dout3);
	DDX_Check(pDX, IDC_CHECK_DOUT4, m_Dout4);
	DDX_Check(pDX, IDC_CHECK_DOUT5, m_Dout5);
	DDX_Check(pDX, IDC_CHECK_DOUT6, m_Dout6);
	DDX_Check(pDX, IDC_CHECK_DOUT7, m_Dout7);
	DDX_Text(pDX, IDC_EDIT_AOUT0, m_Aout0);
	DDX_Text(pDX, IDC_EDIT_AOUT1, m_Aout1);
	DDX_Text(pDX, IDC_EDIT_AOUT2, m_Aout2);
	DDX_Text(pDX, IDC_EDIT_AOUT3, m_Aout3);
	DDX_Text(pDX, IDC_AIN_0, m_Ain0);
	DDX_Text(pDX, IDC_AIN_1, m_Ain1);
	DDX_Text(pDX, IDC_AIN_2, m_Ain2);
	DDX_Text(pDX, IDC_AIN_3, m_Ain3);
	DDX_Text(pDX, IDC_EDIT_BOARD, m_Board);
	DDV_MinMaxUInt(pDX, m_Board, 0, 7);
	DDX_Text(pDX, IDC_EDIT_DADSLOT, m_SlotDAD4);
	DDV_MinMaxUInt(pDX, m_SlotDAD4, 1, 6);
	DDX_Text(pDX, IDC_EDIT_DPSSLOT, m_SlotDPS2);
	DDV_MinMaxUInt(pDX, m_SlotDPS2, 1, 6);
	DDX_Text(pDX, IDC_EDIT_SLAVE, m_Station);
	DDV_MinMaxUInt(pDX, m_Station, 3, 123);
	DDX_Text(pDX, IDC_EDIT_CHANNEL, m_Channel);
	DDV_MinMaxUInt(pDX, m_Channel, 0, 1);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDad4slaveDlg, CDialog)
	//{{AFX_MSG_MAP(CDad4slaveDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	ON_BN_CLICKED(IDC_BUTTON1, OnInfoState)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
	USHORT	 usMsgSize = 100;
   char		 sErrMsg[200], sMessage[200];

	max_get_error_message(Error, &usMsgSize, sErrMsg);
	sprintf (sMessage, "Error %04Xh in %s (%s)", Error, sText, sErrMsg);
	AfxMessageBox(sMessage);
	max_clear_error();
}

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg OnButtonStart: Init hardware, load mdds and configure modules
void CDad4slaveDlg::OnButtonStart() 
{
   MAX_ERROR	Error;
	MAX_VERSION	Version;
   ULONG       ulDate;
	char			szVersion[40], szDate[40];
	CString		sInfo, sInfo2;

   UpdateData (TRUE);

   // library initialization
	max_init_lib(0);

   // reset X-Bus 
   Error = max_reset_board(m_Board, 30);
	if (Error) 
   {
		ErrorHandler(Error, "max_reset_board");
		return;
	}

	// get a handle for the Windows CPU (MDDs are PC based)
	Error = max_connect_cpu (m_Board, 0, 0, NULL, &parameter.hMyself);
	if (Error) 
   {
		ErrorHandler(Error, "max_connect_cpu");
		return;
	}
	
	// bus configuration
	parameter.usSlotXDAD4  = m_SlotDAD4;	// slot where the X-DAD-4 is
	parameter.usLayerXDAD4 = 0;				// layer where the X-DAD-4 = 0
	parameter.usSlotXDPS2  = m_SlotDPS2;	// slot where the X-DPS-2 is
	parameter.usLayerXDPS2 = 0;				// layer where the X-DPS-2 = 0
   parameter.usChannel    = m_Channel;    // used profibus channel 
	parameter.ucStation    = m_Station;		// profibus address of this slave
   // the following values configure the number of inputs and outputs of the slave
   // The same values must be used on the side of the master to configure the slave.
	parameter.aucSlaveConfiguration[0] = 99; // ID for 4 analog bus out 
	parameter.aucSlaveConfiguration[1] = 83; // ID for 4 analog bus in
	parameter.aucSlaveConfiguration[2] = 32; // ID for 8 digital bus out 
	parameter.aucSlaveConfiguration[3] = 16; // ID for 8 digital bus in

   // load MDD for the X-DAD-4/U
	Error = max_load_mdd (parameter.hMyself, 
                         parameter.usSlotXDAD4, parameter.usLayerXDAD4, 
                         0, MDD_X_DAD_4U, NULL, 
								 &parameter.hXDAD4);
	if (Error) 
   {
		ErrorHandler(Error, "max_load_mdd for X-DAD-4");
		return;
	}
	
   // load MDD for the X-DPS-2
	Error = max_load_mdd (parameter.hMyself,
							    parameter.usSlotXDPS2, parameter.usLayerXDPS2, 
                         0, MDD_X_DPS_2, NULL, 
								 &parameter.hXDPS2);
	if (Error) 
   {
		ErrorHandler(Error, "max_load_mdd for X-DPS-2");
		return;
	}
   
   max_get_mdd_version(parameter.hXDPS2, &Version, &ulDate);
   max_get_version_string(Version, ulDate, szVersion, szDate);
   sInfo.Format("X-DPS-2: MDD Version %s loaded", szVersion);
   max_get_mdd_version(parameter.hXDAD4, &Version, &ulDate);
   max_get_version_string(Version, ulDate, szVersion, szDate);
	sInfo2.Format("\r\nX-DAD-4: MDD Version %s loaded", szVersion);
	AfxMessageBox(sInfo + sInfo2);

   Error = Configuration();
	if (Error) 
		return;

   // Create a timer
   m_nTimer = SetTimer(1, 100, 0);     // period = 100 ms
   
}

/****************************************************************************/
// Callback function for the bus configuration channel.
// Informs about changes in the state of the Slave.
// It is usefull for diagnostics to implement this callback.
MAX_CALLBACK CDad4slaveDlg::BusCtrlCallback(MAXCHLHND hChl, ULONG ulParam, ULONG ulSize, void *pData)
{
   CString sMsg, sOldText;
   UCHAR   aucData[4]; 

   memcpy(aucData, pData, 4);   
   switch (aucData[0])
   {	// Byte 0 determines the type of event, 
		// other bytes contain details about the event (not for all types)
   case XDPS_PARAM_ERROR: 
      sMsg = "\r\nError in parameters from master";
      break;
   case XDPS_CFG_ERROR: 
      sMsg.Format("\r\nError %X in configuration from master", aucData[1]);
      break;
   case XDPS_OFFLINE: 
      sMsg = "\r\nGone Offline";
      break;
   case XDPS_DATA_XCHG:
      sMsg.Format("\r\nGone to/Left data exchange state (state=%X)", aucData[1]);
		prcDlg->m_usStartCommunication = 1;
      break;
   case XDPS_RCV_CTRL_CMD:
      sMsg.Format("\r\nControl command %X received from master", aucData[1]);
      break;
   case XDPS_SLAVE_ADDR:
      sMsg.Format("\r\nNew slave address received (%X)", aucData[1]);
      break;
   case XDPS_BAUDRATE_DETECT:
      sMsg.Format("\r\nBaudrate detected %X", aucData[1]);
      break;
   case XDPS_WATCHDOG_TIMEOUT:
      sMsg = "\r\nWatchdog timeout";
      break;
   default:
      sMsg.Format("\r\nUnknown event %X, %X, %X, %X", 
                  aucData[0], aucData[1], aucData[2], aucData[3]);
      break;
   }
   prcDlg->m_Events.GetWindowText(sOldText);
   prcDlg->m_Events.SetWindowText(sOldText + sMsg);
}

/****************************************************************************/
// Callback function for the DEVICE_BUS_OUT channels.
// It is normally not recommended to use this - 
// just to analyze in case of failures
MAX_CALLBACK CDad4slaveDlg::DataCallback(MAXCHLHND hChl, ULONG ulParam, ULONG ulSize, void *pData)
{
	CString sOldText;

   prcDlg->m_Events.GetWindowText(sOldText);
   prcDlg->m_Events.SetWindowText(sOldText + "\r\nData telegram received");
}

/****************************************************************************/
// Procedure to configure channels
MAX_ERROR CDad4slaveDlg::Configuration ()
{
	ULONG		   ulCfgSize, ulSize;
	UCHAR			ucData = 0;
	USHORT		ausData[4] = {0,0,0,0};
	CPS_XDPS		rcCpsDps;
	CPS_XDAD4	rcCpsDad;
   MAX_ERROR	Error;

	// open control channel to configure slave
	rcCpsDps.usDevice  = DEVICE_CTRL;
   rcCpsDps.usAddress = parameter.ucStation;
   rcCpsDps.usChannel = parameter.usChannel;
   rcCpsDps.usFlags   = _CP_EXCLUSIVE;
   rcCpsDps.usMode    = 0;
   rcCpsDps.usTimeout = 0;
   rcCpsDps.usIndexFirst = 0;
   rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
	Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
                            &rcCpsDps, BusCtrlCallback, 1, &parameter.hBusCtrl);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hBusCtrl");
		return Error;
	}
	
   // download profibus configuration
  	ulCfgSize = sizeof(parameter.aucSlaveConfiguration);
	Error = max_write_channel_block (parameter.hBusCtrl,
	                                 &ulCfgSize,
											   parameter.aucSlaveConfiguration);
	if (Error) 
   {
		ErrorHandler(Error, "max_write_channel_block");
		return Error;
	}

	// start profibus
	Error = max_channel_control(parameter.hBusCtrl, CMD_START, 0, NULL);
	if (Error) 
   {
		ErrorHandler(Error, "max_channel_control");
		return Error;
	}

   // open a watchdog channel
	rcCpsDps.usDevice = DEVICE_WATCHDOG;
   rcCpsDps.usFlags = 0;
   rcCpsDps.usIndexFirst = 0;
   rcCpsDps.usIndexLast = rcCpsDps.usIndexFirst;
	rcCpsDps.usTimeout = 1000;	
	Error = max_open_channel (parameter.hXDPS2, sizeof(rcCpsDps),
	                          &rcCpsDps, NULL, 0, 
									  &parameter.hWatchdog);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hWatchdog");
		return Error;
	}

	// open a channel to all AOUTs
	rcCpsDad.usDevice	    = DEVICE_AOUT;
   rcCpsDad.usIndexFirst = 0;
   rcCpsDad.usIndexLast  = 3;
   rcCpsDad.usFlags		 = _CP_EXCLUSIVE + _CP_UNCORRECTED;
   rcCpsDad.usReadMode	 = IO_MODE_RAM;
   rcCpsDad.usWriteMode  = IO_MODE_DIRECT;
   rcCpsDad.usRange		 = RANGE_BIP_10V;
   Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
	               			  &rcCpsDad, NULL, 0, 
									  &parameter.hAout);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hAout");
		return Error;
	}
  
   // open bus channel that receives the AOUT values from the bus
   rcCpsDps.usDevice  = DEVICE_BUS_OUT;
   rcCpsDps.usChannel = parameter.usChannel;
   rcCpsDps.usFlags   = _CP_EXCLUSIVE;
   rcCpsDps.usMode    = 0;
   rcCpsDps.usTimeout = 0;
   rcCpsDps.usIndexFirst = 0;             // AOUT are configuration element 0
   rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
	Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
	                         &rcCpsDps, NULL, 0, 
                            &parameter.hAoutBus);
/* DEVICE_BUS_OUT channel can be opened with callback, 
	but this increases the CPU load. Use it only for debugging! 
	Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
	                         &rcCpsDps, DataCallback, 0, 
                            &parameter.hAoutBus);
*/
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hAoutBus");
		return Error;
	}

   // open a channel to all AINs
	rcCpsDad.usDevice	    = DEVICE_AIN_DIFF;
   rcCpsDad.usIndexFirst = 0;
   rcCpsDad.usIndexLast  = 3;
   rcCpsDad.usFlags		 = _CP_UNCORRECTED;
   rcCpsDad.usReadMode	 = IO_MODE_DIRECT;
   rcCpsDad.usWriteMode  = 0;
   rcCpsDad.usRange		 = RANGE_BIP_10V;
   Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
	                          &rcCpsDad, NULL, 0, 
									  &parameter.hAin);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hAin");
		return Error;
	}
   
   // open bus channel that transmits the AIN values on the bus
   rcCpsDps.usDevice  = DEVICE_BUS_IN;
   rcCpsDps.usChannel = parameter.usChannel;
   rcCpsDps.usFlags   = _CP_EXCLUSIVE;
   rcCpsDps.usMode    = 0;
   rcCpsDps.usTimeout = 0;
   rcCpsDps.usIndexFirst = 1;             // AIN are configuration element 1
   rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
	Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
	                         &rcCpsDps, NULL, 0, 
                            &parameter.hAinBus);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hAinBus");
		return Error;
	}

	// open a channel to DOUTs
	rcCpsDad.usDevice	    = DEVICE_DOUT;
   rcCpsDad.usIndexFirst = 9;
   rcCpsDad.usIndexLast  = 16;
   rcCpsDad.usFlags		 = _CP_EXCLUSIVE;
   rcCpsDad.usReadMode	 = IO_MODE_DIRECT;
   rcCpsDad.usWriteMode  = IO_MODE_DIRECT;
   Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
	              				  &rcCpsDad, NULL, 0, 
									  &parameter.hDout);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hDout");
		return Error;
	}

   // open bus in channel that receives the DOUT values from the bus
   rcCpsDps.usDevice  = DEVICE_BUS_OUT;
   rcCpsDps.usChannel = parameter.usChannel;
   rcCpsDps.usFlags   = _CP_EXCLUSIVE;
   rcCpsDps.usMode    = 0;
   rcCpsDps.usTimeout = 0;
   rcCpsDps.usIndexFirst = 2;             // DOUT are configuration element 2
   rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
	Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
	                         &rcCpsDps, NULL, 0, 
                            &parameter.hDoutBus);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hDoutBus");
		return Error;
	}
   
   // open a channel to DINs
	rcCpsDad.usDevice	    = DEVICE_DIN;
   rcCpsDad.usIndexFirst = 0;
   rcCpsDad.usIndexLast  = 7;
   rcCpsDad.usFlags		 = _CP_EXCLUSIVE;
   rcCpsDad.usReadMode	 = IO_MODE_DIRECT;
   rcCpsDad.usWriteMode  = 0;
   Error = max_open_channel (parameter.hXDAD4, sizeof(CPS_XDAD4), 
	                			  &rcCpsDad, NULL, 0, 
									  &parameter.hDin);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hDin");
		return Error;
	}

   // open bus out channel that transmits the DIN values on the bus
   rcCpsDps.usDevice  = DEVICE_BUS_IN;
   rcCpsDps.usChannel = parameter.usChannel;
   rcCpsDps.usFlags   = _CP_EXCLUSIVE;
   rcCpsDps.usMode    = 0;
   rcCpsDps.usTimeout = 0;
   rcCpsDps.usIndexFirst = 3;             // DIN are configuration element 3
   rcCpsDps.usIndexLast  = rcCpsDps.usIndexFirst;
	Error = max_open_channel(parameter.hXDPS2, sizeof(rcCpsDps), 
	                         &rcCpsDps, NULL, 0, 
                            &parameter.hDinBus);
	if (Error) 
   {
		ErrorHandler(Error, "max_open_channel hDinBus");
		return Error;
	}

	// write once to DEVICE_BUS_IN channels to start data exchange
   ulSize = sizeof(UCHAR);
   Error = max_write_channel_block (parameter.hDinBus, &ulSize, &ucData);
   ulSize = sizeof(ausData);
   Error = max_write_channel_block (parameter.hAinBus, &ulSize, ausData);
	if (Error) 
   {
		ErrorHandler(Error, "max_write_channel_block hDinBus");
		return Error;
	}

   return ERR_OK;
}

/****************************************************************************/
// Procedure to acquire data from bus and output it on the X-DAD-4 and
// to acquire data from the X-DAD-4 and output it on the bus.
MAX_ERROR CDad4slaveDlg::Operate()
{
   LONG     alAin[4];
   USHORT   ausAinBus[4];
   LONG     alAout[4];
   USHORT   ausAoutBus[4];
   ULONG    ulSize;
   USHORT   usDigVal, i;
   MAX_ERROR Error;
    
   // trigger the watchdog
   Error = max_trigger_channel(parameter.hWatchdog);
	if (m_usStartCommunication == 0)
	{
		if (Error) 
		{
			ErrorHandler(Error, "bus operation");
		}
		return 0;
	}

   // acquire analog inputs
   ulSize = sizeof(alAin);
   Error = max_read_channel_block (parameter.hAin, &ulSize, alAin);
   // the AIN values are encoded: 1 = 1mV
   for(i=0;i<4;i++)
      ausAinBus[i] = (USHORT)(alAin[i] / 1000);

   // display data 
   m_Ain0 = alAin[0];
   m_Ain1 = alAin[1];
   m_Ain2 = alAin[2];
   m_Ain3 = alAin[3];

   // write analog input values to profibus
   ulSize = sizeof(ausAinBus);
   Error = max_write_channel_block (parameter.hAinBus, &ulSize, ausAinBus);

   // acquire digital inputs
   Error = max_read_channel_ushort (parameter.hDin, &usDigVal);

   // display data 
   if (usDigVal & 0x01)
      m_Din0 = TRUE;
   else
      m_Din0 = FALSE;
   if (usDigVal & 0x02)
      m_Din1 = TRUE;
   else
      m_Din1 = FALSE;
   if (usDigVal & 0x04)
      m_Din2 = TRUE;
   else
      m_Din2 = FALSE;
   if (usDigVal & 0x08)
      m_Din3 = TRUE;
   else
      m_Din3 = FALSE;
   if (usDigVal & 0x10)
      m_Din4 = TRUE;
   else
      m_Din4 = FALSE;
   if (usDigVal & 0x20)
      m_Din5 = TRUE;
   else
      m_Din5 = FALSE;
   if (usDigVal & 0x40)
      m_Din6 = TRUE;
   else
      m_Din6 = FALSE;
   if (usDigVal & 0x80)
      m_Din7 = TRUE;
   else
      m_Din7 = FALSE;

   // write digital input values to profibus
   ulSize = sizeof(UCHAR);
   Error = max_write_channel_block (parameter.hDinBus, &ulSize, &usDigVal);

   // read values for AOUT from the profibus
   ulSize = sizeof(ausAoutBus);
   Error = max_read_channel_block (parameter.hAoutBus, &ulSize, ausAoutBus);

   // write AOUT values to X-DAD-4
   // the AOUT values are encoded: 1 = 1mV
   for(i=0;i<4;i++)
      alAout[i] = (LONG)ausAoutBus[i] * 1000;
   ulSize = sizeof(alAout);
   Error = max_write_channel_block (parameter.hAout, &ulSize, alAout);

   // display data 
   m_Aout0 = alAout[0];
   m_Aout1 = alAout[1];
   m_Aout2 = alAout[2];
   m_Aout3 = alAout[3];

   // read values for DOUT from the profibus
   ulSize = sizeof(UCHAR);
   usDigVal = 0;
   Error = max_read_channel_block (parameter.hDoutBus, &ulSize, &usDigVal);

   // display data 
   if (usDigVal & 0x01)
      m_Dout0 = TRUE;
   else
      m_Dout0 = FALSE;
   if (usDigVal & 0x02)
      m_Dout1 = TRUE;
   else
      m_Dout1 = FALSE;
   if (usDigVal & 0x04)
      m_Dout2 = TRUE;
   else
      m_Dout2 = FALSE;
   if (usDigVal & 0x08)
      m_Dout3 = TRUE;
   else
      m_Dout3 = FALSE;
   if (usDigVal & 0x10)
      m_Dout4 = TRUE;
   else
      m_Dout4 = FALSE;
   if (usDigVal & 0x20)
      m_Dout5 = TRUE;
   else
      m_Dout5 = FALSE;
   if (usDigVal & 0x40)
      m_Dout6 = TRUE;
   else
      m_Dout6 = FALSE;
   if (usDigVal & 0x80)
      m_Dout7 = TRUE;
   else
      m_Dout7 = FALSE;

   // write DOUT values to X-DAD-4
   Error = max_write_channel_ushort (parameter.hDout, usDigVal);
	if (Error) 
   {
		ErrorHandler(Error, "bus operation");
	}
   UpdateData (FALSE);
	return Error;
}

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg Nachrichten-Handler

BOOL CDad4slaveDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	SetIcon(m_hIcon, TRUE);			// Groes Symbol verwenden
	SetIcon(m_hIcon, FALSE);		// Kleines Symbol verwenden
	prcDlg = this;
	// default settings
   m_Board = 1;
   m_SlotDAD4 = 5;
   m_SlotDPS2 = 6;
   m_Station  = 3;      
	m_usStartCommunication = 0;
	m_Events.SetWindowText("");
   UpdateData(FALSE);
   return TRUE;  
}

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg OnTimer: Do the data exchange periodically
void CDad4slaveDlg::OnTimer(UINT nIDEvent) 
{
   Operate();
	CDialog::OnTimer(nIDEvent);
}

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg OnClose: stop timer
void CDad4slaveDlg::OnClose() 
{
   // remove timer object
   KillTimer(m_nTimer);
   // terminate maxw32 usage
   max_exit_lib();
	CDialog::OnClose();
}

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg::OnInfoState: INFO_DEVICE about the slave
// Usefull for diagnostics.
void CDad4slaveDlg::OnInfoState() 
{
	MAX_ERROR	Error;
   USHORT      usSize=4;
	UCHAR			Data[4];
   CString     sData, sNewLine, sState, sOldText;

	Error = max_channel_info(parameter.hBusCtrl, INFO_DEVICE, &usSize, Data);
	if (Error)
   {
      ErrorHandler (Error, "max_channel_info (DEVICE_INFO)");
   }
   else
   {
      sNewLine.Format("State of the Slave = %Xh\r\nDP-State=", *(USHORT*)Data);
      switch (Data[0] & 0x30)
      {
      case 0x00: sState = "Wait Prm";
         break;
      case 0x10: sState = "Wait Cfg";
         break;
      case 0x20: sState = "Data Exchange";
         break;
      default: sState = "Illegal DP state";
         break;
      }
      sNewLine = sNewLine + sState;
      if ((Data[0] & 1) == 0)
      {
         sState = ", Offline ";
         sNewLine = sNewLine + sState;
      }
      if (Data[0] & 8)
      {
         sState = ", Access Violation ";
         sNewLine = sNewLine + sState;
      }
      sState.Format("\r\nWD-State=%X, \r\nBaudrate=%X, ",  
                     (Data[0] & 0xC0) >> 6, Data[1] & 0x0F);
	   AfxMessageBox(sNewLine + sState);
   }
}

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg OnPaint (as originally generated by MFC assistant)
void CDad4slaveDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // Gertekontext fr Zeichnen
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDad4slaveDlg OnQueryDragIcon (as originally generated by MFC assistant)
HCURSOR CDad4slaveDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}




