/****************************************************************************/
/*                                                                          */
/* PC-Sample for buffered real time data acquisition with X-AD14-20         */
/*                                                                          */
/* The program uses the realtime program buf_daq.exe in the RT directory.   */
/*                                                                          */
/* Note: With this program maximum performance can not be reached!          */
/*       The main intention of this program is to demonstrate parallel      */
/*       processing on the MAX-PC and the Windows PC. If higher performance */
/*       is required both this program and the realtime program have to be  */
/*       optimized.                                                         */
/*                                                                          */
/* Initialization (done in main())                                          */
/* The PC program first initializes the board.                              */
/* Then it downloads the realtime program, initializes it by setting some   */
/* parameters and starts the realtime program by calling the tasks' start   */
/* function.                                                                */
/* The MDD for the analog input module is loaded on the CPU module.         */
/* The PC-program also creates a buffer on the MAX-PC that is used to buffer*/
/* the measured data values. 	                                              */
/*                                                                          */
/* Function of the realtime-program                                         */
/* The continous acquisition of the analog values is done totally by the    */
/* realtime program.                                                        */
/* The realtime program is called according to a timer of the CPU-module.   */
/* Each time the timer expires an interrupt is generated on the CPU-module. */
/* The Interrupt-Service-Routine (Procedure 0 of the task) then reads one   */
/* analog input channel and stores the value in the buffer.		             */
/* (The realtime program can also run as NI or TI task).                    */ 
/* If a the buffer is filled up to a certain level, a SRQ will be sent to   */
/* this PC-program.                                                         */
/*                                                                          */
/* SRQ-reaction (on the PC)                                                 */
/* If the PC receives the SRQ the library will call the SRQ-Service-function*/
/* (called 'ReadMeasuredData'). That function examines the received SRQ.    */
/* For SRQs that indicate that the number of values is available in the CPU */
/* modules buffer it reads out the buffer then. Other SRQs indicate errors  */
/* in the realtime system.                                                  */
/*                                                                          */
/* Copyright SORCUS Computer GmbH 2001 ... 2004                             */ 
/****************************************************************************/
/*
* History:
* 22.04.2004: Add a call of the function max_macro_interruptible to avoid interrupts at
*             the real time data acquisition by the PC.
* 21.03.2001: The communication mechanism between this program and the buf_dacq.exe
*             program running on the MAX-PC has changed: now the realtime task does
*             not stop measurement until the buffered data has been read by the PC.
*             Measurement continues after sending a SRQ. 
*             This results in a better sampling rate.
*/

#include "windows.h"
#include "..\..\rt\c\buf_dacq.h"
#include "stdio.h"
#include "conio.h"

#define SRQ_DATA_READY	0x8181	// SRQ-word used to signal 'data available'
#define SRQ_RT_ERROR		0x9292	// SRQ-word used to signal errors
#define FIRST_AIN			0			// first analog input to read (0..19)
#define LAST_AIN			4			// last analog input to read (0..19)
#define BUFFERSIZE		60000	// size of the data buffer 

USHORT	 usBoardNr = 1;			// see contol panel 'SORCUS boards'
USHORT	 usCPUSlot, usCPULayer;	// Location of the CPU module
USHORT	 usADSlot,  usADLayer;	// Location of the X-AD14-20 module
MAXMODHND hModul;
MAXMDDHND hMdd;
MAXBUFHND hBuffer;
USHORT	 MEASURE_TASK = 66;
USHORT	 usChannel;
long		 *plData;

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
	USHORT	 usMsgSize = 100;
	char		 sErrMsg[200];

	max_get_error_message(Error, &usMsgSize, sErrMsg);
	printf ("\n\rError %04Xh in %s (%s)", Error, sText, sErrMsg);
	max_call_proc(hModul, MEASURE_TASK, 4);
//	_getch();									// user has to confirm
	max_clear_error();
}

/****************************************************************************/
// Cleanup procedure
void ExitProgram(USHORT FreeMemory)
{
	max_exit_lib();
	if (FreeMemory)
		free (plData);
}

/****************************************************************************/
// SRQ service Procedure
MAX_CALLBACK ReadMeasuredData (MAXMODHND hSrqModul, ULONG ulMsg)
{
	ULONG			ulSize, i;
	MAX_ERROR	Error;
	double		fpValue;

	if (hModul != hSrqModul)				// SRQ from another module
		return;									// ignore it			
	if (ulMsg == SRQ_DATA_READY) { 
		// the realtime program signals that data is available
		// read out measured data values from the buffer on the CPU-module
		// (as many as possible)
		ulSize = BUFFERSIZE;
		Error = max_get_buffer(hBuffer, MAX_BUFFER_MOVE_MAX, &ulSize, plData);
		// the function returns in 'ulSize' how many bytes could be read
		if (Error) 
			ErrorHandler(Error, "max_get_buffer");
		else {
			// display the analog values (format: 1 = 1 Microvolt)
			for (i=0; i<ulSize/sizeof(long); i++) {
				fpValue = (double)plData[i] / 1000000;
				if (usChannel == FIRST_AIN)
					printf("\n\r");		// start new line when all channels are displayed
				printf("AIN-%02d=%.2fV ", usChannel, fpValue);
				usChannel++;
				if (usChannel > LAST_AIN)
					usChannel = FIRST_AIN;
			}
		}
	}
	else {
 		if (ulMsg == SRQ_RT_ERROR) {
			// the realtime program signals an error, code is stored in a parameter
			max_read_par_ushort(hModul, MEASURE_TASK, 0, &Error);
			ErrorHandler(Error, "RT-program");
		}
		else
			ErrorHandler((USHORT)ulMsg, "(SRQ-Message)");
	}
}

/****************************************************************************/
void main () 
{
	MAX_ERROR		Error;
	MAX_OS_INFO		rcOsInfo;
	MAX_VERSION		Version;
	DAQ_INIT_DATA	rcRTInitData;
	USHORT			usInsize, usOutsize;
	ULONG				ulDate, ulSize; 	
	char				szVersion[12], szDate[12];
	MAX_PGM_TYPE	rcPrgInfo;
	MAX_BUFFER_TYPE rcBufferPar;
	char				sFilename[80];

	usADSlot  = 1;								// where the X-AD14-20 is
	usADLayer = 0;

	printf ("Sample program for buffered analog data acquisition with X-AD14-20");
	Error = max_init_lib (0);
	// display some information about the system
	max_get_lib_version (&Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);
	printf ("\n\rLib Version: %s (%s)", szVersion, szDate);
	Error = max_get_drv_version (&Version, &ulDate);
	if (Error) {
		printf ("Error %X", Error);
		max_clear_error();
	}
	else {
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf ("\n\rDriver Version: %s (%s)", szVersion, szDate);
	}

	// reset the board
	Error = max_reset_board (usBoardNr, 20);
	if (Error) {
		ErrorHandler(Error, "max_reset_board");
		ExitProgram(0);
		return;
	}
	
	// connect to the CPU module
	Error = max_connect_cpu (usBoardNr, MAX_AUTO_SLOT, 0, &rcOsInfo, &hModul);
	if (Error) {
		ErrorHandler(Error, "max_connect_cpu");
		ExitProgram(0);
		return;
	}
	
	// load or activate an operating system
	if (rcOsInfo.usType == MAX_MINI_OSX) {
		Error = max_load_osx (hModul, "ROM");
		if (Error) {
			ErrorHandler(Error, "max_load_osx");
			ExitProgram(0);
			return;
		}
		Error = max_get_os_version(hModul, &rcOsInfo);
		if (Error) 
			ErrorHandler(Error, "max_get_os_version");
	}
	Error = max_get_slot(hModul, &usCPUSlot, &usCPULayer);
	max_get_version_string(rcOsInfo.Version, rcOsInfo.ulDate, szVersion, szDate);
	printf ("\n\rConnected to CPU on slot %X, Layer %X: OS Type=%X, Version=%s (%s)",
			   usCPUSlot, usCPULayer, rcOsInfo.usType, szVersion, szDate);

	// If the communication with the PC-host is interruptible on the X-MAX-1
	// the interrupt-driven on-board data aquisition will not be influenced by 
	// the communication with the host PC. (recommended setting)
    Error = max_macro_interruptible(hModul, 1);
	if (Error) 
		ErrorHandler(Error, "max_macro_interruptible");

	// install SRQ service procedure
	Error = max_set_service(hModul, ReadMeasuredData);
	if (Error) {
		ErrorHandler(Error, "max_set_service");
		ExitProgram(0);
		return;
	}

	// download realtime program
	strcpy (sFilename, "..\\..\\rt\\c\\buf_dacq.exe");
	rcPrgInfo.szName	= sFilename;
	rcPrgInfo.usFlags = MAX_TASKNO_BY_USER + 
							  MAX_CALL_AUTO_INIT +
							  MAX_TASKTYPE_BY_INSTALL;
	// type of task can be selected here - change usTaskType and usIrq
	rcPrgInfo.usTaskType = MAX_II_TASK;
	rcPrgInfo.usIrq		= MAX1_IRQ_TIMERA;
	// usIrq=0					 for usTaskType = MAX_NI_TASK
	// usIrq=MAX1_IRQ_TIMERA for usTaskType = MAX_II_TASK
	// usIrq=MAX1_IRQ_TIMERC for usTaskType = MAX_TI_TASK
	// consider also setting of task parameter 6 (timer rate)
	rcPrgInfo.usNo		= DAQ_PROG_NO;
	rcPrgInfo.usLevel = 3;
	rcPrgInfo.usUsage = 0;
	rcPrgInfo.usTask  = MEASURE_TASK;
	rcPrgInfo.ulDataSize = 0;
	Error = max_transfer_and_install(hModul, &rcPrgInfo, &MEASURE_TASK);
	if (Error) {
		ErrorHandler(Error, "max_transfer_and_install");
		ExitProgram(0);
		return;
	}
	else { 
		max_get_prog_version(hModul, MEASURE_TASK, &Version, &ulDate);
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf ("\n\rProgram buf_daq.exe downloaded onto CPU-module, ");
		printf ("Version %s", szVersion);
	}
 
	// load MDD for X-AD14-20
 	Error = max_load_mdd (hModul, usADSlot, usADLayer, 0, 
		                   MDD_X_AD14_20, NULL, &hMdd);
	if (Error) {
		ErrorHandler(Error, "max_load_mdd");
		ExitProgram(0);
		return;
	}
	max_get_mdd_version(hMdd, &Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);
	printf ("\n\rMDD for X-AD14-20 downloaded onto CPU-module, ");
	printf ("Version %s", szVersion);

	// init some parameters of the RT-program
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 4, usADSlot);  // slot of the AD module
	// select frequency: for TI-task set parameter value in ms, for II-tasks in timer-tics:
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 6, 120);       // timer rate
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 8, FIRST_AIN); // first channel
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 10, LAST_AIN); // last channel
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 14, SRQ_DATA_READY); // SRQ word
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 16, SRQ_RT_ERROR);   // SRQ word
	if (Error) {
		ErrorHandler(Error, "max_write_par_ushort");
		ExitProgram(0);
		return;
	}
	
	// create a fifo buffer in the RAM of the CPU module
	rcBufferPar.usStrategy	= MAX_ALLOC_DOWN_ABS;
	rcBufferPar.usAlignment = 2;
	rcBufferPar.usTask		= MEASURE_TASK;           
	rcBufferPar.usUsage		= 0;           
	ulSize = BUFFERSIZE;
	Error = max_create_buffer(hModul, &rcBufferPar, &ulSize, &hBuffer);
	// get the buffer number that can be transfered to the realtime program
	Error = max_get_buffer_id (hBuffer, &rcRTInitData.ulBufferID);
	if (Error) {
		ErrorHandler(Error, "max_create_buffer");
		ExitProgram(0);
		return;
	}

	// memory allocation for measured data
	plData = (long*)malloc(BUFFERSIZE);
	if (plData == NULL) {
		ErrorHandler(ERR_MEMORY_ALLOCATION, "malloc");
		ExitProgram(0);
		return;
	}

	// call measure init procedure of the realtime program
	usInsize  = sizeof(DAQ_INIT_DATA);
	usOutsize = 0;
	rcRTInitData.usADSlot		= usADSlot;
   rcRTInitData.usInputMode   = DEVICE_AIN_SE;
   rcRTInitData.usInputRange  = RANGE_BIP_5V;
   rcRTInitData.usSettleTime	= 800;
   rcRTInitData.usFirstChannel= FIRST_AIN;
   rcRTInitData.usLastChannel = LAST_AIN;
	Error = max_call_func(hModul, MEASURE_TASK, 2, &usInsize, &rcRTInitData,
   		                &usOutsize, &ulSize);
	if (Error) {
		ErrorHandler(Error, "max_call_func");
		ExitProgram(1);
		return;
	}

	usChannel = FIRST_AIN;
	printf ("\n\rPress any key to start measurement");
	_getch();

	// start RT-measuring
	Error = max_call_proc(hModul, MEASURE_TASK, 3);
	if (Error) {
		ErrorHandler(Error, "max_call_proc");
		ExitProgram(1);
		return;
	}
	
	// now all activity is done on the X-MAX-1 and in the ReadMeasuredData procedure
	while (!_kbhit());

	Error = max_read_par_ulong(hModul, MEASURE_TASK, 120, &ulDate);
	if (Error == ERR_OK)
		printf("\n\r%d analog values acquired", ulDate);
	
	Error = max_call_proc(hModul, MEASURE_TASK, 4);	// stop realtime program
	ExitProgram(1);
	return;
}
