{****************************************************************************}
{*                                                                          *}
{* PC-Sample for buffered real time data acquisition with X-AD14-20         *}
{*                                                                          *}
{* The program uses the realtime program buf_daq.exe in the RT directory.   *}
{*                                                                          *}
{* Note: With this program maximum performance can not be reached!          *}
{*       The main intention of this program is to demonstrate parallel      *}
{*       processing on the MAX-PC and the Windows PC. If higher performance *}
{*       is required both this program and the realtime program have to be  *}
{*       optimized.                                                         *}
{*                                                                          *}
{* Initialization (done in main())                                          *}
{* The PC program first initializes the board.                              *}
{* Then it downloads the realtime program, initializes it by setting some   *}
{* parameters and starts the realtime program by calling the tasks' start   *}
{* function.                                                                *}
{* The MDD for the analog input module is loaded on the CPU module.         *}
{* The PC-program also creates a buffer on the MAX-PC that is used to buffer*}
{* the measured data values. 	                                            *}
{*                                                                          *}
{* Function of the realtime-program                                         *}
{* The continous acquisition of the analog values is done totally by the    *}
{* realtime program.                                                        *}
{* The realtime program is called according to a timer of the CPU-module.   *}
{* Each time the timer expires an interrupt is generated on the CPU-module. *}
{* The Interrupt-Service-Routine (Procedure 0 of the task) then reads one   *}
{* analog input channel and stores the value in the buffer.		    *}
{* If a the buffer is filled up to a certain level, a SRQ will be sent to   *}
{* this PC-program.                                                         *}
{*                                                                          *}
{* SRQ-reaction (on the PC)                                                 *}
{* If the PC receives the SRQ the library will call the SRQ-Service-function*}
{* (called 'ReadMeasuredData'). That function examines the received SRQ.    *}
{* For SRQs that indicate that the number of values is available in the CPU *}
{* modules buffer it reads out the buffer then. Other SRQs indicate errors  *}
{* in the realtime system.                                                  *}
{*                                                                          *}
{* Copyright SORCUS Computer GmbH 2001	                                    *}
{****************************************************************************}
{*
* History:
* 22.04.2004: Add a call of the function max_macro_interruptible to avoid interrupts at
*             the real time data acquisition by the PC.
* 21.03.2001: The communication mechanism between this program and the buf_dacq.exe
*             program running on the MAX-PC has changed: now the realtime task does
*             not stop measurement until the buffered data has been read by the PC.
*             Measurement continues after sending a SRQ.
*             This results in a better sampling rate.
*}
{****************************************************************************}

unit dacq;

interface

{ the program uses the following SORCUS units
  Maxlib   - all functions and procedure declarations
  X_error  - defined error message constants
  maxtype  - all type definitions used in the library
  maxdefs  - various constant definitions
  maxmdd   - needed for MDDs: It defines CPS-types and constants
}
uses
  Windows, Maxlib, X_error, maxtype, maxdefs, maxmdd,
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls;

type TForm1 = class(TForm)
   InitButton: TButton;
   Info: TMemo;
   MeasureCtrlButton: TButton;
   Data: TMemo;
   procedure InitButtonClick(Sender: TObject);
   procedure MeasureCtrlButtonClick(Sender: TObject);
    procedure CloseProgram(Sender: TObject; var Action: TCloseAction);
   private
   { Private-Deklarationen }
   public
   { Public-Deklarationen }
end;

{ record type used to transfer a set of configuration-data to the Realtime program }
type DAQ_INIT_DATA = record
   usADSlot      : WORD;
   usInputMode   : WORD;
   usInputRange  : WORD;
   usSettleTime  : WORD;
   usFirstChannel: WORD;
   usLastChannel : WORD;
   ulBufferID    : LONGWORD;
end;

Type DATAPTR = ^LONGINT;

{ global variables }
var
  Form1                 : TForm1;
  usBoardNr    	        : WORD = 1;
  usCPUSlot, usCPULayer : WORD;
  usADSlot, usADLayer   : WORD;
  hModul                : MAXMODHND;
  hMdd                  : MAXMDDHND;
  hBuffer               : MAXBUFHND;
  MEASURE_TASK          : WORD = 66;
  usChannel             : WORD;
  plStart               : DATAPTR;
  MeasureStatus         : WORD;

const
  SRQ_DATA_READY : WORD = $8181;
  SRQ_RT_ERROR	 : WORD = $9292;
  FIRST_AIN      : WORD = 0;
  LAST_AIN	 : WORD = 4;
  BUFFERSIZE	 : LONGWORD = 60000;	{ size of the data buffer }
  { Os that should be activated }
  OsX            : Array[0..79] of char = 'ROM';

implementation

{$R *.DFM}

{******************************************************************************}
{** Subroutine for error display and error reset                             **}
{******************************************************************************}
procedure ErrorMessage (Error: MAX_ERROR; const location : PCHAR);
var
   usMsgSize : WORD;
   text      : string;
   szErrMsg  : Array [0..200] of char;
begin
   usMsgSize := 200;
   max_get_error_message (Error, usMsgSize, @szErrMsg[0]);
   text := 'Error ' + IntToHex(Error, 4) + ' in ' + string(location) + ' (' +  string(szErrMsg) + ')';
   MessageDlg(text, mtError, [mbYes], 0);
   max_clear_error();
end;

{******************************************************************************}
{** Callback-Subroutine for SRQs                                             **}
{******************************************************************************}
PROCEDURE ReadMeasuredData (hSrqModul : MAXMODHND; ulMsg : Longword); stdcall;
var
   ulSize, i     : LONGWORD;
   Error         : MAX_ERROR;
   plReadData    : DATAPTR;
   Voltage       : double;
   line_i        : string;
begin
   if (hModul <> hSrqModul) then	{ SRQ from another module - ignore it }
      exit;
   if (ulMsg = SRQ_DATA_READY) then begin
      { the realtime program signals that data is available
        read out measured data values from the buffer on the CPU-module
        (as many as possible) }
      ulSize := BUFFERSIZE;
      plReadData := plStart;
      Error  := max_get_buffer(hBuffer, MAX_BUFFER_MOVE_MAX, ulSize, plReadData^);
      { the function returns in 'ulSize' how many bytes could be read }
      if (Error <> ERR_OK) then
         ErrorMessage(Error, 'max_get_buffer')
      else begin
	{ display the analog values (format: 1 = 1 Microvolt) }
        plReadData := plStart;
        line_i := '';
        ulSize := ulSize div sizeof(longword);
        for i:=0 to ulSize do begin
           Voltage := plReadData^ / 1000000;
           line_i  := line_i + 'AIN-' + IntToStr(usChannel) + ' = ' +
                      FloatToStrF(Voltage, ffFixed, 7, 3) + 'V  ';
	   if (usChannel = LAST_AIN) then begin
              Form1.Data.Lines.Add(line_i);
              line_i := '';
              usChannel := FIRST_AIN;
           end
           else
              Inc(usChannel);
           Inc(plReadData);
         end;
      end;
   end
   else begin
       if (ulMsg = SRQ_RT_ERROR) then begin
          { the realtime program signals an error }
          max_read_par_ushort(hModul, MEASURE_TASK, 0, Error);
          ErrorMessage (Error, 'Error in RT-program');
       end
       else
	  ErrorMessage(ulMsg, '(SRQ-Message)');
   end;
end;

{******************************************************************************}
{** Initialization of the board and the modules                              **}
{******************************************************************************}
procedure TForm1.InitButtonClick(Sender: TObject);
var
   Error               : MAX_ERROR;
   rcOsInfo            : MAX_OS_INFO;
   Version             : MAX_VERSION;
   rcRTInitData        : DAQ_INIT_DATA;
   usInsize, usOutsize : WORD;
   ulDate, ulSize      : LONGWORD;
   szVersion, szDate   : ARRAY[0..11] of char;
   rcPrgInfo           : MAX_PGM_TYPE;
   rcBufferPar         : MAX_BUFFER_TYPE;
   BoardInfo           : MAX_BOARD_TYPE;
   ModuleInfo          : MAX_MODULINFO_TYPE;
   sFilename           : ARRAY[0..79] of char;

begin
   { location of the X-AD14-20 }
   usADSlot  := 1;
   usADLayer := 0;

   { library initialization }
   Error := max_init_lib (0);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_init_lib');

   { display version info }
   max_get_lib_version (Version, ulDate);
   max_get_version_string (Version, ulDate, @szVersion[0], @szDate[0]);
   Info.Lines.Add('Lib Version: ' + string(szVersion) + ' (' + string(szDate) + ')' );
   Error := max_get_drv_version (Version, ulDate);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_drv_version')
   else begin
      max_get_version_string(Version, ulDate, @szVersion[0], @szDate[0]);
      Info.Lines.Add('Drv Version: ' + szVersion + ' (' + szDate + ')');
   end;

   { reset MAX6pci board and connect program to the X-MAX-1 }
   Error := max_reset_board (usBoardNr, 10);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_reset_board');
      exit;
   end;

   { connect program to the X-MAX-1 and get a handle for the module
     if only one CPU module is plugged in MAX_AUTO_SLOT will cause the
     max_connect_cpu function to find that CPU module automatically }
   Error := max_connect_cpu (usBoardNr, MAX_AUTO_SLOT, 0, rcOsInfo, hModul);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_connect_cpu');
      exit;
   end;

   { download / activate an operating system onto the X-MAX-1 }
   if (rcOsInfo.usType = MAX_MINI_OSX) then begin
      { MINI_OSX supports only a subset of the library functions - so use a full OsX }
      Error := max_load_osx (hModul, OsX);
      if (Error <> ERR_OK) then begin
         ErrorMessage (Error, 'max_load_osx');
         exit;
      end;
      Error := max_get_os_version(hModul, rcOsInfo);
      if (Error <> ERR_OK) then begin
         ErrorMessage (Error, 'max_get_os_version');
         exit;
      end;
   end;
   Error := max_get_slot(hModul, usCPUSlot, usCPULayer);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_get_slot');
      exit;
   end;
   Info.Lines.Add('Connected to CPU on Slot ' + IntToHex(usCPUSlot, 2)
                   + ', Layer ' + IntToHex(usCPULayer, 2));
   max_get_version_string (rcOsInfo.Version, rcOsInfo.ulDate, @szVersion[0], @szDate[0]);
   Info.Lines.Add('OsX Version: ' + string(szVersion) + ' (' + string(szDate) + ')' );

   { If the communication with the PC-host is interruptible on the X-MAX-1
     the interrupt-driven on-board data aquisition will not be influenced by
     the communication with the host PC. (recommended setting) }
   Error := max_macro_interruptible(hModul, 1);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_macro_interruptible');
      exit;
   end;

   { display info about the MAX6pci board and the X-MAX-1 }
   Error := max_get_board_info(usBoardNr, BoardInfo);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_board_info');
   Info.Lines.Add('Board ' +  IntToHex(usBoardNr, 2) +
                  ' Type ' +  IntToHex(BoardInfo.usType, 4) +
                  ' Version ' + IntToHex(BoardInfo.usVersion, 2) +
                  ' Serial No. ' + IntToStr(BoardInfo.ulSerNr));
   Error := max_get_module_info(usBoardNr, usCPUSlot, usCPULayer, ModuleInfo);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_module_info');
   Info.Lines.Add('CPU-Module on slot ' +  IntToHex(usCPUSlot, 2) +
                  ' Type ' + IntToHex(ModuleInfo.usType, 2) +
                  ' Version ' + IntToHex(ModuleInfo.usVersion, 2) +
                  ' Variant ' + IntToHex(ModuleInfo.usVariant, 2) +
                  ' Category ' + IntToHex(ModuleInfo.usVariant, 2));

   { install SRQ service procedure }
   Error := max_set_service(hModul, @ReadMeasuredData);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_set_service');

   { download realtime program }
   sFilename := '..\..\rt\pas\buf_dacq.exe';
   rcPrgInfo.szName	:= sFilename;
   rcPrgInfo.usLevel    := 3;
   rcPrgInfo.usNo	:= $9005;
   rcPrgInfo.usFlags    := MAX_TASKNO_BY_INSTALL + MAX_CALL_AUTO_INIT + MAX_TASKTYPE_BY_INSTALL;
   { usIrq := MAX1_IRQ_TIMERA for usTaskType := MAX_II_TASK }
   { usIrq := MAX1_IRQ_TIMERC for usTaskType := MAX_TI_TASK }
   { usIrq := 0 for usTaskType := MAX_NI_TASK }
   { parameter 6 of MEASURE_TASK depends on this setting also }
   rcPrgInfo.usIrq	:= MAX1_IRQ_TIMERA;
   rcPrgInfo.usTaskType := MAX_II_TASK;
   rcPrgInfo.usUsage    := 0;
   rcPrgInfo.usTask     := MEASURE_TASK;
   rcPrgInfo.ulDataSize := 0;
   Error := max_transfer_and_install(hModul, rcPrgInfo, MEASURE_TASK);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_transfer_and_install')
   else begin
      max_get_prog_version(hModul, MEASURE_TASK, Version, ulDate);
      max_get_version_string (Version, ulDate, @szVersion[0], @szDate[0]);
      Info.Lines.Add('Program ' + IntToStr(rcPrgInfo.usNo) + ' installed as task ' +
                      IntToStr(MEASURE_TASK) + ', Version: ' + string(szVersion));
   end;

   { search path for Module-device-driver can be set optionally }
{   max_set_mdd_path ('..\..\..\..\mdd'); }
   { load MDD for X-AD14-20 }
   Error := max_load_mdd (hModul, usADSlot, usADLayer, 0, MDD_X_AD14_20, rcPrgInfo, hMdd);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_load_mdd')
   else begin
      max_get_mdd_version(hMdd, Version, ulDate);
      max_get_version_string(Version, ulDate, szVersion, szDate);
      Info.Lines.Add('MDD installed, Version: ' + string(szVersion));
      Error := max_get_module_info(usBoardNr, usADSlot, usADLayer, ModuleInfo);
      if (Error <> ERR_OK) then
         ErrorMessage (Error, 'max_get_module_info');
      Info.Lines.Add('X-AD14-20-Module on slot ' +  IntToHex(usADSlot, 2) +
                     ' Type ' + IntToHex(ModuleInfo.usType, 2) +
                     ' Version ' + IntToHex(ModuleInfo.usVersion, 2) +
                     ' Variant ' + IntToHex(ModuleInfo.usVariant, 2) +
                     ' Category ' + IntToHex(ModuleInfo.usVariant, 2));

      { init some parameters of the RT-program }
      Error := max_write_par_ushort (hModul, MEASURE_TASK, 4, usADSlot);  // slot of the AD module
      Error := max_write_par_ushort (hModul, MEASURE_TASK, 6, 180);       // timer rate (12=10us, 18=15us)
      Error := max_write_par_ushort (hModul, MEASURE_TASK, 8, FIRST_AIN); // first channel
      Error := max_write_par_ushort (hModul, MEASURE_TASK, 10, LAST_AIN); // last channel
      Error := max_write_par_ushort (hModul, MEASURE_TASK, 14, SRQ_DATA_READY); // SRQ word
      Error := max_write_par_ushort (hModul, MEASURE_TASK, 16, SRQ_RT_ERROR);   // SRQ word

      { create a fifo buffer in the RAM of the CPU module }
      rcBufferPar.usStrategy  := MAX_ALLOC_DOWN_ABS;
      rcBufferPar.usAlignment := 2;
      rcBufferPar.usTask      := MEASURE_TASK;
      rcBufferPar.usUsage     := 0;
      ulSize := BUFFERSIZE;
      Error := max_create_buffer(hModul, rcBufferPar, ulSize, hBuffer);
      { get the buffer number that can be transfered to the realtime program }
      Error := max_get_buffer_id (hBuffer, rcRTInitData.ulBufferID);
      if (Error <> ERR_OK) then
         ErrorMessage (Error, 'max_create_buffer');

      // memory allocation for measured data
      try
         GetMem(plStart, BUFFERSIZE);
      except
         ErrorMessage(ERR_MEMORY_ALLOCATION, 'GetMem');
      end;

      { call measure init procedure of the realtime program }
      usInsize  := sizeof(DAQ_INIT_DATA);
      usOutsize := 0;
      rcRTInitData.usADSlot      := usADSlot;
      rcRTInitData.usInputMode   := DEVICE_AIN_SE;
      rcRTInitData.usInputRange  := RANGE_BIP_5V;
      rcRTInitData.usSettleTime  := 1000;
      rcRTInitData.usFirstChannel:= FIRST_AIN;
      rcRTInitData.usLastChannel := LAST_AIN;
      Error := max_call_func(hModul, MEASURE_TASK, 2, usInsize, rcRTInitData,
                             usOutsize, ulSize);
      if (Error <> ERR_OK) then
         ErrorMessage (Error, 'max_call_func');
      MeasureStatus := 0;
   end;
   InitButton.Enabled := FALSE;
end;

{******************************************************************************}
{** Start / Stop measurement                                                 **}
{******************************************************************************}
procedure TForm1.MeasureCtrlButtonClick(Sender: TObject);
var
   Error : MAX_ERROR;
begin
   if (MeasureStatus = 0) then begin
      { start RT-measuring }
      Error := max_call_proc(hModul, MEASURE_TASK, 3);
      MeasureCtrlButton.Caption := 'Stop Measurement';
      MeasureStatus := 1;
      usChannel := FIRST_AIN;
   end
   else begin
      { stop RT-measuring }
      Error := max_call_proc(hModul, MEASURE_TASK, 4);
      MeasureCtrlButton.Caption := 'Start Measurement';
      MeasureStatus := 0;
   end;
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_call_proc');
end;

{******************************************************************************}
{** Exit program                                                             **}
{******************************************************************************}
procedure TForm1.CloseProgram(Sender: TObject; var Action: TCloseAction);
begin
   max_exit_lib;  
   FreeMem(plStart);
end;

end.
