/****************************************************************************/
// Sample Real Time program
// The program measures the analog inputs of the X-AD14-20
// and write the values into a buffer.
// If the buffer is filled with a certain number of values a srq
// is sent to the PC, which can then read out the values from the buffer.
//
// Program Number 0x9005
//
// (C)2000-2001 SORCUS Computer GmbH
/****************************************************************************/
// History
// version 1.D: Increase of performance by using max_set_error_check_level
//              max_cpu_speed_control

#pragma option -r-

// _RTLIB_ must be defined in every real time program
#define _RTLIB_

#include "buf_dacq.h"
#include "dos.h"
#include "stdlib.h"

#define VERSION         '1'
#define REVISION        'D'

#define ERR_IN_AUTO_INIT 0x1AE0

// PDT (Program Descriptor Table) data structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
   void  MAXEXPORT (*measure_init) (USHORT *pusInsize, DAQ_INIT_DATA *pInit,
                                    USHORT *pusOutsize, USHORT *pDataOut);
   void* MAXEXPORT start;
   void* MAXEXPORT stop;
} rcPDT;

// parameter range of the task
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;
// 0
   MAX_ERROR    Error;              // global error variable
   USHORT       usProgState;        // State of the program
   USHORT       usXAD14_slot;       // location of the X-AD14-20
   USHORT       usTimerRate;        // sampling rate
   USHORT       usFirstChannel;     // first analog input to be measured
// 10
   USHORT       usLastChannel;      // last analog input to be measured
   USHORT       usChannel;          // index of actual channel that is measured
   USHORT       usBufferFullSRQ;    // SRQ word that is used to indicate a filled buffer
   USHORT       usErrorSRQ;         // SRQ word that is used to indicate an error
   USHORT       usHostNo;           // number of target for SRQs
// 20
   ULONG        ulBufferSize;       // size of the buffer
   USHORT       usReleaseSRQ;       // flag that is set while buffer is filled
                                    // less than 1/3 and reset while buffer is filled
                                    // more than 1/3
   USHORT       usDummy;            // reserved
   MAXBUFHND    hBuffer;            // Handle for data buffer
   MAXMODHND    hMyself;            // handle for the CPU
   MAXMODHND    hHost;              // handle for the PC
// 40
   MAXCHLHND    hAin[20];           // handles for the analog input channels
// 120
   ULONG        ulSampleCount;      // number of samples
   MAXMDDHND    hMdd;               // handle for the X-AD14-20 MDD
   MAX_OS_INFO  rcOsX;
   ULONG        ulValid;
   USHORT       usErrCount;
} parameter;

// states of the program
enum
{
   PROG_NOT_READY,
   PROG_INITIALIZED,
   PROG_READY,
   PROG_STARTED,
};

// Prototypes
void MAXEXPORT MainProc (void);
void MAXEXPORT AutoInit (void);
void MAXEXPORT MeasureInit(USHORT *pusInsize, DAQ_INIT_DATA *prcInit,
                           USHORT *pusOutsize, USHORT *pusDataOut);
void MAXEXPORT Start (void);
void MAXEXPORT Stop (void);
void ErrorHandler (USHORT usOffset);

/****************************************************************************/
// in case of error send srq to PC
void ErrorHandler(USHORT usOffset)
{
   parameter.usErrCount++;
   max_send_srq (parameter.hHost, 0, parameter.usErrorSRQ + usOffset);
}

/****************************************************************************/
// init task parameter
void MAXEXPORT AutoInit(void)
{
   max_entry_proc();
   // init parameters (some may be overwritten by the PC-program)
   parameter.usProgState      = PROG_NOT_READY;
   parameter.usTimerRate      = 12; // = 10us
   parameter.usXAD14_slot     = 1;
   parameter.usFirstChannel   = 0;
   parameter.usLastChannel    = 19;
   parameter.ulBufferSize     = 0;
   parameter.usBufferFullSRQ  = 0x8064;
   parameter.usErrorSRQ       = 0x8060;
   parameter.usHostNo         = 1;     // PC via PCI bridge
   parameter.ulSampleCount    = 0;
   parameter.ulValid          = 0;
   parameter.usErrCount       = 0;
   parameter.Error = max_init_lib(0);
   if (parameter.Error == ERR_OK) {
      // get a handle for the X-MAX-1
      parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,
                                         &parameter.rcOsX, &parameter.hMyself);
      if (parameter.Error == ERR_OK)
         parameter.usProgState = PROG_INITIALIZED;
   }
   max_exit_proc();
};

#pragma argsused
/****************************************************************************/
// open channels
void MAXEXPORT MeasureInit(USHORT *pusInsize, DAQ_INIT_DATA *prcInit,
                           USHORT *pusOutsize, USHORT *pusDataOut)
{
   MAX_ERROR   Error;
   CPS_XAD1420 CPS_Ain;
   USHORT      i;
   ULONG       v;

   max_entry_func();
   Error = ERR_OK;
   if (*pusInsize != sizeof(DAQ_INIT_DATA))
      Error = ERR_WRONG_NUMBER_OF_INPUTS;
   else
      if (parameter.usProgState != PROG_INITIALIZED)
         Error = ERR_IN_AUTO_INIT;
   if (Error == ERR_OK) {
      parameter.usXAD14_slot   = prcInit->usADSlot;
      parameter.usFirstChannel = prcInit->usFirstChannel;
      parameter.usLastChannel  = prcInit->usLastChannel;

      // get a handle for the host PC (to send SRQs)
      Error = max_connect_cpu (parameter.usHostNo, 0, 0, NULL, &parameter.hHost);
      if (Error == ERR_OK) {
         // get a handle for the MDD
         Error = max_load_mdd (parameter.hMyself, parameter.usXAD14_slot, 0, 0,
							          MDD_X_AD14_20, NULL, &parameter.hMdd);
         if (Error == ERR_OK) {
          Error = max_get_buffer_handle (parameter.hMyself, prcInit->ulBufferID,
											        &parameter.hBuffer);
            if (Error == ERR_OK)
               // get buffersize
               Error = max_get_buffer_status(parameter.hBuffer,
   						                        &v, &parameter.ulBufferSize);
            if (Error == ERR_OK) {
               CPS_Ain.usDevice     = prcInit->usInputMode;
               CPS_Ain.usFlags      = _CP_UNCORRECTED;
               CPS_Ain.usRange      = prcInit->usInputRange;
               CPS_Ain.usReadMode   = IO_MODE_DIRECT;
               CPS_Ain.usSettleTime = prcInit->usSettleTime;
               // open channels
               i=0;
               for (i=parameter.usFirstChannel; i<=parameter.usLastChannel; i++) {
                  CPS_Ain.usIndexFirst = i;
                  CPS_Ain.usIndexLast  = i;
                  Error = max_open_channel (parameter.hMdd, sizeof (CPS_XAD1420),
                                            &CPS_Ain, NULL, 0, &parameter.hAin[i]);
                  if (Error != ERR_OK)
                     break;
               }
            }
         }
         // increase performance by not checking for invalid handles
         // and wrong data type check in the library
         max_set_error_check_level(0);
         // switch CPU into the fastest mode
         Error = max_cpu_speed_control(parameter.hMyself, MAX1_CPU_HYPER_SPEED,
                                       MAX1_SPEED_100MHZ);
         if (Error == ERR_OK)
            parameter.usProgState = PROG_READY;
      }
   }
   max_exit_func(Error);
}

/****************************************************************************/
// start the task
void MAXEXPORT Start(void)
{
   USHORT usTaskType;
   MAX_TI_TYPE rcTIParams;

   max_entry_proc();
   if (parameter.usProgState == PROG_READY) {
      max_clear_buffer(parameter.hBuffer);  // empty buffer
      parameter.usProgState  = PROG_STARTED;
      parameter.usChannel    = parameter.usFirstChannel;
      parameter.usReleaseSRQ = 1;

      // activate the task
      // installation determines  the task type and the interrupt source
      // actual task type can be read from the TDT
      usTaskType = parameter.rcTDT.usFlags & 0x0003;
      switch (usTaskType)
      {
         case MAX_NI_TASK:
            parameter.Error = max_wakeup_ni_task(parameter.hMyself,
                                                 parameter.rcTDT.usTaskNo, 1);
            break;
         case MAX_II_TASK:
            // init and start Timer
            parameter.Error = max_set_timer(parameter.hMyself,
                                            MAX1_TIMER_A, MAX1_TIMER_MODE_INT,
                                            parameter.usTimerRate);
            parameter.Error = max_wakeup_ii_task(parameter.hMyself,
                                                 parameter.rcTDT.usTaskNo);
            break;
         case MAX_TI_TASK:
            rcTIParams.usPriority = 0;
            rcTIParams.ulHoldOff  = 0;
            rcTIParams.ulIntervall= (ULONG)parameter.usTimerRate;
            rcTIParams.ulCalls    = 0;
            parameter.Error = max_wakeup_ti_task(parameter.hMyself,
                                                 parameter.rcTDT.usTaskNo,
                                                 &rcTIParams);
            break;
      }
   }
   max_exit_proc();
}

/****************************************************************************/
// main procedure of the task (called automatically by OsX after starting the task)
void MAXEXPORT MainProc(void)
{
   long  Data;
	ULONG	ulSize, ulFree;

   max_entry_proc();
   if (parameter.usProgState == PROG_STARTED) {
      // read one input channel
      parameter.Error = max_read_channel_long(parameter.hAin[parameter.usChannel], &Data);
      if (parameter.Error != ERR_OK)
         ErrorHandler(0);
      else
      {
		   // write value into buffer
 		   ulSize = sizeof(Data);
         parameter.Error = max_set_buffer(parameter.hBuffer, MAX_BUFFER_MOVE_ABS,
   		                                 &ulSize, &Data);
         if (parameter.Error != ERR_OK)
            ErrorHandler(1);
         else {
            parameter.Error = max_get_buffer_status(parameter.hBuffer,
   											                &parameter.ulValid,
                                                    &ulFree);
            if (parameter.Error != ERR_OK)
               ErrorHandler(2);
            else {
               // check if SRQ has to be sent
               if (parameter.usReleaseSRQ) {
                  // no SRQ has been sent since buffer is filled more than 1/3
                  if (parameter.ulValid >= parameter.ulBufferSize / 3) {
                     parameter.usReleaseSRQ = 0;
                     // if buffer is filled more than 1/3 send SRQ to the host
			            parameter.Error = max_send_srq(parameter.hHost, 0,
                                                    parameter.usBufferFullSRQ);
                     if (parameter.Error != ERR_OK)
                        ErrorHandler(3);
                   }
               }
               else {
                  // SRQ has been sent already since buffer is filled more than 1/3
                  if (parameter.ulValid < parameter.ulBufferSize / 3)
                     // enable SRQs again if buffer state falls below 1/3
                     parameter.usReleaseSRQ = 1;
               }
            }
         }
         if (parameter.usChannel == parameter.usLastChannel)
            parameter.usChannel = parameter.usFirstChannel;
         else
            parameter.usChannel++;
         parameter.ulSampleCount++;
		}
   }
   max_exit_proc();
}

/****************************************************************************/
// stop the task
void MAXEXPORT Stop(void)
{
   max_entry_proc();
   max_sleep_task (parameter.hMyself, parameter.rcTDT.usTaskNo);
   parameter.usProgState = PROG_READY;
   max_exit_proc();
}

/****************************************************************************/
void main ()                 /* ---- PREPARE ---- */
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_II_TASK +
                                  MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = MAX1_IRQ_TIMERA;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = DAQ_PROG_NO;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / sizeof(void*);

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;
   rcPDT.measure_init     = MeasureInit;
   rcPDT.start            = Start;
   rcPDT.stop             = Stop;
   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
