{******************************************************************************
 Sample Real Time program

 The program measures the analog inputs of the X-AD14-20
 and write the values into a buffer.
 If the buffer is filled with a certain number of values a srq
 is sent to the PC, which can then read out the values from the buffer.

 Program Number $9005

 (C)2000 SORCUS Computer GmbH
{******************************************************************************}

{ Compiler settings }
{$F+}                                     { FAR Calls ! }
{$N-}                                     { No Coprocessor }
{$E-}                                     { No Emulation (application specific) }
{$R-}                                     { No Range-Check }
{$S-}                                     { No Stack-Checking }
{$I-}                                     { No IO-Checking }
{$V-}                                     { No strict vat string check}

{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses
   Maxlib, X_error, maxtype, maxdefs, maxmdd;
{ the program uses the following SORCUS units
  Maxlib   - all functions and procedure declarations
  X_error  - defined error message constants
  maxtype  - all type definitions used in the library
  maxdefs  - various constant definitions
  maxmdd   - needed for MDDs: It defines CPS-types and constants
}

{ constant definitions }
const
   PROGRAMM_NUMBER : WORD = $9005;
   VERSION   : char       = '1';
   REVISION  : char       = 'E';
   SRQ_ERROR : WORD       = $8182;

   PROG_NOT_READY         = 0;
   PROG_INITIALIZED       = 1;
   PROG_READY             = 2;
   PROG_STARTED           = 3;

   ERR_IN_AUTO_INIT       = $1AE0;

type

{ program descriptor table }
pdt_type = RECORD
   rcHead    : MAX_PDT_HEAD;        { standard PDT header }
   main_proc : POINTER;             { pointer to proc. 0  }
   auto_init : POINTER;             { pointer to proc. 1  }
   measure_init : POINTER;          { pointer to func. 2  }
   start     : POINTER;             { pointer to proc. 3  }
   stop      : POINTER;             { pointer to proc. 4  }
end;

{ parameter range (is part of the program
  because PDT-flag MAXPDT_PARAMETER_IN_PROG is set) }
parameter_type = RECORD
   rcTDT          : MAX_TDT_TYPE ;     { must be before the parameter range }
{ Offset 0: }
   Error          : MAX_ERROR;   { global error variable }
   usProgState    : WORD;        { State of the program  }
   usXAD14_slot   : WORD;        { location of the X-AD14-20 }
   usTimerRate    : WORD;        { sampling rate         }
   usFirstChannel : WORD;        { first analog input to be measured }
{ Offset 10 }
   usLastChannel  : WORD;        { last analog input to be measured }
   usChannel      : WORD;        { index of actual channel that is measured }
   usBufferFullSRQ: WORD;        { SRQ word that is used to indicate a filled buffer}
   usErrorSRQ     : WORD;        { SRQ word that is used to indicate an error}
   usHostNo       : WORD;        { number of target for SRQs}
{ Offset  20 }
   ulBufferSize   : LONGWORD;    { size of the buffer }
   usReleaseSRQ   : WORD;        { flag that is set while buffer is filled
                                   less than 1/3 and reset while buffer is filled
                                   more than 1/3 }
   usDummy        : WORD;        { reserved }
   hBuffer        : MAXBUFHND;   { Handle for data buffer        }
   hMyself        : MAXMODHND;   { handle for the CPU            }
   hHost          : MAXMODHND;   { handle for the PC             }
{ Offset  40 }
   hAin           : Array[0..19] of MAXCHLHND; { handles for the analog input channels}
{ Offset  120 }
   ulSampleCount  : LONGWORD;    { number of samples }
   hMdd           : MAXMDDHND;   { handle for the X-AD14-20 MDD  }
   rcOsX          : MAX_OS_INFO;
   ulValid        : LONGWORD;    { counter for number of valid bytes in buffer }
   usErrCount     : WORD;        { error counter }
end;

{ record type used to transfer a set of configuration-data to the Realtime program }
type DAQ_INIT_DATA = record
   usADSlot      : WORD;
   usInputMode   : WORD;
   usInputRange  : WORD;
   usSettleTime  : WORD;
   usFirstChannel: WORD;
   usLastChannel : WORD;
   ulBufferID    : LONGWORD;
end;

{ global variables }
var
   rcPDT               : pdt_type;
   parameter           : parameter_type;

{****************************************************************************}
{ in case of error send srq to PC                                            }
PROCEDURE ErrorHandler;
begin
   Inc(parameter.usErrCount);
   parameter.Error := max_send_srq (parameter.hHost, 0, SRQ_ERROR);
end;

{****************************************************************************}
{ Init task parameter                                                        }
PROCEDURE AutoInit; far;
begin
   max_entry_proc;

   parameter.usProgState      := PROG_NOT_READY;
   parameter.usTimerRate      := 24;	{ =20us, 119=100 us }
   parameter.usXAD14_slot     := 1;
   parameter.usFirstChannel   := 0;
   parameter.usLastChannel    := 19;
   parameter.ulBufferSize     := 0;
   parameter.usBufferFullSRQ  := $8064;
   parameter.usErrorSRQ       := $8060;
   parameter.usHostNo         := 1;     { PC via PCI bridge }
   parameter.ulSampleCount    := 0;
   parameter.usErrCount       := 0;
   parameter.ulValid          := 0;
   parameter.Error            := max_init_lib(0);
   if (parameter.Error = ERR_OK) then begin
      { get a handle for the X-MAX-1 }
      parameter.Error := max_connect_cpu (0, MAX_MYSELF, 0,
                                          parameter.rcOsX, parameter.hMyself);
      if (parameter.Error = ERR_OK) then
         parameter.usProgState := PROG_INITIALIZED;
   end;
   max_exit_proc;
end;

{****************************************************************************}
{ main procedure of the task (called automatically by OsX after activating the task) }
PROCEDURE MainProc; far;
var
   Data   : LONGINT;
   ulSize : LONGWORD;
   ulFree: LONGWORD;
begin
   max_entry_proc;

   if (parameter.usProgState = PROG_STARTED) then begin
      { read one input channel }
      parameter.Error := max_read_channel_long(parameter.hAin[parameter.usChannel], Data);
      if (parameter.Error <> ERR_OK) then
         ErrorHandler;
      { write value into buffer }
      ulSize := sizeof(LONGINT);
      parameter.Error := max_set_buffer(parameter.hBuffer, MAX_BUFFER_MOVE_ABS,
				        ulSize, Data);
      if (parameter.Error <> ERR_OK) then
         ErrorHandler
      else begin
         parameter.Error := max_get_buffer_status(parameter.hBuffer,
                                                  parameter.ulValid, ulFree);
         if (parameter.Error <> ERR_OK) then
            ErrorHandler
         else begin
            { check if SRQ has to be sent }
            if (parameter.usReleaseSRQ = 1) then begin
               { no SRQ has been sent since buffer is filled more than 1/3 }
               if (parameter.ulValid >= (parameter.ulBufferSize / 3)) then begin
                  parameter.usReleaseSRQ := 0;
                  { if buffer is filled more than 1/3 send SRQ to the host }
		  parameter.Error := max_send_srq(parameter.hHost, 0,
                                                  parameter.usBufferFullSRQ);
                  if (parameter.Error <> ERR_OK) then
                     ErrorHandler;
               end;
            end
            else begin
               { SRQ has been sent already since buffer is filled more than 1/3 }
               if (parameter.ulValid < (parameter.ulBufferSize / 3)) then
                  { enable SRQs again if buffer state falls below 1/3 }
                  parameter.usReleaseSRQ := 1;
            end;
         end;
      end;
      if (parameter.usChannel = parameter.usLastChannel) then
         parameter.usChannel := parameter.usFirstChannel
      else
         Inc(parameter.usChannel);
      Inc(parameter.ulSampleCount);
   end;
   max_exit_proc;
end;

{****************************************************************************}
{ open channels                                                              }
PROCEDURE MeasureInit (VAR pusInsize : WORD; VAR prcInit : DAQ_INIT_DATA;
                       VAR pusOutsize: WORD; VAR  pusDataOut : WORD); far;
var
   Error     : MAX_ERROR;
   CPS_Ain   : CPS_XAD1420;
   i         : WORD;
   Dummy     : MAX_OS_INFO;
   rcPrgInfo : MAX_PGM_TYPE;
   v         : LONGWORD;

begin
   max_entry_func;
   Error := ERR_OK;
   if (pusInsize <> sizeof(DAQ_INIT_DATA)) then
      Error := ERR_WRONG_NUMBER_OF_INPUTS
   else begin
      if (parameter.usProgState <> PROG_INITIALIZED) then
         Error := ERR_IN_AUTO_INIT;
   end;
   if (Error = ERR_OK) then begin
      parameter.usXAD14_slot   := prcInit.usADSlot;
      parameter.usFirstChannel := prcInit.usFirstChannel;
      parameter.usLastChannel  := prcInit.usLastChannel;

      { get a handle for the host PC (to send SRQs) }
      Error := max_connect_cpu (parameter.usHostNo, 0, 0, Dummy, parameter.hHost);
      if (Error = ERR_OK) then
         { get a handle for the MDD }
         Error := max_load_mdd (parameter.hMyself, parameter.usXAD14_slot, 0, 0,
		                $8024, rcPrgInfo, parameter.hMdd);
      if (Error = ERR_OK) then
         Error := max_get_buffer_handle (parameter.hMyself, prcInit.ulBufferID,
                                         parameter.hBuffer);
      if (Error = ERR_OK) then
         { get buffersize }
         Error := max_get_buffer_status(parameter.hBuffer, v, parameter.ulBufferSize);
      if (Error = ERR_OK) then begin
         CPS_Ain.usDevice     := prcInit.usInputMode;
         CPS_Ain.usFlags      := _CP_UNCORRECTED;
         CPS_Ain.usRange      := prcInit.usInputRange;
         CPS_Ain.usReadMode   := IO_MODE_DIRECT;
         CPS_Ain.usSettleTime := prcInit.usSettleTime;
         { open channels }
         for i:=parameter.usFirstChannel to (parameter.usLastChannel+1) do begin
            CPS_Ain.usIndexFirst := i;
            CPS_Ain.usIndexLast  := i;
            Error := max_open_channel (parameter.hMdd, sizeof (CPS_XAD1420),
                                       CPS_Ain, NIL, 0, parameter.hAin[i]);
            if (Error <> ERR_OK) then
               break;
         end;
       end;
       { increase performance by not checking for invalid handles
          and wrong data type check in the library }
       max_set_error_check_level(0);
       { switch CPU into the fastest mode }
       Error := max_cpu_speed_control (parameter.hMyself, MAX1_CPU_HYPER_SPEED,
                                       MAX1_SPEED_100MHZ);
       if (Error = ERR_OK) then
          parameter.usProgState := PROG_READY;
   end;
   max_exit_func(Error);
end;

{****************************************************************************}
{ Start the task                                                              }
PROCEDURE Start; far;
var
   rcTIParams : MAX_TI_TYPE;
begin
   max_entry_proc;
   if (parameter.usProgState = PROG_READY) then begin
      max_clear_buffer(parameter.hBuffer);  { empty buffer }
      parameter.usProgState  := PROG_STARTED;
      parameter.usChannel    := parameter.usFirstChannel;
      parameter.usReleaseSRQ := 1;

      { activate the task }
      { installation determines wether program runs as NI- or II-Task }
      if ((parameter.rcTDT.usFlags AND 3) = MAX_NI_TASK) then
         parameter.Error := max_wakeup_ni_task(parameter.hMyself,
                                               parameter.rcTDT.usTaskNo, 1)
      else begin
         if ((parameter.rcTDT.usFlags AND 3) = MAX_II_TASK) then begin
            { init and start Timer A }
            parameter.Error := max_set_timer(parameter.hMyself, MAX1_TIMER_A,
                                             MAX1_TIMER_MODE_INT, parameter.usTimerRate);
            parameter.Error := max_wakeup_ii_task(parameter.hMyself,
                                             parameter.rcTDT.usTaskNo);
         end
         else
            if ((parameter.rcTDT.usFlags AND 3) = MAX_TI_TASK) then begin
               rcTIParams.usPriority := 0;
               rcTIParams.ulHoldOff  := 0;
               rcTIParams.ulIntervall:= parameter.usTimerRate;
               rcTIParams.ulCalls    := 0;
               parameter.Error := max_wakeup_ti_task(parameter.hMyself,
                                                     parameter.rcTDT.usTaskNo,
                                                     rcTIParams);
            end;
      end;
   end;
   max_exit_proc;
end;

{****************************************************************************}
{ Stop the task                                                              }
PROCEDURE Stop; far;
var
   usTask : WORD;
begin
   usTask := max_entry_proc;
   parameter.Error := max_sleep_task (parameter.hMyself, usTask);
   parameter.usProgState := PROG_READY;
   max_exit_proc;
end;

{****************************************************************************}

begin
   { the PDT-flags determine the task-type and
    information how the data and parameter range are allocated }
   rcPDT.rcHead.usFlags         := MAX_II_TASK +
                                   MAXPDT_PARAMETER_IN_PROG; { parameter are part of the program }

   { interrupt source (0 for NI-tasks) }
   rcPDT.rcHead.usInterrupt     := MAX1_IRQ_TIMERA;

   { the program number identifies the program uniquely }
   rcPDT.rcHead.usProgNo        := PROGRAMM_NUMBER;

   { number of global task procedures in the PDT }
   rcPDT.rcHead.usProcedures    := (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) div 4;

   { the following lines are just for documentation and information purpose }
   rcPDT.rcHead.ucType          := 1;   { always ! }
   rcPDT.rcHead.ucSize          := sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        := VERSION;
   rcPDT.rcHead.cRevision       := REVISION;
   rcPDT.rcHead.ucCPU           := MAX_CPU486;
   rcPDT.rcHead.ucCoProc        := _NOCOPROC;
   rcPDT.rcHead.ucLanguage      := MAXRT_LANGUAGE_PAS;
   rcPDT.rcHead.ucProgType      := 0;   { not used }
   rcPDT.rcHead.ulRes1          := 0;
   rcPDT.rcHead.ulRes2          := 0;
   rcPDT.rcHead.ulHyperAdr      := 0;   { feature not available yet }

   { address and size of the tasks data range
     only when MAXPDT_DATA_IN_PROG is set in PDT-flags }
   rcPDT.rcHead.ulDataAdr       := 0;
   rcPDT.rcHead.ulDataSize      := 0;

   { address and size of the tasks parameter range
     only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags }
   rcPDT.rcHead.ulParAdr  := max_get_physical_addr(@parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize := sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   { initialize global procedure addresses in PDT }
   rcPDT.main_proc        := @MainProc;
   rcPDT.auto_init        := @AutoInit;
   rcPDT.measure_init     := @MeasureInit;
   rcPDT.start            := @Start;
   rcPDT.stop             := @Stop;
   { transfer PDT information to the OsX }
   max_init_program (@rcPDT);
end.