/****************************************************************************/
// EXDAC
//
// Sample program for real time data acquisition.
//
// (C)2002 SORCUS Computer GmbH
/****************************************************************************/

#pragma option -r-

// _RTLIB_ must be defined in every real time program
#define _RTLIB_

#include "..\exdacifc.h"
#include "dos.h"
#include "stdlib.h"

#define VERSION         '1'
#define REVISION        'A'

// PDT (Program Descriptor Table) data structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
   void  MAXEXPORT (*measure_init) (USHORT *pusInsize, DAQ_INIT_DATA *pInit,
                                    USHORT *pusOutsize, USHORT *pDataOut);
   void* MAXEXPORT start;
   void* MAXEXPORT stop;
} rcPDT;

// parameter range of the task
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;
// 0: State machine parameter
   MAX_ERROR    Error;              // global error variable
   USHORT       usProgState;        // State of the program
   ULONG        ulSampleCount;      // number of samples
   USHORT       usReleaseSRQ;       // flag that is set while buffer is filled
                                    // less than 1/3 and reset while buffer is filled
                                    // more than 1/3
// 10: Configuration parameter
   USHORT       usBufferFullSRQ;    // SRQ word that is used to indicate a filled buffer
   USHORT       usErrorSRQ;         // SRQ word that is used to indicate an error
   USHORT       usFinishSRQ;        // SRQ word that is used to indicate end of measurement
   USHORT       usHostNo;           // number of target for SRQs
   USHORT       usTimerRate;        // sampling rate
   ULONG			 ulMaxSamples;			// number of data acquisitions (0=forever)
   ULONG        ulBufferSize;       // size of the buffer
// 28: Store Handles
   MAXBUFHND    hBuffer;            // Handle for data buffer
   MAXMODHND    hMyself;            // handle for the CPU
   MAXMODHND    hHost;              // handle for the PC
   MAXCHLHND    hAin;           		// handle for the analog input channel
	MAXMDDHND    hADMdd;             // handle for the X-AD14-20 MDD
// 48: store active OsX version 
   MAX_OS_INFO  rcOsX;
} parameter;

// states of the program
enum
{
   PROG_NOT_READY,
   PROG_INITIALIZED,
   PROG_READY,
   PROG_STARTED,
};

// Prototypes
void MAXEXPORT MainProc (void);
void MAXEXPORT AutoInit (void);
void MAXEXPORT MeasureInit(USHORT *pusInsize, DAQ_INIT_DATA *prcInit,
                           USHORT *pusOutsize, USHORT *pusDataOut);
void MAXEXPORT Start (void);
void MAXEXPORT Stop (void);
void ErrorHandler (void);

/****************************************************************************/
// #1 : init task parameter
void MAXEXPORT AutoInit(void)
{
   max_entry_proc();

   // init program state
   parameter.usProgState      = PROG_NOT_READY;

  	// the following parameters are set to default values
	// they can be set by the PC using max_write_par_ushort:
	parameter.usFinishSRQ	   = SRQ_FINISHED;
   parameter.usBufferFullSRQ  = SRQ_DATA_READY;
   parameter.usErrorSRQ       = SRQ_ERROR;
   parameter.usHostNo         = 1;     // PC via PCI bridge

  	// start library
   parameter.Error = max_init_lib(0);
   if (parameter.Error == ERR_OK)
      // get a handle for the X-MAX-1
      parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,
                                         &parameter.rcOsX, &parameter.hMyself);
   if (parameter.Error == ERR_OK)
      parameter.usProgState = PROG_INITIALIZED;
   max_exit_proc();
};

#pragma argsused
/****************************************************************************/
// #2 : open channels 
void MAXEXPORT MeasureInit(USHORT *pusInsize, DAQ_INIT_DATA *prcInit,
                           USHORT *pusOutsize, USHORT *pusDataOut)
{
   MAX_ERROR   Error;
   CPS_XAD1420 CPS_Ain;
   ULONG       ulValid;
   USHORT		usMode;

   max_entry_func();
   Error = ERR_OK;

   parameter.usTimerRate = prcInit->usTimeValue;
   parameter.ulMaxSamples = prcInit->ulAcquisitions;
   if (*pusInsize != sizeof(DAQ_INIT_DATA))
      Error = ERR_WRONG_NUMBER_OF_INPUTS;
   else
      if (parameter.usProgState != PROG_INITIALIZED)
         Error = ERR_INVALID_HANDLE;

	if (Error == ERR_OK)
      // get a handle for the host PC (to send SRQs)
      Error = max_connect_cpu (parameter.usHostNo, 0, 0, NULL, &parameter.hHost);
   if (Error == ERR_OK)
      // get a handle for the buffer
      Error = max_get_buffer_handle (parameter.hMyself, prcInit->ulBufferID,
								             &parameter.hBuffer);
   if (Error == ERR_OK)
      // get buffersize
      Error = max_get_buffer_status(parameter.hBuffer,
 			                           &ulValid, &parameter.ulBufferSize);

   if (Error == ERR_OK)
     	// get a handle for the X-AD14-20 MDD
   	Error = max_load_mdd (parameter.hMyself, prcInit->usADSlot, 0, 0,
					          	 MDD_X_AD14_20, NULL, &parameter.hADMdd);
  	if (Error == ERR_OK)
  	{
     	// open a channel for analog inputs AIN-0 and AIN-1
     	CPS_Ain.usDevice     = DEVICE_AIN_SE;
     	CPS_Ain.usIndexFirst = 0;
     	CPS_Ain.usIndexLast  = 1;
     	CPS_Ain.usRange      = RANGE_BIP_10V;
      CPS_Ain.usFlags      = _CP_UNCORRECTED;
      CPS_Ain.usReadMode   = IO_MODE_DIRECT;
      CPS_Ain.usSettleTime = 500;
      Error = max_open_channel (parameter.hADMdd, sizeof (CPS_XAD1420),
                                &CPS_Ain, NULL, 0, &parameter.hAin);
   }

   // increase performance by not checking for invalid handles
   // and wrong data type check in the library
   max_set_error_check_level(0);
   // switch CPU cache into the fastest mode
   usMode = 3;
   max_cache_control(parameter.hMyself, &usMode);
   Error = max_cpu_speed_control(parameter.hMyself, MAX1_CPU_HYPER_SPEED,
                                 MAX1_SPEED_100MHZ);
   if (Error == ERR_OK)
       parameter.usProgState = PROG_READY;

	max_exit_func(Error);
}

/****************************************************************************/
// #3 : start the task
void MAXEXPORT Start(void)
{
   max_entry_proc();
   if (parameter.usProgState == PROG_READY)
   {
      max_clear_buffer(parameter.hBuffer);  // empty buffer
      parameter.usProgState  = PROG_STARTED;
      parameter.usReleaseSRQ = 1;
	   parameter.ulSampleCount= 0;

      // activate the task: init and start Timer
      parameter.Error = max_set_timer(parameter.hMyself,
                                      MAX1_TIMER_A, MAX1_TIMER_MODE_INT,
                                      parameter.usTimerRate);
      parameter.Error = max_wakeup_ii_task(parameter.hMyself,
                                           parameter.rcTDT.usTaskNo);
   }
   max_exit_proc();
}

/****************************************************************************/
// stop the task
void MAXEXPORT Stop(void)
{
   max_entry_proc();
   max_sleep_task (parameter.hMyself, parameter.rcTDT.usTaskNo);
   parameter.usProgState = PROG_READY;
   max_exit_proc();
}

/****************************************************************************/
// #0 : main procedure of the task (called automatically by OsX after starting the task)
void MAXEXPORT MainProc(void)
{
	ULONG			ulSize, ulFree, ulValid;
	OSX_BUFFER_DATA MeasureData;

   max_entry_proc();
   if (parameter.usProgState == PROG_STARTED)
   {
      // read channels
		ulSize = 8; 	// 2 long values for analog inputs
      parameter.Error = max_read_channel_block(parameter.hAin, &ulSize, MeasureData.alAin);
      if (parameter.Error != ERR_OK)
         ErrorHandler();
      else
      {  // write values into buffer
 		   ulSize = sizeof(MeasureData);
         parameter.Error = max_set_buffer(parameter.hBuffer, MAX_BUFFER_MOVE_ABS,
   		                                 &ulSize, &MeasureData);
         if (parameter.Error != ERR_OK)
            ErrorHandler();
         else
         {
				// continuous acquisition
	         parameter.Error = max_get_buffer_status(parameter.hBuffer,
	   										                &ulValid, &ulFree);
            // check if SRQ has to be sent
	         if (parameter.usReleaseSRQ)
	         {
	            // no SRQ has been sent since buffer is filled more than 1/3
	            if (ulValid >= (parameter.ulBufferSize / 3))
	            {
	               parameter.usReleaseSRQ = 0;
	               // if buffer is filled more than 1/3 send SRQ to the host
				      parameter.Error = max_send_srq(parameter.hHost, 0,
	                                              parameter.usBufferFullSRQ);
	               if (parameter.Error != ERR_OK)
	                  ErrorHandler();
	            }
            }
	         else
	         {  // SRQ has been sent already since buffer is filled more than 1/3
	            if (ulValid < (parameter.ulBufferSize / 3))
	               // enable SRQs again if buffer state falls below 1/3
	               parameter.usReleaseSRQ = 1;
            }
	         parameter.ulSampleCount++;
				// check if desired number of data acquisitions is done
           	if (parameter.ulSampleCount == parameter.ulMaxSamples)
            {  
               // if number of acquisitions is reached
               parameter.Error = max_send_srq(parameter.hHost, 0,
                                              parameter.usFinishSRQ);
					// call stop procedure to stop measurement
					max_call_proc(parameter.hMyself, parameter.rcTDT.usTaskNo, 4);
            }
         }
		}
   }
   max_exit_proc();
}

/****************************************************************************/
void main ()                 /* ---- PREPARE ---- */
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_II_TASK +
                                  MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = MAX1_IRQ_TIMERA;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = EXDAC_PROG_NO;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / sizeof(void*);

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;
   rcPDT.measure_init     = MeasureInit;
   rcPDT.start            = Start;
   rcPDT.stop             = Stop;
   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}

/****************************************************************************/
// in case of error send srq to PC
void ErrorHandler()
{
   max_send_srq (parameter.hHost, 0, parameter.usErrorSRQ);
}

