/****************************************************************************/
// Sample Real Time program (NI-Task)
//
// It activates the parallel port on the X-MAX-1 module and provides a function
// to print a given string to a line oriented printer
//
// (C)2001 SORCUS Computer GmbH
/****************************************************************************/

#pragma option -r-

#define _RTLIB_                        // must be defined in every real time
                                       // program before including max_lib.h
#include "max_lib.h"
#include "x_error.h"
#include "dos.h"

#define PROGRAMM_NUMBER 0xa006
#define VERSION         '1'
#define REVISION        'A'

//register addresses of the parallel port
#define DATA_REGISTER         0x378
#define STATUS_REGISTER       0x379
#define CONTROL_REGISTER      0x37a

//declare functions
void MAXEXPORT auto_init();
void MAXEXPORT main_proc();
void MAXEXPORT print_line(USHORT *pusLength, UCHAR *pucString, USHORT *pusRetsize, void *pRetData);
MAX_ERROR max_init_lpt(MAXMODHND hCPUModule);
MAX_ERROR max_print_uchar(MAXMODHND hCPUModule,UCHAR character);

// PDT structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
   // any number of user defined procedures may follow (optionally)
   void* MAXEXPORT print_line;
} rcPDT;

// parameter range structure
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;                 // always before the first parameter !
// offset 0
   MAX_ERROR    Error;                 // Errorcode
// offset 2
   USHORT       usTimeout;             // Time to wait for printer to be
                                       // ready
   USHORT       usStrobeWait;          // amount of time to hold STROBE until
                                       // releasing it again
   MAXMODHND    hMe;                   // Handle for the CPU module
   MAX_OS_INFO  rcOsX;                 // Version info about OsX
} parameter;


/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
void MAXEXPORT main_proc(void)
{
   //do nothing

   max_entry_proc();

   max_exit_proc();
}

/****************************************************************************/
// Init task parameter
void MAXEXPORT auto_init(void)
{
   max_entry_proc();
   parameter.Error = max_init_lib(0);
   if (parameter.Error == ERR_OK)
   {
      // get a handle for the X-MAX-1
      parameter.Error = max_connect_cpu (
      0, MAX_MYSELF, 0, &parameter.rcOsX, &parameter.hMe);
   }

   parameter.usTimeout=100;
   parameter.usStrobeWait=50;

   //enable parallel port
   parameter.Error = max_init_lpt(parameter.hMe);

   max_exit_proc();
}

/****************************************************************************/
//enable the parallel port and set it to bidirectional mode
MAX_ERROR max_init_lpt(MAXMODHND hCPUModule)
{
	UCHAR lpt_control;
	MAX_ERROR Error;

	//activate parallel port
	Error = max_write_io_indexed(hCPUModule,0x22,0x39,0xff,0x2);
	if (Error!=0) return Error;
   Error = max_write_io_indexed(hCPUModule,0x22,0xd1,0xf7,0x4);
	if (Error!=0) return Error;

	//bidirectional mode
	Error = max_write_io_indexed(hCPUModule,0x22,0xd2,0xff,0x2);
	if (Error!=0) return Error;

	//read control register
	Error = max_read_io_byte(hCPUModule,0x37a,&lpt_control);
	if (Error!=0) return Error;

	//set SLCTIN and unset INIT pin
	Error = max_write_io_byte(hCPUModule,0x37a,lpt_control|0x8|0x4);

	return Error;
}

/****************************************************************************/
//print a single character
MAX_ERROR max_print_uchar(MAXMODHND hCPUModule,UCHAR character)
{
	MAX_ERROR Error;
	UCHAR lpt_status,lpt_control;
	int i;

	Error = max_read_io_byte(hCPUModule,STATUS_REGISTER,&lpt_status);
   if (Error!=0) return Error;

   //mask out BUSY bit
	lpt_status=lpt_status & 0x80;
	i=0;

	//whait for printer to be ready
	while (lpt_status!=0x80)
	{
      //check for timeout
		if (i==parameter.usTimeout) return 0x01e0;

		Error = max_read_io_byte(hCPUModule,STATUS_REGISTER,&lpt_status);
		if (Error!=0) return Error;

		lpt_status=lpt_status & 0x80;
		i++;
	}

	Error = max_write_io_byte(hCPUModule,DATA_REGISTER,character);
	if (Error!=0) return Error;

	//read control register
	Error = max_read_io_byte(hCPUModule,CONTROL_REGISTER,&lpt_control);;
	if (Error!=0) return Error;

	//set strobe
	Error = max_write_io_byte(hCPUModule,CONTROL_REGISTER,lpt_control|0x1);
	if (Error!=0) return Error;

    //wait a while
    for (i=0;i<parameter.usStrobeWait;i++)

	//unset strobe
	Error = max_write_io_byte(hCPUModule,CONTROL_REGISTER,lpt_control&0xfe);

	return Error;
}

/****************************************************************************/
// print a line
// pusLength is a pointer to the number of characters in the line
// pucString is the string to print
// pusRetsize and pRetData are not used but need to be declared 
void MAXEXPORT print_line(USHORT *pusLength, UCHAR *pucString,
                                 USHORT *pusRetsize, void *pRetData)
{
   int i;

   max_entry_func();

   for (i=0;i<*pusLength;i++)
   {
      parameter.Error = max_print_uchar(parameter.hMe,pucString[i]);
      if (parameter.Error!=ERR_OK) break;
   }

   max_exit_func(parameter.Error);
}

/****************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_NI_TASK +
                                  MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = 0;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = main_proc;
   rcPDT.auto_init        = auto_init;
   rcPDT.print_line       = print_line;

   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
