/****************************************************************************/
/*                                                                          */
/* PC-Sample for buffered real time data acquisition with X-DAD-4           */
/*                                                                          */
/* In particluar it is shown how to open a MDD-channel in the PC-program    */
/* and how this channel-handle can be transfered to a realtime-program.     */ 
/*                                                                          */
/* The program uses the realtime program dad4smpl.exe in the RT directory.  */
/*                                                                          */
/* Note: With this program maximum performance can not be reached!          */
/*       The main intention of this program is to demonstrate parallel      */
/*       processing on the MAX-PC and the Windows PC. If higher performance */
/*       is required both this program and the realtime program have to be  */
/*       optimized.                                                         */
/*                                                                          */
/* Initialization (done in main())                                          */
/* The PC program first initializes the board.                              */
/* Then it downloads the realtime program, initializes it by setting some   */
/* parameters and starts the realtime program by calling the tasks' start   */
/* function.                                                                */
/* The MDD for the module is loaded onto the CPU module.                    */
/* The PC-program creates a buffer on the MAX-PC that is used to buffer     */
/* the measured data values. 	                                              */
/* The PC-program opens a channel to one analog input of the module and     */
/* exports the retrieved channel handle to the realtime-program.            */
/*                                                                          */
/* Function of the realtime-program                                         */
/* The continous acquisition of the analog values is done totally by the    */
/* realtime program.                                                        */
/* The realtime program is called according to a timer of the CPU-module.   */
/* Each time the timer expires an interrupt is generated on the CPU-module. */
/* The Interrupt-Service-Routine (Procedure 0 of the task) then reads one   */
/* analog input channel and stores the value in the buffer.		             */
/* If a the buffer is filled up to a certain level, a SRQ will be sent to   */
/* this PC-program.                                                         */
/*                                                                          */
/* SRQ-reaction (on the PC)                                                 */
/* If the PC receives the SRQ the library will call the SRQ-Service-function*/
/* (called 'ReadMeasuredData'). That function examines the received SRQ.    */
/* For SRQs that indicate that the number of values is available in the CPU */
/* modules buffer it reads out the buffer then. Other SRQs indicate errors  */
/* in the realtime system.                                                  */
/*                                                                          */
/* Copyright SORCUS Computer GmbH 2001	                                     */ 
/****************************************************************************/

#include "windows.h"
#include "stdio.h"
#include "conio.h"
#include "max_lib.h"
#include "x_mdd.h"

#define SRQ_DATA_READY	0x8181	// SRQ-word used to signal 'data available'
#define SRQ_RT_ERROR		0x9292	// SRQ-word used to signal errors
#define BUFFERSIZE		1000		// size of the data buffer 

USHORT	 usBoardNr = 1;			// see contol panel 'SORCUS boards'
USHORT	 usCPUSlot, usCPULayer;	// Location of the CPU module
USHORT	 usADSlot,  usADLayer;	// Location of the X-DAD-4 module
MAXMODHND hModul;						// handle for the CPU modules
MAXMDDHND hMdd;						// handle for the module-device-driver
MAXBUFHND hBuffer;					// handle for the data buffer
USHORT	 MEASURE_TASK = 68;		// task number of the real-time program
long		 *plData;					// pointer to measured data
USHORT	 *pusData;					// pointer to measured data
USHORT	 usInputRange;
ULONG		 ulDataType;				

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
	USHORT	 usMsgSize = 100;
	char		 sErrMsg[200];

	max_get_error_message(Error, &usMsgSize, sErrMsg);
	printf ("\n\rError %04Xh in %s (%s)", Error, sText, sErrMsg);
	max_call_proc(hModul, MEASURE_TASK, 4);
//	_getch();									// user has to confirm
	max_clear_error();
}

/****************************************************************************/
// Cleanup procedure
void ExitProgram(USHORT FreeMemory)
{
	max_exit_lib();
	if (FreeMemory)
		free (plData);
}

/****************************************************************************/
// SRQ service Procedure
MAX_CALLBACK ReadMeasuredData (MAXMODHND hSrqModul, ULONG ulMsg)
{
	ULONG			ulSize, i;
	MAX_ERROR	Error;
	double		fpValue;

	if (hModul != hSrqModul)				// SRQ from another module
		return;									// ignore it			
	if (ulMsg == SRQ_DATA_READY) { 
		// the realtime program signals that data is available
		// read out measured data values from the buffer on the CPU-module
		// (as many as possible)
		ulSize = BUFFERSIZE;
		Error = max_get_buffer(hBuffer, MAX_BUFFER_MOVE_MAX, &ulSize, plData);
		// the function returns in 'ulSize' how many bytes could be read
		if (Error) 
			ErrorHandler(Error, "max_get_buffer");
		else 
		{	// display the analog values 

/*			// the following lines are only needed if channels data is
			// read in hardware-format (see max_open_channel)
			if (ulDataType == DATA_USHORT) 
			{   
				if (usInputRange == RANGE_BIP_10V) 
				{
					for (i=0; i<ulSize/sizeof(USHORT); i++) 
					{
						if (pusData[i] & 0x800)	{ // negative value
							pusData[i] -= 0x800;
							fpValue = (double)pusData[i] * 10;
							fpValue /= 0x7FF;
							fpValue -= 10;
						}
						else {
							fpValue = (double)pusData[i] * 10;
							fpValue /= 0x7FF;
						}
						printf("\n\r%+.3fV", fpValue);
					}
				}
				else
					printf("\n\rDisplay of values only for Range +/-10V");
			}
			else 
*/			{	// measurment in normalized-analog-format
				for (i=0; i<ulSize/sizeof(long); i++) {
					fpValue = (double)plData[i] / 1000000;
					printf("\n\r%+.3fV", fpValue);
				}
			}
		}
	}
	else {
 		if (ulMsg == SRQ_RT_ERROR) {
			// the realtime program signals an error, code is stored in a parameter
			max_read_par_ushort(hModul, MEASURE_TASK, 0, &Error);
			ErrorHandler(Error, "RT-program");
		}
		else
			ErrorHandler((USHORT)ulMsg, "(SRQ-Message)");
	}
}

/****************************************************************************/
void main () 
{
	MAX_ERROR		Error;
	MAX_OS_INFO		rcOsInfo;				// Info about the OsX activated on the CPU module
	MAX_PGM_TYPE	rcPrgInfo;				// Info about the real-time program
	MAX_BUFFER_TYPE rcBufferPar;			// configuration data for the data buffer
	CPS_XDAD4		rcChannelCfg;			// configuration data for the AIN channel
	MAXCHLHND		hAin; 					// handle for AIN
	MAX_VERSION		Version;
	ULONG				ulDate, ulSize, ulBufferID;
	USHORT			usSize;
	char				szVersion[12], szDate[12];
	char				sFilename[80];			

	usADSlot  = 5;								// where the X-DAD-4 is
	usADLayer = 0;

	printf ("Sample program for buffered analog data acquisition with X-DAD-4");
	Error = max_init_lib (0);
	// display some information about the system
	max_get_lib_version (&Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);
	printf ("\n\rLib Version: %s (%s)", szVersion, szDate);
	Error = max_get_drv_version (&Version, &ulDate);
	if (Error) {
		printf ("Error %X", Error);
		max_clear_error();
	}
	else {
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf ("\n\rDriver Version: %s (%s)", szVersion, szDate);
	}

	// reset the board
	Error = max_reset_board (usBoardNr, 20);
	if (Error) {
		ErrorHandler(Error, "max_reset_board");
		ExitProgram(0);
		return;
	}
	
	// connect to the CPU module
	Error = max_connect_cpu (usBoardNr, MAX_AUTO_SLOT, 0, &rcOsInfo, &hModul);
	if (Error) {
		ErrorHandler(Error, "max_connect_cpu");
		ExitProgram(0);
		return;
	}
	
	// load or activate an operating system
	if (rcOsInfo.usType == MAX_MINI_OSX) 
	{
		Error = max_load_osx (hModul, "ROM");
		if (Error) {
			ErrorHandler(Error, "max_load_osx");
			ExitProgram(0);
			return;
		}
		Error = max_get_os_version(hModul, &rcOsInfo);
		if (Error) 
			ErrorHandler(Error, "max_get_os_version");
	}
	Error = max_get_slot(hModul, &usCPUSlot, &usCPULayer);
	max_get_version_string(rcOsInfo.Version, rcOsInfo.ulDate, szVersion, szDate);
	printf ("\n\rConnected to CPU on slot %X, Layer %X: OS Type=%X, Version=%s (%s)",
			   usCPUSlot, usCPULayer, rcOsInfo.usType, szVersion, szDate);

	// If the communication with the PC-host is interruptible on the X-MAX-1
	// the interrupt-driven on-board data aquisition will not be influenced by 
	// the communication with the host PC. (recommended setting)
   Error = max_macro_interruptible(hModul, 0);
	if (Error) 
		ErrorHandler(Error, "max_macro_interruptible");

	// install SRQ service procedure
	Error = max_set_service(hModul, ReadMeasuredData);
	if (Error) {
		ErrorHandler(Error, "max_set_service");
		ExitProgram(0);
		return;
	}

	// download realtime program
	strcpy (sFilename, "..\\..\\rt\\c\\dad4smpl.exe");
	rcPrgInfo.szName	= sFilename;
	rcPrgInfo.usFlags = MAX_TASKNO_BY_USER + 
							  MAX_CALL_AUTO_INIT +
							  MAX_TASKTYPE_BY_INSTALL;
	// type of task can be selected here - change usTaskType and usIrq
	rcPrgInfo.usTaskType = MAX_II_TASK;
	rcPrgInfo.usIrq		= MAX1_IRQ_TIMERA;
	rcPrgInfo.usNo		= 0x9007;
	rcPrgInfo.usLevel = 3;
	rcPrgInfo.usUsage = 0;
	rcPrgInfo.usTask  = MEASURE_TASK;
	rcPrgInfo.ulDataSize = 0;
	Error = max_transfer_and_install(hModul, &rcPrgInfo, &MEASURE_TASK);
	if (Error) {
		ErrorHandler(Error, "max_transfer_and_install");
		ExitProgram(0);
		return;
	}
	else { 
		max_get_prog_version(hModul, MEASURE_TASK, &Version, &ulDate);
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf ("\n\rProgram dad4smpl.exe downloaded onto CPU-module, ");
		printf ("Version %s", szVersion);
	}
 
//	max_set_mdd_path("..\\..\\..\\..\\mdd"); // optionally
	// load MDD for X-DAD-4
 	Error = max_load_mdd (hModul, usADSlot, usADLayer, 0, 
		                   MDD_X_DAD_4U, NULL, &hMdd);
	if (Error) {
		ErrorHandler(Error, "max_load_mdd");
		ExitProgram(0);
		return;
	}

	max_get_mdd_version(hMdd, &Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);
	printf ("\n\rMDD for X-DAD-4 downloaded onto CPU-module, ");
	printf ("Version %s", szVersion);

	// init some parameters of the RT-program
	// select frequency in timer-tics:
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 4, 120);	// timer rate
	// select SRQs used for communication from realtime-program to PC
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 14, SRQ_DATA_READY);
	Error = max_write_par_ushort (hModul, MEASURE_TASK, 16, SRQ_RT_ERROR);  
	if (Error) {
		ErrorHandler(Error, "max_write_par_ushort");
		ExitProgram(0);
		return;
	}
	
	// create a fifo buffer in the RAM of the CPU module
	rcBufferPar.usStrategy	= MAX_ALLOC_DOWN_ABS;
	rcBufferPar.usAlignment = 2;
	rcBufferPar.usTask		= MEASURE_TASK;           
	rcBufferPar.usUsage		= 0;           
	ulSize = BUFFERSIZE;
	Error = max_create_buffer(hModul, &rcBufferPar, &ulSize, &hBuffer);
	// the buffer handle can not be transfered to the realtime-program, so 
	// use max_get_buffer_id to get the process-independent buffer number
	Error = max_get_buffer_id (hBuffer, &ulBufferID);
	if (Error) {
		ErrorHandler(Error, "max_create_buffer");
		ExitProgram(0);
		return;
	}
	// transfer buffer ID to measure-task
	Error = max_write_par_ulong (hModul, MEASURE_TASK, 6, ulBufferID); 

	// memory allocation for measured data
	plData  = (long*)malloc(BUFFERSIZE);
	pusData = (USHORT*)plData;
	if (plData == NULL) {
		ErrorHandler(ERR_MEMORY_ALLOCATION, "malloc");
		ExitProgram(0);
		return;
	}

	// open a channel to AIN-1
	rcChannelCfg.usDevice	  = DEVICE_AIN_DIFF;
   rcChannelCfg.usIndexFirst = 1;
   rcChannelCfg.usIndexLast  = rcChannelCfg.usIndexFirst;
   rcChannelCfg.usFlags		  = _CP_UNCORRECTED;
   rcChannelCfg.usReadMode	  = IO_MODE_DIRECT;
   rcChannelCfg.usWriteMode  = 0;
   rcChannelCfg.usRange		  = RANGE_BIP_10V;
   Error = max_open_channel (hMdd, sizeof(CPS_XDAD4), &rcChannelCfg, NULL, 0, &hAin);
	if (Error) {
		ErrorHandler(Error, "max_open_channel");
		ExitProgram(0);
		return;
	}

	// save data type and range (it is needed for display of the values)
	usInputRange = RANGE_BIP_10V;
	usSize = 4;
	Error = max_channel_info(hAin, INFO_DATA_TYPE, &usSize, &ulDataType);
	if (Error) {
		ErrorHandler(Error, "max_channel_info");
		ExitProgram(0);
		return;
	}
	
	// transfer the channel handle to MEASURE_TASK, handle will be set invalid in
	// this process. Function 4 of MEASURE_TASK receives the handle and converts it
	Error = max_export_channel_handle (hModul, MEASURE_TASK, 4, &hAin);
	if (Error) {
		ErrorHandler(Error, "max_export_handle");
		ExitProgram(0);
		return;
	}

	printf ("\n\rPress any key to start measurement");
	_getch();

	// start RT-measuring
	Error = max_call_proc(hModul, MEASURE_TASK, 2);
	if (Error) {
		ErrorHandler(Error, "max_call_proc");
		ExitProgram(1);
		return;
	}
	
	// now all activity is done on the X-MAX-1 and in the ReadMeasuredData procedure
	while (!_kbhit());
	
	// read out total number of acquisitions
	Error = max_read_par_ulong(hModul, MEASURE_TASK, 44, &ulDate);
	if (Error == ERR_OK)
		printf("\n\r%d analog values acquired", ulDate);
	
	// stop realtime-program	
	Error = max_call_proc(hModul, MEASURE_TASK, 3);	// stop realtime program
	ExitProgram(1);
	return;
}
