/****************************************************************************/
// Sample Real Time program
// The program measures one analog input of the X-DAD-4
// and writes the value into a buffer.
// If the buffer is filled with a certain number of values a srq
// is sent to the PC, which can then read out the values from the buffer.
// This program works together with the PC-sample hndlxchg, which
// creates the measurment-data-buffer, loads the MDD and opens the channel.
//
// Program Number 0x9007
//
// (C)2001 SORCUS Computer GmbH
/****************************************************************************/

#pragma option -r-

// _RTLIB_ must be defined in every real time program
#define _RTLIB_

#include "dos.h"
#include "stdlib.h"
#include "max_lib.h"
#include "x_mdd.h"

#define VERSION         '1'
#define REVISION        'A'

#define ERR_IN_AUTO_INIT 0x1AE0

// PDT (Program Descriptor Table) data structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
   void* MAXEXPORT start;
   void* MAXEXPORT stop;
   void  MAXEXPORT (*receive_handle) (USHORT *pusInsize, MAX_CHANNEL_TRANSFER_TYPE *prcChlData,
                                      USHORT *pusOutsize, USHORT *pDataOut);
} rcPDT;

// parameter range of the task
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;
// 0
   MAX_ERROR    Error;              // global error variable
   USHORT       usProgState;        // State of the program
   USHORT       usTimerRate;        // sampling rate
   ULONG        ulBufferID;         // process-independent ID for a buffer
// 10
   ULONG        ulBufferSize;       // size of the buffer
   USHORT       usBufferFullSRQ;    // SRQ word that is used to indicate a filled buffer
   USHORT       usErrorSRQ;         // SRQ word that is used to indicate an error
   USHORT       usReleaseSRQ;       // flag that is set while buffer is filled
                                    // less than 1/3 and reset while buffer is filled
                                    // more than 1/3
// 20
   USHORT       usHostNo;           // number of target for SRQs
   USHORT       usErrCount;         // conter for errors
   MAXBUFHND    hBuffer;            // Handle for data buffer
   MAXMODHND    hMyself;            // handle for the CPU
   MAXMODHND    hHost;              // handle for the PC
   MAXCHLHND    hAin;               // handle for the analog input channel
// 40
   ULONG        ulDataType;         // type of channel access
   ULONG        ulSampleCount;      // number of samples
   ULONG        ulValid;            // size of the buffer
   MAX_OS_INFO  rcOsX;
} parameter;

// states of the program
enum
{
   PROG_NOT_READY,
   PROG_INITIALIZED,
   PROG_READY,
   PROG_STARTED,
};

// Prototypes
void MAXEXPORT MainProc (void);
void MAXEXPORT AutoInit (void);
void MAXEXPORT ReceiveHandle(USHORT *pusInsize, MAX_CHANNEL_TRANSFER_TYPE *prcInit,
                             USHORT *pusOutsize, USHORT *pusDataOut);
void MAXEXPORT Start (void);
void MAXEXPORT Stop (void);
void ErrorHandler (USHORT usOffset);

/****************************************************************************/
// in case of error send srq to PC
void ErrorHandler(USHORT usOffset)
{
   parameter.usErrCount++;
   max_send_srq (parameter.hHost, 0, parameter.usErrorSRQ + usOffset);
}

/****************************************************************************/
// init task parameter
void MAXEXPORT AutoInit(void)
{
   max_entry_proc();
   // init parameters (some may be overwritten by the PC-program)
   parameter.usProgState      = PROG_NOT_READY;
   parameter.usTimerRate      = 38; // = 10us
   parameter.ulBufferSize     = 0;
   parameter.usBufferFullSRQ  = 0x8064;
   parameter.usErrorSRQ       = 0x8060;
   parameter.usHostNo         = 1;     // PC via PCI bridge
   parameter.ulSampleCount    = 0;
   parameter.ulValid          = 0;
   parameter.usErrCount       = 0;
   parameter.Error = max_init_lib(0);
   if (parameter.Error == ERR_OK) {
      // get a handle for the X-MAX-1
      parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,
                                         &parameter.rcOsX, &parameter.hMyself);
      if (parameter.Error == ERR_OK)
         parameter.usProgState = PROG_INITIALIZED;
   }
   max_exit_proc();
};

#pragma argsused
/****************************************************************************/
// open channels
void MAXEXPORT ReceiveHandle(USHORT *pusInsize, MAX_CHANNEL_TRANSFER_TYPE *prcInit,
                             USHORT *pusOutsize, USHORT *pusDataOut)
{
   MAX_ERROR   Error;
   ULONG       v;
   USHORT      usSize;

   max_entry_func();
   Error = ERR_OK;
   if (*pusInsize != sizeof(MAX_CHANNEL_TRANSFER_TYPE))
      Error = ERR_WRONG_NUMBER_OF_INPUTS;
   else
      if (parameter.usProgState != PROG_INITIALIZED)
         Error = ERR_IN_AUTO_INIT;
   if (Error == ERR_OK) {
      // get a handle for the host PC (to send SRQs)
      Error = max_connect_cpu (parameter.usHostNo, 0, 0, NULL, &parameter.hHost);
      // parameter.ulBufferID must be set from the process that created the buffer
      // before this task can retrieve a buffer handle
      if (Error == ERR_OK)
         Error = max_get_buffer_handle (parameter.hMyself, parameter.ulBufferID,
									             &parameter.hBuffer);
      // get buffersize
      if (Error == ERR_OK)
         Error = max_get_buffer_status(parameter.hBuffer,
					                        &v, &parameter.ulBufferSize);
      // get handle for the AIN-channel that was opened by the PC program
      if (Error == ERR_OK)
         Error = max_convert_channel_handle(prcInit, &parameter.hAin);
      if (Error == ERR_OK)
         parameter.usProgState = PROG_READY;
	   // save data type and range (it is needed for display of the values)
	   usSize = 4;
      if (Error == ERR_OK)
	      Error = max_channel_info(parameter.hAin, INFO_DATA_TYPE, &usSize, &parameter.ulDataType);
   }
   max_exit_func(Error);
}

/****************************************************************************/
// start the task
void MAXEXPORT Start(void)
{
   max_entry_proc();
   if (parameter.usProgState == PROG_READY) {
      max_clear_buffer(parameter.hBuffer);  // empty buffer
      parameter.usProgState  = PROG_STARTED;
      parameter.usReleaseSRQ = 1;
      parameter.Error = max_set_timer(parameter.hMyself,
                                      MAX1_TIMER_A, MAX1_TIMER_MODE_INT,
                                      parameter.usTimerRate);
      parameter.Error = max_wakeup_ii_task(parameter.hMyself,
                                           parameter.rcTDT.usTaskNo);
   }
   max_exit_proc();
}

/****************************************************************************/
// main procedure of the task (called automatically by OsX after starting the task)
void MAXEXPORT MainProc(void)
{
   union {
      USHORT  us;
      LONG    l;
   } Data;
	ULONG	  ulSize, ulFree;

   max_entry_proc();
   if (parameter.usProgState == PROG_STARTED) {
      // read one input channel
      if (parameter.ulDataType == DATA_USHORT) {
 		   ulSize = sizeof(USHORT);
         parameter.Error = max_read_channel_ushort(parameter.hAin, &Data.us);
      }
      else {
 		   ulSize = sizeof(LONG);
         parameter.Error = max_read_channel_long(parameter.hAin, &Data.l);
      }
      if (parameter.Error != ERR_OK)
         ErrorHandler(0);
      else
      {
		   // write value into buffer
         parameter.Error = max_set_buffer(parameter.hBuffer, MAX_BUFFER_MOVE_ABS,
   		                                 &ulSize, &Data);
         if (parameter.Error != ERR_OK)
            ErrorHandler(1);
         else {
            parameter.Error = max_get_buffer_status(parameter.hBuffer,
   											                &parameter.ulValid,
                                                    &ulFree);
            if (parameter.Error != ERR_OK)
               ErrorHandler(2);
            else {
               // check if SRQ has to be sent
               if (parameter.usReleaseSRQ) {
                  // no SRQ has been sent since buffer is filled more than 1/3
                  if (parameter.ulValid >= parameter.ulBufferSize / 3) {
                     parameter.usReleaseSRQ = 0;
                     // if buffer is filled more than 1/3 send SRQ to the host
			            parameter.Error = max_send_srq(parameter.hHost, 0,
                                                    parameter.usBufferFullSRQ);
                     if (parameter.Error != ERR_OK)
                        ErrorHandler(3);
                   }
               }
               else {
                  // SRQ has been sent already since buffer is filled more than 1/3
                  if (parameter.ulValid < parameter.ulBufferSize / 3)
                     // enable SRQs again if buffer state falls below 1/3
                     parameter.usReleaseSRQ = 1;
               }
            }
         }
         parameter.ulSampleCount++;
		}
   }
   max_exit_proc();
}

/****************************************************************************/
// stop the task
void MAXEXPORT Stop(void)
{
   max_entry_proc();
   max_sleep_task (parameter.hMyself, parameter.rcTDT.usTaskNo);
   parameter.usProgState = PROG_READY;
   max_exit_proc();
}

/****************************************************************************/
void main ()                 /* ---- PREPARE ---- */
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_II_TASK +
                                  MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = MAX1_IRQ_TIMERA;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = 0x9007;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / sizeof(void*);

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;
   rcPDT.receive_handle   = ReceiveHandle;
   rcPDT.start            = Start;
   rcPDT.stop             = Stop;
   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
