/*************************************************************************/
//
// SAMPLE01
//
// Demonstrates program initialization of a PC program that uses MAX6pci.
// Download a real-time program onto a X-MAX-1 that lets the LED blink. 
// Set some parameter in the real-time program.
// Optionally the use of Service-Requests is shown.
//
// The file maxw32.lib (from the corresponding directory MSVC or BC)
// has to be added to the project
//
/* Copyright SORCUS Computer GmbH 2001	                                     */ 
/*************************************************************************/

// History of this sample program:
// Version 1.B shows some more library features:
//		MAX_AUTO_SLOT for max_connect_cpu
//		MAX1_NEWEST_OSX for max_load_osx
//		Errorhandling is done in a separate function
//		selection between real-time-program as ii, ti or ni task
//		SRQ-handler installed

#include "windows.h"				// defines basic data types  
#include "max_lib.h"				// all definitions for the MAXW32
#include "stdio.h"				// for printf 
#include "conio.h"				// for getche and _kbhit

#define VERSION_SAMPLE	1
#define REV_SAMPLE		'B'
#define SRQ_LED_CHANGE	0x89AB

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
	USHORT	 usMsgSize = 100;
	char		 sErrMsg[200];

	max_get_error_message(Error, &usMsgSize, sErrMsg);
	printf ("\n\rError %X in %s (%s)", Error, sText, sErrMsg);
	max_clear_error();
}

/****************************************************************************/
// Procedure to handle SRQs
MAX_CALLBACK SrqServiceProcedure (MAXMODHND hModul, ULONG ulMsg)
{
	if (ulMsg == SRQ_LED_CHANGE)
		printf ("\n\rSRQ %X (Led changed state)", ulMsg);
	else
		printf ("\n\rSRQ %X received", ulMsg);
}	

/****************************************************************************/
int main ()
{
	USHORT	 usBoardNr	= 1;	// number given in the setup-tool
										// (see 'All SORCUS Boards' in the Windows control panel)
	USHORT	 usCPUSlot	= 1;	// slot of the X-MAX-1
										// (see printed number on the carrier board)
	USHORT	 usCPULayer = 0;  // layer where the X-MAX-1 is
										// (directly on the MAX6 carrier board is 0)
	MAX_ERROR	Error;
	MAXMODHND	hModul;
	MAX_OS_INFO	rcOsInfo;
	MAX_VERSION	Version;
	ULONG			ulDate; 	
	char			szVersion[12], szDate[12], szOsX[12];
	char			filename[80];
	int			key;
	MAX_PGM_TYPE rcPrgInfo;
	USHORT		usTask = 0;
	USHORT		usLEDBlinkrate;
	
	printf("\n\rMAX-Sample01 - Version %d.%c\n\r", VERSION_SAMPLE, REV_SAMPLE);
	printf("\n\rInitialize a X-MAX-1 module...\n\r");

	// Init library
	Error = max_init_lib (0);

	// check if library is initialized successfully
	if (Error != ERR_OK) {
		ErrorHandler(Error, "max_init_lib");
		return 1;
	}
	max_get_lib_version (&Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);

	// get version information
	printf("\n\rMAXW32.DLL-Library Version: %s (%s)", szVersion, szDate);
	Error = max_get_drv_version (&Version, &ulDate);
	if (Error) 
		ErrorHandler(Error, "max_get_drv_version");
	else {
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf("\n\rMAXNT4.SYS-Driver  Version: %s (%s)", szVersion, szDate);
	}
	
	// max_reset_board can be called also at any time but
	// it is only required if max_board_reacting returns an error
	Error = max_reset_board (usBoardNr, 10);
	if (Error) {
		ErrorHandler(Error, "max_reset_board");
		return 1;
	}

	// connect program to the X-MAX-1 and get a handle for the module
	// if only one CPU module is plugged in MAX_AUTO_SLOT will cause the
	// max_connect_cpu function to find that CPU module automatically
	Error = max_connect_cpu (usBoardNr, MAX_AUTO_SLOT, 0, &rcOsInfo, &hModul);
	if (Error) {
		ErrorHandler(Error, "max_connect_cpu");
		return 1;
	}

	// if the basic MINI-OSX is the active operating system,
	// activate the full ROM-OsX or load a RAM-OsX from a file
	if (rcOsInfo.usType == MAX_MINI_OSX) {
		// with MAX1_NEWEST_OSX it automatically 
		// searches for the newest OsX-file first in the actual directory
		// the in the SNW32\OSX directory
//		Error = max_load_osx (hModul, MAX1_NEWEST_OSX);
		Error = max_load_osx (hModul, "ROM");
		if (Error) {
			ErrorHandler(Error, "max_load_osx");
			return 1;
		}
	}

	// OsX Version
	Error = max_get_os_version(hModul, &rcOsInfo);
	if (Error) {
		ErrorHandler(Error, "max_get_os_version");
		return 1;
	}

	// Location of the CPU module
	Error = max_get_slot(hModul, &usCPUSlot, &usCPULayer);
	if (Error) 
		ErrorHandler(Error, "max_get_slot");
	max_get_version_string(rcOsInfo.Version, rcOsInfo.ulDate, szVersion, szDate);
	switch (rcOsInfo.usType) {
		case MAX_MINI_OSX: strcpy (szOsX, "Mini-OsX");
								 break;
		case MAX_ROM_OSX : strcpy (szOsX, "ROM-OsX");
								 break;
		case MAX_RAM_OSX : strcpy (szOsX, "RAM-OsX");
								 break;
	}
	printf("\n\rConnected to CPU on slot %X, layer %X: %s, Version=%s (%s)",
			 usCPUSlot, usCPULayer, szOsX, szVersion, szDate);

	// install SRQ handler
	Error = max_set_service(hModul, SrqServiceProcedure);
	if (Error) 
		ErrorHandler(Error, "max_set_service");

	printf("\n\r\n\rDownload a Real-Time-Program onto the module");
	printf("\n\rInstall it as NI-Task, II-Task or TI-Task ? (n/i/t) ");
	key = getche();
	if (key == 'i') {
		// parameter for installation of led blink program as II task
		strcpy (filename, "..\\..\\rt\\led_ii\\c\\led_ii.exe");
		rcPrgInfo.usNo			= 0x9001;
		usLEDBlinkrate			= 50; // divider for 100Hz base frequency
	}
	else {
		if (key == 'n') {
			// parameter for installation of led blink program as NI task
			strcpy (filename, "..\\..\\rt\\led_ni\\c\\led_ni.exe");
			rcPrgInfo.usNo			= 0x9000;
			usLEDBlinkrate			= 10000; // frequency depends on performance of the X_MAX-1
		}
		else {
			if (key != 't')
				return 1;
			// parameter for installation of led blink program as TI task
			strcpy (filename, "..\\..\\rt\\led_ti\\c\\led_ti.exe");
			rcPrgInfo.usNo			= 0x9002;
			usLEDBlinkrate			= 500;	// 1 unit = 1 ms
		}
	}

	// download selected RT-program on the X-MAX-1
	rcPrgInfo.szName		= filename;
	rcPrgInfo.usTaskType = 0;	// parameter is taken from the PDT
	rcPrgInfo.usIrq		= 0;	// parameter is taken from the PDT
	rcPrgInfo.usTask		= 50;	
	rcPrgInfo.ulDataSize = 0;	// no data range
	rcPrgInfo.usFlags		= MAX_TASKNO_BY_USER + // use the given task number
								  MAX_CALL_AUTO_INIT;  // call procedure 1	
	rcPrgInfo.usLevel		= 3;	
	rcPrgInfo.usUsage		= 0;
  	Error = max_transfer_and_install(hModul, &rcPrgInfo, &usTask);
	if (Error) {
		ErrorHandler(Error, "max_transfer_and_install");
		return 1;
	}
	
	// version of the RT program
	Error = max_get_prog_version(hModul, usTask, &Version, &ulDate);
	if (Error)
		ErrorHandler(Error, "max_get_prog_version");
	else {
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf("\n\rProgram %d successfully installed as Task %X, Version %s", 
		       rcPrgInfo.usNo, usTask, szVersion);
	}
	
	// By calling the following functions to set parameters of the task 
	// you can change the behaviour of the Real time program:
	// change the LED blink frequency
	Error = max_write_par_ushort(hModul, usTask, 4, usLEDBlinkrate); 
	// enable the realtime-program to send a SRQ with every change of the LED
	Error = max_write_par_ushort(hModul, usTask, 6, 1); //0=disable
	// set the word that the Real Time program should use for SRQs
	Error = max_write_par_ushort(hModul, usTask, 10, SRQ_LED_CHANGE); 
	// inform RT-program to whom it should send SRQs (1=PC via PCI bus, 2=serial)
	Error = max_write_par_ushort(hModul, usTask, 12, 1); 
	if (Error) 
		ErrorHandler(Error, "max_write_par_ushort");

	printf("\n\r\n\rLet the LED on the module blink (under control of the Real-Time program)\n\r");
	// call start procedure (led starts blinking)
	Error = max_call_proc(hModul, usTask, 2);
	if (Error) {
		ErrorHandler(Error, "max_call_proc");
		return 1;
	}
	printf("\n\rLED on the X-MAX-1 is blinking..."); 
	printf("\n\rPress any key to end the program...");
#ifdef __BORLANDC__
	while (!kbhit());
#else
	while (!_kbhit());
#endif
	// call stop procedure (led stops blinking)
	Error = max_call_proc(hModul, usTask, 3);
	if (Error) {
		ErrorHandler(Error, "max_call_proc");
		return 1;
	}

	// exit library
	max_exit_lib();
	return 0;
}