{*************************************************************************

 PC_INIT

 Demonstrates program initialization of a PC program that uses MAX6pci.

 Download a real-time program onto a X-MAX-1 that lets the LED blink.

 The program was generated and tested with Delphi 5.0.

 Copyright SORCUS Computer GmbH 2001
*************************************************************************}

unit Init;

interface

{ the program uses the following SORCUS units
  Maxlib   - all functions and procedure declarations
  X_error  - defined error message constants
  maxtype  - all type definitions used in the library
  maxdefs  - various constant definitions
  maxmdd   - needed for MDDs: It defines CPS-types and constants
}
uses
  Windows, Maxlib, X_error, maxtype, maxdefs, maxmdd,
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  ExtCtrls;

type
   TStartForm = class(TForm)
   StartButton   : TButton;
   ExitButton    : TButton;
   RTStart       : TButton;
   RTStop        : TButton;
   TaskSelect    : TRadioGroup;
   NIRadioButton : TRadioButton;
   IIRadioButton : TRadioButton;
   TIRadioButton : TRadioButton;
   Info          : TMemo;
   SRQs          : TMemo;
   RTCtrlGroupBox: TGroupBox;
   procedure StartButtonClick(Sender: TObject);
   procedure NIRadioButtonClick(Sender: TObject);
   procedure IIRadioButtonClick(Sender: TObject);
   procedure TIRadioButtonClick(Sender: TObject);
   procedure RTStartClick(Sender: TObject);
   procedure ExitButtonClick(Sender: TObject);
   procedure RTStopClick(Sender: TObject);
   private
   { Private-Deklarationen }
   public
   { Public-Deklarationen }
end;

var
   StartForm             : TStartForm;
   usBoardNr             : WORD;       { Board number according to installation }
   usCPUSlot, usCPULayer : WORD;       { Location of the X-MAX-1 module }
   hModul                : MAXMODHND;  { Handle for the X-MAX-1 CPU module }
   usTask                : WORD;       { tasknumber of the real-time program }
   usLEDBlinkrate        : WORD;       { blink frequency of the MAX-PC led }

const
   { Os that should be activated }
   OsX                   : Array[0..79] of char = 'ROM'; { OsX that should be activated on the X-MAX-1 }
   VERSION_SAMPLE        : WORD = 1;
   REV_SAMPLE            : char = 'C';
   SRQ_LED_CHANGE        : WORD = $89AB;

implementation

{$R *.DFM}

{******************************************************************************}
{** Subroutine for error display and error reset                             **}
{******************************************************************************}
procedure ErrorMessage (Error: MAX_ERROR; const location : PCHAR);
var
   usMsgSize : WORD;
   text      : string;
   szErrMsg  : Array [0..200] of char;
begin
   usMsgSize := 200;
   max_get_error_message (Error, usMsgSize, @szErrMsg[0]);
   text := 'Error ' + IntToHex(Error, 4) + ' in ' + string(location) + ' (' +  string(szErrMsg) + ')';
   MessageDlg(text, mtError, [mbYes], 0);
   max_clear_error();
end;

{******************************************************************************}
{** Callback-Subroutine for SRQs                                             **}
{******************************************************************************}
PROCEDURE MyService (hSrqModul : MAXMODHND; Message : Word); stdcall;
begin
   StartForm.SRQs.Lines.Add('Srq ' + IntToHex(Message, 2));
end;

{******************************************************************************}
procedure TStartForm.StartButtonClick(Sender: TObject);
var
   Error         : MAX_ERROR;
   rcOsInfo      : MAX_OS_INFO;
   Version       : MAX_VERSION;
   BoardInfo     : MAX_BOARD_TYPE;
   ModuleInfo    : MAX_MODULINFO_TYPE;
   ulDate        : LONGWORD;
   szVersion     : Array [0..12] of char;
   szDate        : Array [0..12] of char;

begin
   usBoardNr := 1;

   { library initialization }
   Error := max_init_lib (0);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_init_lib');

   { display version info }
   max_get_lib_version (Version, ulDate);
   max_get_version_string (Version, ulDate, @szVersion[0], @szDate[0]);
   Info.Lines.Add('Lib Version: ' + string(szVersion) + ' (' + string(szDate) + ')' );
   Error := max_get_drv_version (Version, ulDate);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_drv_version')
   else begin
      max_get_version_string(Version, ulDate, @szVersion[0], @szDate[0]);
      Info.Lines.Add('Drv Version: ' + szVersion + ' (' + szDate + ')');
   end;

   { reset MAX6pci board and connect program to the X-MAX-1 }
   Error := max_reset_board (usBoardNr, 10);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_reset_board');
      exit;
   end;

   { connect program to the X-MAX-1 and get a handle for the module
     if only one CPU module is plugged in MAX_AUTO_SLOT will cause the
     max_connect_cpu function to find that CPU module automatically }
   Error := max_connect_cpu (usBoardNr, MAX_AUTO_SLOT, 0, rcOsInfo, hModul);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_connect_cpu');
      exit;
   end;

   { download a operating system onto the X-MAX-1 (optional) }
   if (rcOsInfo.usType = MAX_MINI_OSX) then begin
      { MINI_OSX supports only a subset of the library functions - so use a full OsX }
      Error := max_load_osx (hModul, OsX);
      if (Error <> ERR_OK) then begin
         ErrorMessage (Error, 'max_load_osx');
         exit;
      end;
      Error := max_get_os_version(hModul, rcOsInfo);
      if (Error <> ERR_OK) then begin
         ErrorMessage (Error, 'max_get_os_version');
         exit;
      end;
   end;
   Error := max_get_slot(hModul, usCPUSlot, usCPULayer);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_get_slot');
      exit;
   end;
   Info.Lines.Add('Connected to CPU on Slot ' + IntToHex(usCPUSlot, 2)
                   + ', Layer ' + IntToHex(usCPULayer, 2));
   max_get_version_string (rcOsInfo.Version, rcOsInfo.ulDate, @szVersion[0], @szDate[0]);
   Info.Lines.Add('OsX Version: ' + string(szVersion) + ' (' + string(szDate) + ')' );

   { display info about the MAX6pci board and the X-MAX-1 }
   Error := max_get_board_info(usBoardNr, BoardInfo);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_board_info');
   Info.Lines.Add('Board ' +  IntToHex(usBoardNr, 2) + ' Type ' + IntToHex(BoardInfo.usType, 4) +
                            ' Version ' + IntToHex(BoardInfo.usVersion, 2) + ' Serial No. ' + IntToStr(BoardInfo.ulSerNr));
   Error := max_get_module_info(usBoardNr, usCPUSlot, usCPULayer, ModuleInfo);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_module_info');
   Info.Lines.Add('CPU-Module on slot' +  IntToStr(usCPUSlot) + ' Type ' + IntToHex(ModuleInfo.usType, 2) +
                            ' Version ' + IntToHex(ModuleInfo.usVersion, 2) + ' Variant ' + IntToHex(ModuleInfo.usVariant, 2)
                             + ' Category ' + IntToHex(ModuleInfo.usVariant, 2));

   { install a SRQ service procedure that is called via callback by the library }
   Error := max_set_service(hModul, @MyService);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_set_service');

   { control of windows control elements }
   ExitButton.Enabled    := TRUE;
   TaskSelect.Enabled    := TRUE;
   NIRadioButton.Enabled := TRUE;
   IIRadioButton.Enabled := TRUE;
   TIRadioButton.Enabled := TRUE;
   StartButton.Enabled   := FALSE;
end;

{******************************************************************************}
procedure TStartForm.ExitButtonClick(Sender: TObject);
begin
   max_sleep_task(hModul, usTask);
   max_exit_lib;
   Application.Terminate;
end;

{******************************************************************************}
procedure TStartForm.NIRadioButtonClick(Sender: TObject);
var
   Error       : MAX_ERROR;
   Version     : MAX_VERSION;
   ulDate      : LONGWORD;
   szVersion   : array [0..11] of char;
   szDate      : array [0..11] of char;
   rcPrgInfo   : MAX_PGM_TYPE;
const
   filename    : array [0..79] of char = '..\..\rt\led_ni\pas\led_ni.exe';
begin
   rcPrgInfo.usNo        := $9000;
   rcPrgInfo.szName      := filename;
   rcPrgInfo.usTaskType  := 0;               { parameter is taken from the PDT }
   rcPrgInfo.usIrq       := 0;               { parameter is taken from the PDT }
   rcPrgInfo.usTask      := 50;
   rcPrgInfo.ulDataSize  := 0;               { no data range }
   rcPrgInfo.usFlags     := MAX_TASKNO_BY_USER + { use the given task number }
                            MAX_CALL_AUTO_INIT;  { call procedure 1          }
   rcPrgInfo.usLevel     := 3;
   rcPrgInfo.usUsage     := 0;
   Error := max_transfer_and_install (hModul, rcPrgInfo, usTask);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_transfer_and_install');
      exit;
   end;

   { version of the RT program }
   Error := max_get_prog_version (hModul, usTask, Version, ulDate);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_prog_version')
   else begin
      max_get_version_string (Version, ulDate, @szVersion[0], @szDate[0]);
      Info.Lines.Add('Program ' + IntToStr(rcPrgInfo.usNo) + ' installed as task ' +
                      IntToStr(usTask) + ', Version: ' + string(szVersion));
   end;
   usLEDBlinkrate := 10000;  { frequency depends on performance of the X_MAX-1 }

   RTStart.Enabled       := TRUE;
   TaskSelect.Enabled    := FALSE;
   NIRadioButton.Enabled := FALSE;
   IIRadioButton.Enabled := FALSE;
   TIRadioButton.Enabled := FALSE;
end;

{******************************************************************************}
procedure TStartForm.IIRadioButtonClick(Sender: TObject);
var
   Error       : MAX_ERROR;
   Version     : MAX_VERSION;
   ulDate      : LONGWORD;
   szVersion   : array [0..11] of char;
   szDate      : array [0..11] of char;
   rcPrgInfo   : MAX_PGM_TYPE;
const
   filename    : array [0..79] of char = '..\..\rt\led_ii\pas\led_ii.exe';
begin
   rcPrgInfo.usNo        := $9001;
   rcPrgInfo.szName      := filename;
   rcPrgInfo.usTaskType  := 0;               { parameter is taken from the PDT }
   rcPrgInfo.usIrq       := 0;               { parameter is taken from the PDT }
   rcPrgInfo.usTask      := 50;
   rcPrgInfo.ulDataSize  := 0;               { no data range }
   rcPrgInfo.usFlags     := MAX_TASKNO_BY_USER + { use the given task number }
                            MAX_CALL_AUTO_INIT;  { call procedure 1 }
   rcPrgInfo.usLevel     := 3;
   rcPrgInfo.usUsage     := 0;
   Error := max_transfer_and_install (hModul, rcPrgInfo, usTask);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_transfer_and_install');
      exit;
   end;

   { version of the RT program }
   Error := max_get_prog_version (hModul, usTask, Version, ulDate);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_prog_version')
   else begin
      max_get_version_string (Version, ulDate, @szVersion[0], @szDate[0]);
      Info.Lines.Add('Program ' + IntToStr(rcPrgInfo.usNo) + ' installed as task ' +
                      IntToStr(usTask) + ', Version: ' + string(szVersion));
   end;
   usLEDBlinkrate := 50;

   RTStart.Enabled       := TRUE;
   TaskSelect.Enabled    := FALSE;
   NIRadioButton.Enabled := FALSE;
   IIRadioButton.Enabled := FALSE;
   TIRadioButton.Enabled := FALSE;
end;

{******************************************************************************}
procedure TStartForm.TIRadioButtonClick(Sender: TObject);
var
   Error       : MAX_ERROR;
   Version     : MAX_VERSION;
   ulDate      : LONGWORD;
   szVersion   : array [0..11] of char;
   szDate      : array [0..11] of char;
   rcPrgInfo   : MAX_PGM_TYPE;
const
   filename    : array [0..79] of char = '..\..\rt\led_ti\pas\led_ti.exe';
begin
   rcPrgInfo.usNo        := $9002;
   rcPrgInfo.szName      := filename;
   rcPrgInfo.usTaskType  := 0;               { parameter is taken from the PDT }
   rcPrgInfo.usIrq       := 0;               { parameter is taken from the PDT }
   rcPrgInfo.usTask      := 50;
   rcPrgInfo.ulDataSize  := 0;               { no data range }
   rcPrgInfo.usFlags     := MAX_TASKNO_BY_USER + { use the given task number }
                            MAX_CALL_AUTO_INIT;  { call procedure 1 }
   rcPrgInfo.usLevel     := 3;
   rcPrgInfo.usUsage     := 0;
   Error := max_transfer_and_install (hModul, rcPrgInfo, usTask);
   if (Error <> ERR_OK) then begin
      ErrorMessage (Error, 'max_transfer_and_install');
      exit;
   end;

   { version of the RT program }
   Error := max_get_prog_version (hModul, usTask, Version, ulDate);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_get_prog_version')
   else begin
      max_get_version_string (Version, ulDate, @szVersion[0], @szDate[0]);
      Info.Lines.Add('Program ' + IntToStr(rcPrgInfo.usNo) + ' installed as task ' +
                      IntToStr(usTask) + ', Version: ' + string(szVersion));
   end;
   usLEDBlinkrate := 500;

   RTStart.Enabled       := TRUE;
   TaskSelect.Enabled    := FALSE;
   NIRadioButton.Enabled := FALSE;
   IIRadioButton.Enabled := FALSE;
   TIRadioButton.Enabled := FALSE;
end;

{******************************************************************************}
procedure TStartForm.RTStartClick(Sender: TObject);
var
   Error : MAX_ERROR;
begin
   { By calling the following functions to set parameters of the task
     you can change the behaviour of the Real time program:
     change the LED blink frequency }
   Error := max_write_par_ushort(hModul, usTask, 4, usLEDBlinkrate);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_write_par_ushort');
   { enable the realtime-program to send a SRQ with every change of the LED }
   Error := max_write_par_ushort(hModul, usTask, 6, 1); { 0=disable }
   { set the word that the Real Time program should use for SRQs }
   Error := max_write_par_ushort(hModul, usTask, 10, SRQ_LED_CHANGE);
   { inform RT-program to whom it should send SRQs (1=PC via PCI bus, 2=serial) }
   Error := max_write_par_ushort(hModul, usTask, 12, 1);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_write_par_ushort');
   { call start procedure (led starts blinking) }
   Error := max_call_proc(hModul, usTask, 2);
   if (Error <> ERR_OK) then
      ErrorMessage (Error, 'max_call_proc');
   RTStop.Enabled := TRUE;
end;

{******************************************************************************}
procedure TStartForm.RTStopClick(Sender: TObject);
begin
   max_sleep_task(hModul, usTask);
   RTStart.Enabled := TRUE;
end;

end.