/****************************************************************************/
// Sample Real Time program (NI-Task)
//
// The program lets the LED on the X-MAX-1 blink
// It can also send a SRQ to the PC with every change of the LED
//
// Program Number 9000h
//
// (C)2000 SORCUS Computer GmbH
/****************************************************************************/

#pragma option -r-

#define _RTLIB_                        // must be defined in every real time
                                       // program before including max_lib.h
#include "max_lib.h"
#include "x_error.h"
#include "dos.h"
#include "stdlib.h"

#define PROGRAMM_NUMBER 0x9000
#define VERSION         '1'
#define REVISION        'C'
#define SRQ_ERROR       0x8182

// PDT structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
   // any number of user defined procedures may follow (optionally)
   void* MAXEXPORT start;
   void* MAXEXPORT stop;
} rcPDT;

// parameter range structure
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;                 // always before the first parameter !
// offset 0
   MAX_ERROR    Error;                 // Errorcode
   USHORT       usLedState;            // Actual state of the LED
   USHORT       usBlinkrate;           // Controls of the blinking frequency
   USHORT       usSendSRQs;            // Control if SRQ should be sent with every
                                       // change of the LED state
   USHORT       usCounter;             // Actual Frequency counter
// offset 10
   USHORT       usSRQMsg;              // SRQ message to be used if usSendSRQs=1
   USHORT       usHostNo;              // number of the target (PC) for srqs
   MAXMODHND    hMyself;               // Handle for the CPU module
   MAXMODHND    hHost;                 // Handle for the PC (to send SRQs)
// offset 20
   MAX_OS_INFO  rcOsX;                 // Version info about OsX
} parameter;

/****************************************************************************/
// in case of error send srq to PC
// this is a local procedure - no max_entry / max_exit must be called
void ErrorHandler(void)
{
   parameter.Error = max_send_srq (parameter.hHost, 0, SRQ_ERROR);
}

/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
void MAXEXPORT MainProc(void)
{
   max_entry_proc();
   if (++parameter.usCounter >= parameter.usBlinkrate) {
      if (parameter.usLedState == 0) {
         max_led_on(parameter.hMyself);
         parameter.usLedState = 1;
      }
      else {
         max_led_off(parameter.hMyself);
         parameter.usLedState = 0;
      } 
      parameter.usCounter = 0;
      // transmit srq to the host pc if parameter usSendSRQs is set
      if (parameter.usSendSRQs)
         max_send_srq (parameter.hHost, 0, parameter.usSRQMsg);
   }
   max_exit_proc();
}

/****************************************************************************/
// Init task parameter
void MAXEXPORT AutoInit(void)
{
   max_entry_proc();
   parameter.usBlinkrate = 8000;
   parameter.usCounter   = 0;
   parameter.usSendSRQs  = 0;
   parameter.usSRQMsg    = 0x9190;
   parameter.usHostNo    = 1;          // number of target for SRQs
   parameter.Error = max_init_lib(0);
   if (parameter.Error == ERR_OK) {
      // get a handle for the X-MAX-1
      parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,
                                         &parameter.rcOsX, &parameter.hMyself);
      if (parameter.Error == ERR_OK) {
         max_led_off(parameter.hMyself);
         parameter.usLedState = 0;          // Led is switched off
      }
   }
   max_exit_proc();
};

/****************************************************************************/
// start the task
void MAXEXPORT Start(void)
{
   USHORT      usTask;

   usTask = max_entry_proc();
   // get a handle for the host PC (to send SRQs via PCI-bus)
   parameter.Error = max_connect_cpu (parameter.usHostNo, 0, 0, NULL, &parameter.hHost);
   if (parameter.Error == ERR_OK)
      parameter.Error = max_wakeup_ni_task (parameter.hMyself, usTask, 1);
   max_exit_proc();
}

/****************************************************************************/
// stop the task
void MAXEXPORT Stop(void)
{
   USHORT usTask;

   usTask = max_entry_proc();
   parameter.Error = max_sleep_task (parameter.hMyself, usTask);
   max_exit_proc();
}

/****************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_NI_TASK +
                                  MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = 0;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;
   rcPDT.start            = Start;
   rcPDT.stop             = Stop;

   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
