{******************************************************************************
   Sample Real Time program
   The program lets the LED on the X-MAX-1 blink
   It can also send a SRQ to the PC with every change of the LED
   Program Number 9000h

   (C)2000 SORCUS Computer GmbH
{******************************************************************************}

{ Compiler settings }
{$F+}                                     { FAR Calls ! }
{$N-}                                     { No Coprocessor }
{$E-}                                     { No Emulation (application specific) }
{$R-}                                     { No Range-Check }
{$S-}                                     { No Stack-Checking }
{$I-}                                     { No IO-Checking }
{$V-}                                     { No strict vat string check}

{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

uses
   Maxlib, X_error, maxtype, maxdefs, maxmdd;
{ the program uses the following SORCUS units
  Maxlib   - all functions and procedure declarations
  X_error  - defined error message constants
  maxtype  - all type definitions used in the library
  maxdefs  - various constant definitions
  maxmdd   - needed for MDDs: It defines CPS-types and constants
}

{ constant definitions }
const
   PROGRAMM_NUMBER : WORD = $9000;
   VERSION   : char       = '1';
   REVISION  : char       = 'D';
   SRQ_ERROR : WORD       = $8182;

type
   { program descriptor table }
   pdt_type = RECORD
      rcHead    : MAX_PDT_HEAD;        { standard PDT header }
      main_proc : POINTER;             { pointer to proc. 0  }
      auto_init : POINTER;             { pointer to proc. 1  }
      start     : POINTER;             { pointer to proc. 2 (optional) }
      stop      : POINTER;             { pointer to proc. 3 (optional) }
   end;

   { parameter range (is part of the program
     because PDT-flag MAXPDT_PARAMETER_IN_PROG is set) }
   parameter_type = RECORD
      rcTDT       : MAX_TDT_TYPE ;     { must be before the parameter range }
{ Offset 0: }
      Error       : MAX_ERROR;
      usLedState  : WORD;
      usBlinkrate : WORD;
      usSendSRQs  : WORD;
      usCounter   : WORD;
{ Offset 10: }
      usSRQMsg    : WORD;
      usHostNo    : WORD;
      hMyself     : MAXMODHND;
      hHost       : MAXMODHND;
{ Offset 20: }
      rcOsX       : MAX_OS_INFO;
   end;

{ global variables }
var
   rcPDT               : pdt_type;
   parameter           : parameter_type;

{****************************************************************************}
{ in case of error send srq to PC                                            }
PROCEDURE ErrorHandler;
begin
   parameter.Error := max_send_srq (parameter.hHost, 0, SRQ_ERROR);
end;

{****************************************************************************}
{ Init task parameter                                                        }
PROCEDURE AutoInit; far;
begin
   max_entry_proc;
   parameter.usBlinkrate := 8000;
   parameter.usCounter   := 0;
   parameter.usSendSRQs  := 0;
   parameter.usSRQMsg    := $9190;
   parameter.usHostNo    := 1;         { PC connected via PCI }
   parameter.Error       := max_init_lib(0);
   if (parameter.Error = ERR_OK) then begin
      { get a handle for the X-MAX-1 }
      parameter.Error := max_connect_cpu (0, MAX_MYSELF, 0,
                                          parameter.rcOsX, parameter.hMyself);
      if (parameter.Error = ERR_OK) then
         parameter.Error := max_led_off (parameter.hMyself);
      parameter.usLedState := 0;       { Led is switched off }
   end;
   max_exit_proc;
end;

{****************************************************************************}
{ main procedure of the task (called automatically by OsX after activating the task) }
PROCEDURE MainProc; far;
begin
   max_entry_proc;
   Inc(parameter.usCounter);
   if (parameter.usCounter > parameter.usBlinkrate) then begin
      if (parameter.usLedState = 0) then begin
         max_led_on(parameter.hMyself);
         parameter.usLedState := 1;
      end
      else begin
         max_led_off(parameter.hMyself);
         parameter.usLedState := 0;
      end;
      parameter.usCounter := 0;
      { transmit srq to the host pc if parameter usSendSRQs is set }
      if (parameter.usSendSRQs <> 0) then
         max_send_srq (parameter.hHost, 0, parameter.usSRQMsg);
   end;
   max_exit_proc;
end;

{****************************************************************************}
{ Start the task                                                              }
PROCEDURE Start; far;
var
   usTask : WORD;
   Dummy  : MAX_OS_INFO;
begin
   usTask := max_entry_proc;
   { get a handle for the host PC (to send SRQs) }
   parameter.Error := max_connect_cpu (parameter.usHostNo, 0, 0, Dummy, parameter.hHost);
   parameter.Error := max_wakeup_ni_task (parameter.hMyself, usTask, 1);
   max_exit_proc;
end;

{****************************************************************************}
{ Stop the task                                                              }
PROCEDURE Stop; far;
var
  usTask : WORD;
begin
   usTask := max_entry_proc;
   parameter.Error := max_sleep_task (parameter.hMyself, usTask);
   max_exit_proc;
end;

{****************************************************************************}

begin
   { the PDT-flags determine the task-type and
    information how the data and parameter range are allocated }
   rcPDT.rcHead.usFlags         := MAX_NI_TASK +
                                   MAXPDT_PARAMETER_IN_PROG; { parameter are part of the program }

   { interrupt source (0 for NI-tasks) }
   rcPDT.rcHead.usInterrupt     := 0;

   { the program number identifies the program uniquely }
   rcPDT.rcHead.usProgNo        := PROGRAMM_NUMBER;

   { number of global task procedures in the PDT }
   rcPDT.rcHead.usProcedures    := (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) div 4;

   { the following lines are just for documentation and information purpose }
   rcPDT.rcHead.ucType          := 1;   { always ! }
   rcPDT.rcHead.ucSize          := sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        := VERSION;
   rcPDT.rcHead.cRevision       := REVISION;
   rcPDT.rcHead.ucCPU           := MAX_CPU486;
   rcPDT.rcHead.ucCoProc        := _NOCOPROC;
   rcPDT.rcHead.ucLanguage      := MAXRT_LANGUAGE_PAS;
   rcPDT.rcHead.ucProgType      := 0;   { not used }
   rcPDT.rcHead.ulRes1          := 0;
   rcPDT.rcHead.ulRes2          := 0;
   rcPDT.rcHead.ulHyperAdr      := 0;   { feature not available yet }

   { address and size of the tasks data range
     only when MAXPDT_DATA_IN_PROG is set in PDT-flags }
   rcPDT.rcHead.ulDataAdr       := 0;
   rcPDT.rcHead.ulDataSize      := 0;

   { address and size of the tasks parameter range
     only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags }
   rcPDT.rcHead.ulParAdr  := max_get_physical_addr(@parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize := sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   { initialize global procedure addresses in PDT }
   rcPDT.main_proc        := @MainProc;
   rcPDT.auto_init        := @AutoInit;
   rcPDT.start            := @Start;
   rcPDT.stop             := @Stop;
   { transfer PDT information to the OsX }
   max_init_program (@rcPDT);
end.