/****************************************************************************/
/*                                                                          */
/* Sample for usage of a MAX6pci as non-intelligent board.                  */
/*                                                                          */
/* The program demonstrates how to initialize the board                     */
/*                          how to load a PC-based Module-Device-Driver     */
/*                          how to open a MDD channel                       */
/*                          how to read analog values from the channel      */
/*                                                                          */
/* If the data acquisition performance is not fast enough or not the        */
/* real time capability is insufficent, plug a X-MAX-1 CPU module onto      */
/* the board and see the sample buf_dacq                                    */
/*                                                                          */
/* Copyright SORCUS Computer GmbH 2001                                      */
/****************************************************************************/

#include "windows.h"
#include "stdio.h"
#include "conio.h"
#include "max_lib.h"
#include "x_mdd.h"

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
	USHORT	 usMsgSize = 100;
	char		 sErrMsg[200];

	max_get_error_message(Error, &usMsgSize, sErrMsg);
	printf ("\n\rError %04Xh in %s (%s)", Error, sText, sErrMsg);
//	_getch();									// user has to confirm
	max_clear_error();
}

/****************************************************************************/
void main () 
{
	MAX_ERROR		Error;
	MAX_OS_INFO		rcOsInfo;
	MAX_VERSION		Version;
	ULONG				ulDate, ulSize;
	char				szVersion[12], szDate[12];
	CPS_XDAD4		rcChannelCfg;
	MAXCHLHND		hAin, hDin; 	
	USHORT			usBoardNr;					
	USHORT			usADSlot, usADLayer;		
	MAXMODHND		hModul;
	MAXMDDHND		hMdd;
	LONG				alData[4];
	double			fpValue;
	USHORT			usChannel, usData;

	usBoardNr = 1;									// see contol panel 'SORCUS boards'
	usADSlot  = 5;									// Location of the X-DAD-4 module
	usADLayer = 0;

	printf ("Sample program : Data acquisition with X-DAD-4");
	Error = max_init_lib (0);
	// display some information about the system
	max_get_lib_version (&Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);
	printf ("\n\rLib Version: %s (%s)", szVersion, szDate);
	Error = max_get_drv_version (&Version, &ulDate);
	if (Error) {
		printf ("Error %X", Error);
		max_clear_error();
	}
	else {
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf ("\n\rDriver Version: %s (%s)", szVersion, szDate);
	}

	// reset the board
	Error = max_reset_board (usBoardNr, 20);
	if (Error) {
		ErrorHandler(Error, "max_reset_board");
		max_exit_lib();
		return;
	}
	
	// get a cpu handle
	Error = max_connect_cpu (usBoardNr, 0, 0, &rcOsInfo, &hModul);
	if (Error) {
		ErrorHandler(Error, "max_connect_cpu");
		max_exit_lib();
		return;
	}
 
	// load MDD for X-DAD-4
 	Error = max_load_mdd (hModul, usADSlot, usADLayer, 0, 
		                   MDD_X_DAD_4U, NULL, &hMdd);
	if (Error) {
		ErrorHandler(Error, "max_load_mdd");
		max_exit_lib();
		return;
	}
	max_get_mdd_version(hMdd, &Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);
	printf ("\n\rMDD for X-DAD-4 loaded, Version %s", szVersion);

	// open a channel to all AINs
	rcChannelCfg.usDevice	  = DEVICE_AIN_DIFF;
   rcChannelCfg.usIndexFirst = 0;
   rcChannelCfg.usIndexLast  = 3;
   rcChannelCfg.usFlags		  = _CP_UNCORRECTED;
   rcChannelCfg.usReadMode	  = IO_MODE_DIRECT;
   rcChannelCfg.usWriteMode  = 0;
   rcChannelCfg.usRange		  = RANGE_BIP_10V;
   Error = max_open_channel (hMdd, sizeof(CPS_XDAD4), &rcChannelCfg, NULL, 0, &hAin);
	if (Error) {
		ErrorHandler(Error, "max_open_channel to AIN");
		max_exit_lib();
		return;
	}

	// open a channel to all DINs
	rcChannelCfg.usDevice	  = DEVICE_DIN;
   rcChannelCfg.usIndexFirst = 0;
   rcChannelCfg.usIndexLast  = 16;
   rcChannelCfg.usFlags		  = _CP_EXCLUSIVE;
   rcChannelCfg.usReadMode	  = IO_MODE_DIRECT;
   rcChannelCfg.usWriteMode  = 0;
   Error = max_open_channel (hMdd, sizeof(CPS_XDAD4), &rcChannelCfg, NULL, 0, &hDin);
	if (Error) {
		ErrorHandler(Error, "max_open_channel to DIN");
		max_exit_lib();
		return;
	}

	printf ("\n\rPress any key to start measurement");
	_getch();

	// Read channels until a key is pressed...
	while (!_kbhit())
	{
		ulSize = 4 * sizeof(LONG);
      Error = max_read_channel_block (hAin, &ulSize, alData);
		if (Error) {
			ErrorHandler(Error, "max_read_channel_block");
			break;
		}
		printf("\n\r");
		for (usChannel=0; usChannel<4; usChannel++) 
		{
			fpValue = (double)alData[usChannel] / 1000000;
			printf("Ain%d=%+.3fV  ", usChannel, fpValue);
		}
      Error = max_read_channel_ushort (hDin, &usData);
		if (Error) {
			ErrorHandler(Error, "max_read_channel_ushort");
			break;
		}
		printf("Din %04X", usData);
		Sleep(20);
	}
	
	max_exit_lib();
	return;
}
