{****************************************************************************}
{*                                                                          *}
{* Sample for usage of a MAX6pci as non-intelligent board.                  *}
{*                                                                          *}
{* The program demonstrates how to initialize the board                     *}
{*                          how to load a PC-based Module-Device-Driver     *}
{*                          how to open a MDD channel                       *}
{*                          how to read analog values from the channel      *}
{*                                                                          *}
{* If the data acquisition performance is not fast enough or not the        *}
{* real time capability is insufficent, plug a X-MAX-1 CPU module onto      *}
{* the board and see the sample buf_dacq                                    *}
{*                                                                          *}
{* Copyright SORCUS Computer GmbH 2001                                      *}
{****************************************************************************}

unit Unit1;

interface

uses
  Windows, Maxlib, X_error, maxtype, maxdefs, maxmdd, StdCtrls,
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TForm1 = class(TForm)
    InitButton: TButton;
    Info: TMemo;
    Measure: TButton;
    DataOutput: TMemo;
    procedure InitButtonClick(Sender: TObject);
    procedure MeasureClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;
  hAin, hDin          : MAXCHLHND;

implementation

{$R *.DFM}

{******************************************************************************}
{** Subroutine for error display and error reset                             **}
{******************************************************************************}
procedure ErrorHandler (Error: MAX_ERROR; const location : PCHAR);
var
   usMsgSize : WORD;
   text      : string;
   szErrMsg  : Array [0..200] of char;
begin
   usMsgSize := 200;
   max_get_error_message (Error, usMsgSize, @szErrMsg[0]);
   text := 'Error ' + IntToHex(Error, 4) + ' in ' + string(location) + ' (' +  string(szErrMsg) + ')';
   MessageDlg(text, mtError, [mbYes], 0);
   max_clear_error();
end;

{******************************************************************************}
{** InitButtonClick : Initialization of the board and the channels           **}
{******************************************************************************}
procedure TForm1.InitButtonClick(Sender: TObject);
var
   rcPrgInfo           : MAX_PGM_TYPE;
   Error               : MAX_ERROR;
   rcOsInfo            : MAX_OS_INFO;
   Version             : MAX_VERSION;
   ulDate              : LONGWORD;
   szVersion, szDate   : ARRAY[0..12] of char;
   rcChannelCfg        : CPS_XDAD4;
   usBoardNr, usADSlot, usADLayer : WORD;
   hModul              : MAXMODHND;
   hMdd                : MAXMDDHND;

begin

   usBoardNr := 1;									// see contol panel 'SORCUS boards'
   usADSlot  := 5;									// Location of the X-DAD-4 module
   usADLayer := 0;

   Info.Lines.Add ('Sample program : Data acquisition with X-DAD-4');
   max_init_lib (0);

   { display some information about the system }
   max_get_lib_version (Version, ulDate);
   max_get_version_string (Version, ulDate, @szVersion[0], @szDate[0]);
   Info.Lines.Add('Lib Version: ' + string(szVersion) + ' (' + string(szDate) + ')' );
   Error := max_get_drv_version (Version, ulDate);
   if (Error <> ERR_OK) then
      ErrorHandler (Error, 'max_get_drv_version')
   else begin
      max_get_version_string(Version, ulDate, @szVersion[0], @szDate[0]);
      Info.Lines.Add('Drv Version: ' + szVersion + ' (' + szDate + ')');
   end;

   { reset the board }
   Error := max_reset_board (usBoardNr, 20);
   if (Error <> 0) then begin
      ErrorHandler(Error, 'max_reset_board');
      max_exit_lib;
      exit;
   end;

   { get a cpu handle }
   Error := max_connect_cpu (usBoardNr, 0, 0, rcOsInfo, hModul);
   if (Error <> 0) then begin
      ErrorHandler(Error, 'max_connect_cpu');
      max_exit_lib;
      exit;
   end;

   { load MDD for X-DAD-4 }
   Error := max_load_mdd (hModul, usADSlot, usADLayer, 0,
                          MDD_X_DAD_4U, rcPrgInfo, hMdd);
   if (Error <> 0) then begin
      ErrorHandler(Error, 'max_load_mdd');
      max_exit_lib;
      exit;
   end;

   max_get_mdd_version(hMdd, Version, ulDate);
   max_get_version_string(Version, ulDate, @szVersion[0], @szDate[0]);
   Info.Lines.Add ('MDD for X-DAD-4 loaded, Version ' + szVersion);

   { open a channel to all AINs }
   rcChannelCfg.usDevice     := DEVICE_AIN_DIFF;
   rcChannelCfg.usIndexFirst := 0;
   rcChannelCfg.usIndexLast  := 3;
   rcChannelCfg.usFlags	     := _CP_UNCORRECTED;
   rcChannelCfg.usReadMode   := IO_MODE_DIRECT;
   rcChannelCfg.usWriteMode  := 0;
   rcChannelCfg.usRange	     := RANGE_BIP_10V;
   Error := max_open_channel (hMdd, sizeof(CPS_XDAD4), rcChannelCfg, NIL, 0, hAin);
   if (Error <> 0) then begin
      ErrorHandler(Error, 'max_open_channel to AIN');
      max_exit_lib;
      exit;
   end;

   { open a channel to all DINs }
   rcChannelCfg.usDevice     := DEVICE_DIN;
   rcChannelCfg.usIndexFirst := 0;
   rcChannelCfg.usIndexLast  := 16;
   rcChannelCfg.usFlags	     := _CP_EXCLUSIVE;
   rcChannelCfg.usReadMode   := IO_MODE_DIRECT;
   rcChannelCfg.usWriteMode  := 0;
   Error := max_open_channel (hMdd, sizeof(CPS_XDAD4), rcChannelCfg, NIL, 0, hDin);
   if (Error <> 0) then begin
      ErrorHandler(Error, 'max_open_channel to DIN');
      max_exit_lib;
      exit;
   end;
   InitButton.Enabled := FALSE;
   Measure.Enabled    := TRUE;
end;

{******************************************************************************}
{** MeasureClick : Read channels                                             **}
{******************************************************************************}
procedure TForm1.MeasureClick(Sender: TObject);
var
   Error               : MAX_ERROR;
   alData              : ARRAY[0..4] of LONGINT;
   fpValue             : double;
   ulSize              : LONGWORD;
   usData, usChannel   : WORD;
   line_i              : string;

begin
   ulSize := 4 * sizeof(LONGINT);
   Error := max_read_channel_block (hAin, ulSize, alData);
   if (Error <> 0) then begin
      ErrorHandler(Error, 'max_read_channel_block');
      max_exit_lib;
      exit;
   end;
   for usChannel:=0 to 3 do begin
      fpValue := alData[usChannel] / 1000000;
      line_i  := line_i + 'AIN-' + IntToStr(usChannel) + ' = ' +
                 FloatToStrF(fpValue, ffFixed, 7, 3) + 'V  ';
   end;

   Error := max_read_channel_ushort (hDin, usData);
   if (Error <> 0) then begin
      ErrorHandler(Error, 'max_read_channel_ushort');
      max_exit_lib;
      exit;
   end;
   Form1.DataOutput.Lines.Add(line_i + 'DINs ' + IntToHex(usData, 4));
end;

end.
