/** \file minican.c
 * \brief example RT program for X-CAN-2
 *
 */

#pragma option -r-

#define _RTLIB_ //!< must be defined in every real time program before including max_lib.h
#include "max_lib.h"
#include "x_error.h"
#include "x_mdd.h"
#include "dos.h"
#include "stdlib.h"
#include "string.h"
#include "minican.h"


/**
* \struct PDT_t
* \brief PDT (Program Descriptor Table)
*
* This struct contains the standard PDT entries
* and additionally a list of entry points for user
* defined functions and procedures
*/
typedef struct
{
	MAX_PDT_HEAD    rcHead;     //!< standard PDT entries
	void* MAXEXPORT main_proc;  //!< proc 0
	void* MAXEXPORT auto_init;  //!< proc 1
	void* MAXEXPORT Start;		//!< proc 2
	void* MAXEXPORT Stop;		//!< proc 3
	void* MAXEXPORT Read0;		//!< func 4
	void* MAXEXPORT Write0;		//!< func 5
	void* MAXEXPORT Read1;		//!< func 6
	void* MAXEXPORT Write1;		//!< func 7
} PDT_t;
PDT_t rcPDT; //!< instance of PDT

/**
* \struct parameter_t
* \brief task parameter range structure
*
* Task parameters can be read and written from outside of the RT task
* (see max_read/write_par_) to set parameters or to get status information
*/
typedef struct 
{
	MAX_TDT_TYPE rcTDT;                 //!< always before the first parameter!
	// offset 0
	MAX_ERROR   Error;                  //!< Error code
	// offset 2
	ULONG       ulTimeStampHi;          //!< 64 bit timestamp high word
	// offset 6
	ULONG       ulTimeStampLo;          //!< 64 bit timestamp low word
	// offset 10
	CAN_INTERFACE_PARAMETERS arcCanParameters[2]; //!< interface parameters (see \ref CAN_INTERFACE_PARAMETERS)
	// offset 50
	USHORT      usSlot;                 //!< slot of the X-CAN-2 module (layer = 0)
	// offset 52
	XCAN_BIT_PARAMS	arcBitTiming[2];    //!< detailed bit timing (read only)
	// offset 80
	CAN_INTERFACE_STATE arcIfcState[2]; //!< interface state (see \ref CAN_INTERFACE_STATE)
	// offset 104
	ULONG       ulCbFilterCount[2];     //!< count acceptance filter callbacks
	// offset 112
	ULONG       ulCbCtrlCount[2];       //!< count control callbacks
	// offset 120
	ULONG       ulTmtPending[2];        //!< count pending messages (sending increments, ACK callbacks decrement)
	MAXMDDHND   hMdd;                   //!< handle for X-CAN-2 MDD
	MAXCHLHND   hCtrl[2];               //!< handle for Ctrl channels (setting bit rate etc.)
	MAXCHLHND   hIn11[2];               //!< handle for 11 bit acceptance filter channels
	MAXCHLHND   hIn29[2];               //!< handle for 29 bit acceptance filter channels
	MAXCHLHND   hOut[2];                //!< handle for universal sender channels
	MAXBUFHND   hBuf[2];                //!< handle for RCV buffer
	MAXMODHND   hMyself;                //!< Handle for the CPU module
	MAX_OS_INFO rcOsX;                  //!< Version info about OsX   
} parameter_t;
parameter_t parameter; //!< instance of task parameters


//*****************************************************************************
// GetBitTiming
/** \brief fill in XCAN_BIT_PARAMS for a given bit rate
* @param    usInterface interface index: 0 | 1
* @param 	prcBitParam pointer to a structure of type XCAN_BIT_PARAMS
* @return   MAX_ERROR
*/
//*****************************************************************************
MAX_ERROR GetBitTiming(USHORT usInterface, XCAN_BIT_PARAMS *prcBitParam)
{
	// bit timing from bit rate
    prcBitParam->SpB       = 1;
    switch (parameter.arcCanParameters[usInterface].ulBitRate)
    {
    case 10000:
        prcBitParam->Tq    = 5000;
        prcBitParam->Tseg1 = 16;
        prcBitParam->Tseg2 = 3;
        prcBitParam->Tsjw  = 2;
		break;
    case 20000:
        prcBitParam->Tq    = 2500;
        prcBitParam->Tseg1 = 16;
        prcBitParam->Tseg2 = 3;
        prcBitParam->Tsjw  = 2;
		break;
    case 50000:
        prcBitParam->Tq    = 1000;
        prcBitParam->Tseg1 = 16;
        prcBitParam->Tseg2 = 3;
        prcBitParam->Tsjw  = 2;
		break;
    case 80000:
        prcBitParam->Tq    = 625;
        prcBitParam->Tseg1 = 16;
        prcBitParam->Tseg2 = 3;
        prcBitParam->Tsjw  = 2;
		break;
    case 83300:
        prcBitParam->Tq    = 750;
        prcBitParam->Tseg1 = 13;
        prcBitParam->Tseg2 = 2;
        prcBitParam->Tsjw  = 1;
		break;
    case 100000:
        prcBitParam->Tq    = 500;
        prcBitParam->Tseg1 = 16;
        prcBitParam->Tseg2 = 3;
        prcBitParam->Tsjw  = 2;
		break;
    case 125000:
        prcBitParam->Tq    = 500;
        prcBitParam->Tseg1 = 13;
        prcBitParam->Tseg2 = 2;
        prcBitParam->Tsjw  = 1;
		break;
    case 250000:
        prcBitParam->Tq    = 250;
        prcBitParam->Tseg1 = 13;
        prcBitParam->Tseg2 = 2;
        prcBitParam->Tsjw  = 1;
		break;
    case 500000:
        prcBitParam->Tq    = 125;
        prcBitParam->Tseg1 = 13;
        prcBitParam->Tseg2 = 2;
        prcBitParam->Tsjw  = 1;
		break;
    case 666000:
        prcBitParam->Tq    = 125;
        prcBitParam->Tseg1 = 9;
        prcBitParam->Tseg2 = 2;
        prcBitParam->Tsjw  = 1;
		break;
    case 800000:
        prcBitParam->Tq    = 125;
        prcBitParam->Tseg1 = 7;
        prcBitParam->Tseg2 = 2;
        prcBitParam->Tsjw  = 1;
		break;
    case 1000000:
        prcBitParam->Tq    = 125;
        prcBitParam->Tseg1 = 5;
        prcBitParam->Tseg2 = 2;
        prcBitParam->Tsjw  = 1;
		break;
    default:
        return ERR_INVALID_PARAMETER;
    }
	
	if (parameter.arcCanParameters[usInterface].ulRbus == 0)
		prcBitParam->Rbus      = 0;
	else
		prcBitParam->Rbus      = 1;
	
	if (parameter.arcCanParameters[usInterface].ulFormat == 11L)
		prcBitParam->MsgFormat = 11;
	if (parameter.arcCanParameters[usInterface].ulFormat == 29L)
		prcBitParam->MsgFormat = 29;
	if (parameter.arcCanParameters[usInterface].ulFormat == 31L)
		prcBitParam->MsgFormat = XCAN_MIXED_FORMAT;

	//copy to parameter space
	memcpy(&parameter.arcBitTiming[usInterface], prcBitParam, sizeof(XCAN_BIT_PARAMS));
	
	return ERR_OK;
}


//*****************************************************************************
// CbFilter
/** \brief callback from acceptance filter
* @param    hChannel	handle of the calling acceptance filter channel
* @param    ulParam		high word: interface index; low word: message format
* @param    ulSize	    size of data pointed to by prcCanMessage
* @param 	prcCanMessage	CAN message received by the acceptance filter
* @return   -
*/
//*****************************************************************************
#pragma argsused
MAX_CALLBACK CbFilter(MAXCHLHND hChannel, ULONG ulParam, ULONG ulSize, void* prcCanMessage)
{
    XCAN_MESSAGE  *prcMsg = (XCAN_MESSAGE*)prcCanMessage;
	CAN_INTERFACE_READ rcCanMessage;
	ULONG ulSizeTmp, ulValid, ulFree;
	USHORT usIndex;
	MAX_ERROR Error;
	
	//get interface index
	usIndex = (USHORT)(ulParam >> 16);

	// check parameter
	if (usIndex > 1)
		return;

	//apply timestamp
	rcCanMessage.ulTimeStampLo = parameter.ulTimeStampLo;
	rcCanMessage.ulTimeStampHi = parameter.ulTimeStampHi;

	//check if there is still free space in the buffer
	parameter.Error = max_get_buffer_status(parameter.hBuf[usIndex], &ulValid, &ulFree);
	if (ulFree < sizeof(CAN_INTERFACE_READ) * 2)
	{
		//set message size
		rcCanMessage.ulDataSize = sizeof(MAX_ERROR);
		//set invalid message id
		rcCanMessage.ulId = 0xffffffffL;
		//set invalid message format
		rcCanMessage.ulFormat = 0xffffffffL;
		//copy message data
		Error = ERR_OSX_BUFFER_FULL;
		memcpy(&rcCanMessage.aucData[0], &Error, rcCanMessage.ulDataSize);
	}
	else
	{
		//get message size
		rcCanMessage.ulDataSize = prcMsg->usDataSize;
		//get message id
		rcCanMessage.ulId = prcMsg->ulCanId;
		//get message format
		rcCanMessage.ulFormat = ulParam & 0x0000ffffL;
		//copy message data
		memcpy(&rcCanMessage.aucData[0], &(prcMsg->aucData[0]), rcCanMessage.ulDataSize);
	}

	//store message in ring buffer
	ulSizeTmp = sizeof(rcCanMessage);
    parameter.Error = max_set_buffer(parameter.hBuf[usIndex], MAX_BUFFER_MOVE_ABS, &ulSizeTmp, &rcCanMessage);

	//count message
	parameter.ulCbFilterCount[usIndex]++;
}


//*****************************************************************************
// CbControl
/** \brief callback from control channel
* @param    hChannel	handle of the calling control channel
* @param    ulParam		interface index
* @param    ulSize	    size of data pointed to by pData
* @param 	pData	    pointer to ULONG; low word: bus state; high word: error info
* @return   -
*/
//*****************************************************************************
#pragma argsused
MAX_CALLBACK CbControl(MAXCHLHND hChannel, ULONG ulParam, ULONG ulSize, void *pData)
{
    // check parameter
    if (ulParam > 1)
        return;

    // copy state
    memcpy(&parameter.arcIfcState[ulParam].usBusState, pData, sizeof(ULONG));
    // set time stamp
	parameter.arcIfcState[ulParam].ulTimeStampHi = parameter.ulTimeStampHi;
	parameter.arcIfcState[ulParam].ulTimeStampLo = parameter.ulTimeStampLo;

	//count callback
	parameter.ulCbCtrlCount[ulParam]++;
}


//*****************************************************************************
// CbAcknowledge
/** \brief callback from universal sender
* @param    hChannel	handle of the calling send channel
* @param    ulParam		interface index
* @param    ulSize	    not used
* @param 	pVoid	    not used
* @return   -
*/
//*****************************************************************************
#pragma argsused
MAX_CALLBACK CbAcknowledge(MAXCHLHND hChannel, ULONG ulParam, ULONG ulSize, void* pVoid)
{
	// check parameter
    if (ulParam > 1)
        return;

	//a message was sent
	parameter.ulTmtPending[ulParam]--;
}


//*****************************************************************************
// MainProc
/** \brief (0): main procedure
*
* called cyclic after activation, increment timestamp
* @param    -
* @return   -
*/
//*****************************************************************************
void MAXEXPORT MainProc(void)
{
	max_entry_proc();
	
	parameter.ulTimeStampLo++;
	if (parameter.ulTimeStampLo == 0)
		parameter.ulTimeStampHi++;
	
	max_exit_proc();
}


//*****************************************************************************
// AutoInit
/** \brief (1): init task parameter
*
* Init task parameters, create ring buffers, activate task
* @param    -
* @return   -
*/
//*****************************************************************************
void MAXEXPORT AutoInit(void)
{
	MAX_BUFFER_TYPE rcBuffer;
	MAX_TI_TYPE rcTaskParams;
	ULONG	ulSize;
	USHORT usTask;
	
	usTask = max_entry_proc();
	
	//init global timestamp
	parameter.ulTimeStampHi = 0;
	parameter.ulTimeStampLo = 0;
	//init interface parameters
	parameter.arcCanParameters[0].ulBitRate = 250000L; //default: 250kBit
	parameter.arcCanParameters[1].ulBitRate = 250000L; //default: 250kBit
	parameter.arcCanParameters[0].ulFormat  = 11;      //default: 11 bit IDs
	parameter.arcCanParameters[1].ulFormat  = 11;      //default: 11 bit IDs
	parameter.arcCanParameters[0].ulRbus    = 1;       //default: termination on
	parameter.arcCanParameters[1].ulRbus    = 1;       //default: termination on
    parameter.arcCanParameters[0].ulCanId   = 0;       //default: receive all
	parameter.arcCanParameters[1].ulCanId   = 0;       //default: receive all
	parameter.arcCanParameters[0].ulCanMask = 0;       //default: receive all
	parameter.arcCanParameters[1].ulCanMask = 0;       //default: receive all
	//init handles
	parameter.hMdd     = 0;
	parameter.hCtrl[0] = 0;
	parameter.hCtrl[1] = 0;
	parameter.hIn11[0] = 0;
	parameter.hIn11[1] = 0;
	parameter.hIn29[0] = 0;
	parameter.hIn29[1] = 0;
	parameter.hOut[0]  = 0;
	parameter.hOut[1]  = 0;
	//default for CANbox
	parameter.usSlot = 2;
	//init callback counters
	parameter.ulCbFilterCount[0] = 0;
	parameter.ulCbFilterCount[1] = 0;
	parameter.ulTmtPending[0]    = 0;
	parameter.ulTmtPending[1]    = 0;
	parameter.ulCbCtrlCount[0]   = 0;
	parameter.ulCbCtrlCount[1]   = 0;

	max_init_lib(0);

	// get a handle for the X-MAX-1
	parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,&parameter.rcOsX, &parameter.hMyself);
	if (parameter.Error != ERR_OK)
		goto exit_auto_init;
	
	//allocate RCV buffer for interface 0
	ulSize = CAN_RCV_BUFFER_SIZE;
	rcBuffer.usStrategy = MAX_ALLOC_DOWN_ABS;
	rcBuffer.usAlignment = 2;
	rcBuffer.usTask = usTask;
	parameter.Error = max_create_buffer(parameter.hMyself, &rcBuffer, &ulSize, &parameter.hBuf[0]);
	if (parameter.Error != ERR_OK)
		goto exit_auto_init;

	//allocate RCV buffer for interface 1
	ulSize = CAN_RCV_BUFFER_SIZE;
	rcBuffer.usStrategy = MAX_ALLOC_DOWN_ABS;
	rcBuffer.usAlignment = 2;
	rcBuffer.usTask = usTask;
	parameter.Error = max_create_buffer(parameter.hMyself, &rcBuffer, &ulSize, &parameter.hBuf[1]);
	if (parameter.Error != ERR_OK)
		goto exit_auto_init;
	
	//wakeup myself
	if ((rcPDT.rcHead.usFlags & 0x0003) == MAX_TI_TASK)
	{
		if (parameter.Error == ERR_OK) 
		{
			rcTaskParams.usPriority  = 0;
			rcTaskParams.ulHoldOff   = 0;
			rcTaskParams.ulIntervall = 1; // = 1ms
			rcTaskParams.ulCalls     = 0;
			parameter.Error = max_wakeup_ti_task(parameter.hMyself, usTask, &rcTaskParams);
		}
	}
	if ((rcPDT.rcHead.usFlags & 0x0003) == MAX_NI_TASK)
	{
		if (parameter.Error == ERR_OK) 
		{
			parameter.Error = max_wakeup_ni_task(parameter.hMyself, usTask, 1);
		}
	}

exit_auto_init:
	
	max_exit_proc();
}

//*****************************************************************************
// Start
/** \brief (2): start CAN communication
*
* Start bus and open required channels.
* In case of error max_clear_error() is called and the procedure is left
* immediately.
* @param    -
* @return   -
*/
//*****************************************************************************
void MAXEXPORT Start(void)
{
	ULONG ulSize, ulParam;
	CPS_XCAN rcCps;
	XCAN_BIT_PARAMS rcBitParam;
	ULONG ulIndex;
	
	max_entry_proc();
	
	//load MDD
	if (parameter.hMdd == NULL)
		parameter.Error = max_load_mdd(parameter.hMyself, parameter.usSlot, 0, 0, 0x803A, NULL, &parameter.hMdd);
	if (parameter.Error != ERR_OK)
	{
		max_clear_error();
		max_exit_proc();
	}
	
	for (ulIndex=0; ulIndex<2; ulIndex++)
	{
		if (parameter.arcCanParameters[ulIndex].ulBitRate != 0)
		{
			// open control channel
			rcCps.usDevice       = DEVICE_CTRL;
			rcCps.usChannel      = ulIndex;
			rcCps.usIndexFirst   = XCAN_BUS_CTRL_INDEX;
			rcCps.usIndexLast    = XCAN_BUS_CTRL_INDEX;
			rcCps.usFlags        = 0;
			
			if (parameter.hCtrl[ulIndex] == NULL)
				parameter.Error = max_open_channel(parameter.hMdd,  sizeof(rcCps), &rcCps, CbControl, ulIndex, &parameter.hCtrl[ulIndex]);
			if (parameter.Error != ERR_OK)
			{
				max_clear_error();
				break;
			}
			
			// config interface
			ulSize = sizeof(XCAN_BIT_PARAMS);
			parameter.Error = GetBitTiming((USHORT)ulIndex, &rcBitParam);
			if (parameter.Error != ERR_OK)
			{
				max_clear_error();
				break;
			}
			parameter.Error = max_write_channel_block(parameter.hCtrl[ulIndex], &ulSize, &rcBitParam);
			if (parameter.Error != ERR_OK)
			{
				max_clear_error();
				break;
			}
			
			//open acceptance filter
			//11 bit
			if ((parameter.arcCanParameters[ulIndex].ulFormat & 11L) == 11L)
			{
				rcCps.usDevice               = DEVICE_CTRL;
            	rcCps.usIndexFirst           = 0;
            	rcCps.usIndexLast            = 0;
            	rcCps.usChannel              = (USHORT)ulIndex;
            	rcCps.ulCanID                = parameter.arcCanParameters[ulIndex].ulCanId;
            	rcCps.usFlags                = _CP_SYNC_CALLBACK;
				ulParam = 11;         //give format info in low word
				ulParam |= (ulIndex << 16); //give interface index in high word
				parameter.Error = max_open_channel(parameter.hMdd,  sizeof(rcCps), &rcCps, CbFilter, ulParam, &parameter.hIn11[ulIndex]);
				if (parameter.Error != ERR_OK)
				{
					max_clear_error();
					break;
				}
				parameter.Error = max_write_channel_ulong(parameter.hIn11[ulIndex], parameter.arcCanParameters[ulIndex].ulCanMask); 
				if (parameter.Error != ERR_OK)
				{
					max_clear_error();
 					break;
				}
			}
			//29 bit
			if ((parameter.arcCanParameters[ulIndex].ulFormat & 29L) == 29L)
			{
				rcCps.usDevice               = DEVICE_CTRL;
            	rcCps.usIndexFirst           = 0;
            	rcCps.usIndexLast            = 0;
            	rcCps.usChannel              = (USHORT)ulIndex;
            	rcCps.ulCanID                = parameter.arcCanParameters[ulIndex].ulCanId;
            	rcCps.usFlags                = XCAN_29BIT_ID | _CP_SYNC_CALLBACK;
				ulParam = 29;         //give format info in low word
				ulParam |= (ulIndex << 16); //give interface index in high word
				parameter.Error = max_open_channel(parameter.hMdd,  sizeof(rcCps), &rcCps, CbFilter, ulParam, &parameter.hIn29[ulIndex]);
				if (parameter.Error != ERR_OK)
				{
					max_clear_error();
					break;
				}
				parameter.Error = max_write_channel_ulong(parameter.hIn29[ulIndex], parameter.arcCanParameters[ulIndex].ulCanMask); 
				if (parameter.Error != ERR_OK)
				{
					max_clear_error();
					break;
				}
			}

			//open universal sender
			rcCps.usDevice               = DEVICE_BUS_OUT;
            rcCps.usChannel              = ulIndex;
            rcCps.ulCanID                = XCAN_GENERAL_ID;
            rcCps.usFlags                = _CP_EXCLUSIVE;
            rcCps.usTimeout              = 1000; //1ms
			parameter.Error = max_open_channel(parameter.hMdd,  sizeof(rcCps), &rcCps, CbAcknowledge, ulIndex, &parameter.hOut[ulIndex]);
			if (parameter.Error != ERR_OK)
			{
				max_clear_error();
				break;
			}
						
			//start bus
			parameter.Error = max_channel_control(parameter.hCtrl[ulIndex], CMD_START, 0, NULL);
		}//if (parameter.arcCanParameters[ulIndex].ulBitRate != 0)
	}//for (ulIndex=0; ulIndex<2; ulIndex++)
	
	
	max_exit_proc();
}


//*****************************************************************************
// Stop
/** \brief (3): stop CAN communication
*
* Stop bus and close all channels.
* In case of error max_clear_error() is called and the procedure is continued
* to get into a defined state again.
* @param    -
* @return   -
*/
//*****************************************************************************
void MAXEXPORT Stop(void)
{
	USHORT i;
	
	max_entry_proc();

	if (parameter.Error != ERR_OK)
		max_clear_error();

	//loop over all interfaces
	for (i=0; i<2; i++)
	{
		//interface is open
		if (parameter.hCtrl[i] != NULL)
		{
			//stop bus
			parameter.Error = max_channel_control(parameter.hCtrl[i], CMD_STOP, 0, NULL);
			if (parameter.Error != ERR_OK)
				max_clear_error();

			//close channels
			parameter.Error = max_close_channel(&parameter.hOut[i]);
			if (parameter.Error != ERR_OK)
				max_clear_error();
			parameter.hOut[i] = NULL;

			if (parameter.hIn11[i] != NULL)
			{
				parameter.Error = max_close_channel(&parameter.hIn11[i]);
				if (parameter.Error != ERR_OK)
					max_clear_error();
				parameter.hIn11[i] = NULL;
			}
			if (parameter.hIn29[i] != NULL)
			{
				parameter.Error = max_close_channel(&parameter.hIn29[i]);
				if (parameter.Error != ERR_OK)
					max_clear_error();
				parameter.hIn29[i] = NULL;
			}
			parameter.Error = max_close_channel(&parameter.hCtrl[i]);
			if (parameter.Error != ERR_OK)
				max_clear_error();
			parameter.hCtrl[i] = NULL;

			//clear ring buffer
			parameter.Error = max_clear_buffer(parameter.hBuf[i]);
			if (parameter.Error != ERR_OK)
				max_clear_error();
		}//if (parameter.hCtrl[i] != NULL)
	}//for (i=0; i<2; i++)
	
	max_exit_proc();
}


//*****************************************************************************
// Write0
/** \brief (5): send data via interface 0
*
* In case of error max_clear_func() is called as the error code is passed to
* the caller with max_exit_func.
* @param    pusLength	size of input data
* @param    prcCanMsg	pointer of type CAN_INTERFACE_WRITE
* @param    pusRetsize	not used
* @param    pRetData	not used
* @return   -
*/
//*****************************************************************************
#pragma argsused
void MAXEXPORT Write0(USHORT *pusLength, CAN_INTERFACE_WRITE *prcCanMsg, USHORT *pusRetsize, void *pRetData)
{
	XCAN_MESSAGE rcCanMessage;
	ULONG ulSize;
	
	max_entry_func();

	//check buffer size
	if (*pusLength < sizeof(CAN_INTERFACE_WRITE))
		max_exit_func(ERR_WRONG_NUMBER_OF_INPUTS);
	
	//compose message for sending
	rcCanMessage.ulCanId = prcCanMsg->ulId;
	rcCanMessage.usDataSize = (USHORT)prcCanMsg->ulDataSize;
	if (prcCanMsg->ulFormat == 11L)
		rcCanMessage.usFlags = 0;
	else
		rcCanMessage.usFlags = XCAN_29BIT_ID;
	
	memcpy(&rcCanMessage.aucData[0], &(prcCanMsg->aucData[0]), rcCanMessage.usDataSize);
	
	//send message
	ulSize = sizeof(rcCanMessage);
	parameter.Error = max_write_channel_block(parameter.hOut[0], &ulSize, &rcCanMessage);

	if (parameter.Error == ERR_OK)
		parameter.ulTmtPending[0]++;
	else
		max_clear_error();
	
	max_exit_func(parameter.Error);
}


//*****************************************************************************
// Write1
/** \brief (7): send data via interface 1
*
* In case of error max_clear_func() is called as the error code is passed to
* the caller with max_exit_func.
* @param    pusLength	size of input data
* @param    prcCanMsg	pointer of type CAN_INTERFACE_WRITE
* @param    pusRetsize	not used
* @param    pRetData	not used
* @return   -
*/
//*****************************************************************************
#pragma argsused
void MAXEXPORT Write1(USHORT *pusLength, CAN_INTERFACE_WRITE *prcCanMsg, USHORT *pusRetsize, void *pRetData)
{
	XCAN_MESSAGE rcCanMessage;
	ULONG ulSize;
	
	max_entry_func();

	//check buffer size
	if (*pusLength < sizeof(CAN_INTERFACE_WRITE))
		max_exit_func(ERR_WRONG_NUMBER_OF_INPUTS);
	
	//compose message for sending
	rcCanMessage.ulCanId = prcCanMsg->ulId;
	rcCanMessage.usDataSize = (USHORT)prcCanMsg->ulDataSize;
	if (prcCanMsg->ulFormat == 11L)
		rcCanMessage.usFlags = 0;
	else
		rcCanMessage.usFlags = XCAN_29BIT_ID;
	
	memcpy(&rcCanMessage.aucData[0], &(prcCanMsg->aucData[0]), rcCanMessage.usDataSize);
	
	//send message
	ulSize = sizeof(rcCanMessage);
	parameter.Error = max_write_channel_block(parameter.hOut[1], &ulSize, &rcCanMessage);

	if (parameter.Error == ERR_OK)
		parameter.ulTmtPending[1]++;
	else
		max_clear_error();
	
	max_exit_func(parameter.Error);
}

//*****************************************************************************
// Read0
/** \brief (4): read data from interface 0
*
* In case of error max_clear_func() is called as the error code is passed to
* the caller with max_exit_func.
* @param    pusLength	not used
* @param    pIn			not used
* @param    pusRetsize	size of the data buffer
* @param    prcCanMsg	pointer of type CAN_INTERFACE_READ
* @return   -
*/
//*****************************************************************************
#pragma argsused
void MAXEXPORT Read0(USHORT *pusLength, void *pIn, USHORT *pusRetsize, CAN_INTERFACE_READ *prcCanMsg)
{
	ULONG ulValid, ulFree, ulSize;

	max_entry_func();

	//check buffer size
	if (*pusRetsize < sizeof(CAN_INTERFACE_READ))
		max_exit_func(ERR_WRONG_NUMBER_OF_RETURNS);
	
	//check if there are messages in the buffer
	parameter.Error = max_get_buffer_status(parameter.hBuf[0], &ulValid, &ulFree);
	if (ulValid >= sizeof(CAN_INTERFACE_READ))
	{
		//limit read size to max. available data
		if ((ULONG)*pusRetsize > ulValid)
			*pusRetsize = (USHORT)ulValid;
		//align read size to multiples of sizeof(CAN_INTERFACE_READ)
		ulSize = (ULONG)*pusRetsize / sizeof(CAN_INTERFACE_READ);
		ulSize *= sizeof(CAN_INTERFACE_READ);

		parameter.Error = max_get_buffer(parameter.hBuf[0], MAX_BUFFER_MOVE_ABS, &ulSize, (void*)prcCanMsg);
		*pusRetsize = (USHORT)ulSize;
	}
	else
	{
		*pusRetsize = (USHORT)0;
	}

	if (parameter.Error != ERR_OK)
		max_clear_error();
	
	max_exit_func(parameter.Error);
}


//*****************************************************************************
// Read1
/** \brief (6): read data from interface 1
*
* In case of error max_clear_func() is called as the error code is passed to
* the caller with max_exit_func.
* @param    pusLength	not used
* @param    pIn			not used
* @param    pusRetsize	size of the data buffer
* @param    prcCanMsg	pointer of type CAN_INTERFACE_READ
* @return   -
*/
//*****************************************************************************
#pragma argsused
void MAXEXPORT Read1(USHORT *pusLength, void *pIn, USHORT *pusRetsize, CAN_INTERFACE_READ *prcCanMsg)
{
	ULONG ulValid, ulFree, ulSize;
	max_entry_func();
	
	//check buffer size
	if (*pusRetsize < sizeof(CAN_INTERFACE_READ))
		max_exit_func(ERR_WRONG_NUMBER_OF_RETURNS);

	//check if there are messages in the buffer
	parameter.Error = max_get_buffer_status(parameter.hBuf[1], &ulValid, &ulFree);
	if (ulValid >= sizeof(CAN_INTERFACE_READ))
	{
		//limit read size to max. available data
		if ((ULONG)*pusRetsize > ulValid)
			*pusRetsize = (USHORT)ulValid;
		//align read size to multiples of sizeof(CAN_INTERFACE_READ)
		ulSize = (ULONG)*pusRetsize / sizeof(CAN_INTERFACE_READ);
		ulSize *= sizeof(CAN_INTERFACE_READ);

		parameter.Error = max_get_buffer(parameter.hBuf[1], MAX_BUFFER_MOVE_ABS, &ulSize, (void*)prcCanMsg);
		*pusRetsize = (USHORT)ulSize;
	}
	else
	{
		*pusRetsize = (USHORT)0;
	}

	if (parameter.Error != ERR_OK)
		max_clear_error();
	
	max_exit_func(parameter.Error);
}


//*****************************************************************************
// main
/** \brief initialize the Program Descriptor Table (PDT);
* called once on installing the program
* @param    -
* @return   -
*/
//*****************************************************************************
void main ()
{
	// the PDT-flags determine the task-type and
	// information how the data and parameter range are allocated
	
	//set flags for task type
	rcPDT.rcHead.usFlags         = MAX_TI_TASK | MAXPDT_PARAMETER_IN_PROG;
	
	// interrupt source
	if ((rcPDT.rcHead.usFlags & 0x0003) == MAX_TI_TASK)
		rcPDT.rcHead.usInterrupt     = MAX1_IRQ_TIMERC;
	else
		rcPDT.rcHead.usInterrupt     = 0;
	
	// the program number identifies the program uniquely
	rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;
	
	// number of global task procedures in the PDT
	rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;
	
	// the following lines are just for documentation and information purpose
	rcPDT.rcHead.ucType          = 1;   // always !
	rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
	rcPDT.rcHead.cVersion        = VERSION;
	rcPDT.rcHead.cRevision       = REVISION;
	rcPDT.rcHead.ucCPU           = MAX_CPU486;
	rcPDT.rcHead.ucCoProc        = _NOCOPROC;
	rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
	rcPDT.rcHead.ucProgType      = 0;   // not used
	rcPDT.rcHead.ulRes1          = 0;
	rcPDT.rcHead.ulRes2          = 0;
	rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet
	
	// address and size of the tasks data range
	// only when MAXPDT_DATA_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulDataAdr       = 0;
	rcPDT.rcHead.ulDataSize      = 0;
	
	// address and size of the tasks parameter range
	// only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
	rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);
	
	// initialize global procedure addresses in PDT
	rcPDT.main_proc        = MainProc;
	rcPDT.auto_init        = AutoInit;
	rcPDT.Start            = Start;
	rcPDT.Stop             = Stop;
	rcPDT.Read0            = Read0;
	rcPDT.Read1            = Read1;
	rcPDT.Write0           = Write0;
	rcPDT.Write1           = Write1;
	
	// transfer PDT information to the OsX
	max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
