/** \file minican.h
 * \brief header for example RT program for X-CAN-2
 * can also be used from PC programs to access the RT task
 *
 */

#ifndef _MINICAN_H_
#define _MINICAN_H_

#ifndef __BORLANDC__
#pragma pack(push, SORCPACK, 1) 
#else
#pragma option -a-
#endif

#define PROGRAMM_NUMBER 0x9020   //!< program number
#define CAN_TASK_NUMBER 0x300    //!< task number of this example
#define VERSION         '1'      //!< version of program
#define REVISION        'C'      //!< revision of program

#define CAN_RCV_BUFFER_SIZE 0x100000L //!< 1MB


/**
* \struct CAN_INTERFACE_READ
* \brief data structure for receiving messages
*
* Received CAN messages are stored in this structure.
* This data type is also used when reading messages via
* function 4 (\ref Read0) or 6 (\ref Read1)
*/
typedef struct 
{
   ULONG    ulTimeStampHi;  //!< upper 32 bit of timestamp
   ULONG    ulTimeStampLo;  //!< lower 32 bit of timestamp
   ULONG    ulFormat;       //!< message format: 11 | 29        
   ULONG    ulId;           //!< message ID
   ULONG    ulDataSize;     //!< size of received CAN message
   UCHAR    aucData[8];     //!< buffer for CAN message
} CAN_INTERFACE_READ;


/**
* \struct CAN_INTERFACE_WRITE 
* \brief data structure for sending messages
*
* This data structure is used when providing a message
* for sending via function 5 (\ref Write0) or 7 (\ref Write1)
*/
typedef struct 
{       
   ULONG    ulId;       //!< message ID
   ULONG    ulFormat;   //!< message format: 11 | 29
   ULONG    ulDataSize; //!< size of CAN message to send
   UCHAR    aucData[8]; //!< buffer for CAN message
} CAN_INTERFACE_WRITE;


/**
* \struct CAN_INTERFACE_PARAMETERS
* \brief interface parameters
*
* For each of the two interfaces such a structure is provided
* in parameter space. It can be read and written using
* max_read/write_par_block (offset \ref CAN_PARAM_INTERFACE0 or \ref CAN_PARAM_INTERFACE1).
* Settings must be made before calling procedure 2 (\ref Start) to take effect.
* \see parameter_t
*/
typedef struct
{
    ULONG   ulBitRate;  //!< bit rate in bits/s; == 0: interface is not used
    ULONG   ulCanId;    //!< IDs accepted by the acceptance filter
    ULONG   ulCanMask;  //!< mask used for the acceptance filter
    ULONG   ulFormat;   //!< == 11: handle 11 bit identifier; == 29: handle 29 bit identifier; == 31: use mixed mode
    ULONG   ulRbus;     //!< == 0: termination resistor off; != 0: termination resistor on
} CAN_INTERFACE_PARAMETERS;

/**
* \struct CAN_INTERFACE_STATE
* \brief interface state
*
* This structure is provided in the parameter space to
* indicate the current bus state. It can be read using
* max_read_par_block (offset \ref CAN_PARAM_STATE0 or \ref CAN_PARAM_STATE1).
* \see parameter_t
*/
typedef struct 
{
    ULONG   ulTimeStampHi;  //!< upper 32 bit of timestamp
    ULONG   ulTimeStampLo;  //!< lower 32 bit of timestamp
    USHORT  usBusState;     //!< bus state: XCAN_BUS_ON, XCAN_BUS_HEAVY, XCAN_BUS_OFF, XCAN_LINE_ERROR
    USHORT  usBusInfo;      //!< error info: 1=stuff error, 2=format error, 3=Ack error, 4+5=bit error, 6=CRC error
} CAN_INTERFACE_STATE;


#ifndef __BORLANDC__
#pragma pack(pop, SORCPACK)
#endif

//task parameters
#define CAN_PARAM_ERROR         0       //!< offset of the global error state
#define CAN_PARAM_TIMESTAMP_HI  2       //!< offset of upper 32 bit of current timestamp
#define CAN_PARAM_TIMESTAMP_LO  6       //!< offset of lower 32 bit of current timestamp
#define CAN_PARAM_INTERFACE0    10      //!< offset of parameters for interface 0
#define CAN_PARAM_INTERFACE1    30      //!< offset of parameters for interface 1
#define CAN_PARAM_SLOT          50      //!< offset of the slot number of X-CAN-2
#define CAN_PARAM_STATE0        80      //!< offset of interface 0 state
#define CAN_PARAM_STATE1        92      //!< offset of interface 1 state
#define CAN_PARAM_COUNT_FILTER0 104     //!< offset of interface 0 RCV counter
#define CAN_PARAM_COUNT_FILTER1 108     //!< offset of interface 1 RCV counter
#define CAN_PARAM_COUNT_ACK0    120     //!< offset of interface 0 ACK counter
#define CAN_PARAM_COUNT_ACK1    124     //!< offset of interface 1 ACK counter
#define CAN_PARAM_COUNT_CTRL0   112     //!< offset of interface 0 ERR counter
#define CAN_PARAM_COUNT_CTRL1   116     //!< offset of interface 1 ERR counter

//task functions
#define CAN_START_INTERFACES    2       //!< procedure Start
#define CAN_STOP_INTERFACES     3       //!< procedure Stop
#define CAN_READ_INTERFACE0     4       //!< function Read0
#define CAN_WRITE_INTERFACE0    5       //!< function Write0
#define CAN_READ_INTERFACE1     6       //!< function Read1
#define CAN_WRITE_INTERFACE1    7       //!< function Write1

//mixed ID
#define CAN_MIXED_ID            (11 | 29)   //!< identifier for mixed mode IDs

#endif
