/****************************************************************************/
// Sample Real Time Program to show the usage of the CQmax-drivers
// for the serial interface of the X-MAX-1 and X-MAX-E modules.
// The following issues are shown:
// 1. Initialization and parametrization of the CQmax drivers
// 2. Transmission of data
// 3. Reception of data
/****************************************************************************/

#pragma option -r-

#define _RTLIB_                        // must be defined in every real time
// program before including max_lib.h
#include "max_lib.h"
#include "x_error.h"
#include "x_mdd.h"
#include "dos.h"
#include "stdlib.h"
#include "string.h"

#define PROGRAMM_NUMBER 0x9013
#define VERSION         '1'		//version of program
#define REVISION        'A'		//revision of program

// PDT structure
struct pdt_type
{
	MAX_PDT_HEAD    rcHead;
	void* MAXEXPORT main_proc;
	void* MAXEXPORT auto_init;
	void* MAXEXPORT start;
	void* MAXEXPORT transmit;
	// any number of user defined procedures may follow (optionally):
	
} rcPDT;

// parameter range structure
struct parameter_type
{
	MAX_TDT_TYPE rcTDT;                 // always before the first parameter !
	// offset 0
	MAX_ERROR   Error;                 	// Offset 0: Errorcode
	// communication parameters: write before calling proc #2 to activate it
	UCHAR			ucBitParChar;				// Offset 2: number of bits par character			
	UCHAR			ucNoOfStopBits;			// Offset 3: number of stop bits 
	ULONG			ulBaudrate;					// Offset 4: baudrate
	UCHAR			ucParity;					// Offset 8: parity
	UCHAR			ucProtocol;					// Offset 9: protocol
	USHORT		usTmtLength;				// Offset 10: number of bytes to be transmitted 
													// with every call of procedure 3
	char			acTmtData[500];			// Offset 12: data to be transmitted
	// the follwoing parameter are set by this task and are read only 
	USHORT		usCQTask;					// Offset 512: task number of the CQmax driver
	UCHAR		 	ucCPULocation;				// Offset 514: location of the CPU module
	UCHAR			ucState;						// Offset 515: dummy
	char			acRcvData[512];			// Offset 516: received data

	MAXMODHND    hMyself;               // Handle for the CPU module
	MAX_OS_INFO  rcOsX;                 // Version info about OsX
	
	//fill in further parameters below:
	
} parameter;


/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
void MAXEXPORT MainProc(void)
{
   USHORT      usInsize, usOutsize, usIndata, usOutdata, usCtrl;
   struct
   {
      USHORT  usState;
      ULONG   ulBuffer;
   } rcState;

	max_entry_proc();
	// first check if there is data available in the receive buffer via CQ function 16
   usOutsize = 0;
   usInsize  = sizeof(rcState);
  	parameter.Error = max_call_func(parameter.hMyself, parameter.usCQTask, 16,
							&usOutsize, &usOutdata, &usInsize, &rcState);
	if (parameter.Error == ERR_OK)
   {
		if (rcState.ulBuffer > 0)
		{	// if data available read it out
    		usOutsize = 0;
		   usInsize  = (USHORT)rcState.ulBuffer;
        	parameter.Error = max_call_func(parameter.hMyself, parameter.usCQTask, 17, 
										&usOutsize, &usOutdata, &usInsize, parameter.acRcvData);
		}	
	}
	max_exit_proc();
}

/****************************************************************************/
// Init task parameter
void MAXEXPORT AutoInit(void)
{
	USHORT 	usCPUSlot, usCPULayer;
	max_entry_proc();
	
	max_init_lib(0);
	// set default communication parameter (9600, 8, n, 1)
	parameter.ulBaudrate = 0x9600; 	// this is 9600 Baud
	parameter.ucBitParChar = 8;
	parameter.ucNoOfStopBits = 1;
	parameter.ucParity = 0;
	parameter.ucProtocol = 0;
	strcpy(parameter.acTmtData, "This is a test string");
	parameter.usTmtLength = strlen(parameter.acTmtData);

	// get a handle for the X-MAX-1
	parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,&parameter.rcOsX, &parameter.hMyself);

	if (parameter.Error == ERR_OK)
		parameter.Error = max_get_slot (parameter.hMyself, &usCPUSlot, &usCPULayer);
	// transform CPU location into the format that is required by CQmax:
	parameter.ucCPULocation = (UCHAR)usCPUSlot;
	parameter.ucCPULocation <<= 4;
	parameter.ucCPULocation += (UCHAR)usCPULayer;
	max_exit_proc();
};

/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
//	Checks if the CQ driver is installed successfully and sets the parameter
void MAXEXPORT Start(void)
{
	USHORT 	usInstance, usOutsize, usCtrl, usInsize, usIndata;
	UCHAR		ucState;
	max_entry_proc();

	// check out if the CQ-driver-program A008hex is installed and get its task number
	usInstance = 0;
	parameter.Error = max_get_task_number_prog(parameter.hMyself, 0xA008, 
															&usInstance, &parameter.usCQTask);
	if (parameter.Error == ERR_OK)
	{
		if (usInstance > 1)
	  	{	// CQ-driver-program A008hex is installed more than once, e.g. for X-COM-4 modules.
			// It is possible to find out which of the tasks is responsible for the 
			// X-MAX-1 resp. X-MAX-E by checking parameter 8 of each task and compare it 
			// to the slot of the CPU module
	  	}
		else
		{	// CQ-driver-program A008hex is installed once. Check its status:
			parameter.Error = max_read_par_uchar(parameter.hMyself, parameter.usCQTask, 0, &ucState);
			if (ucState == 0)	
			{	// CQmax has not been initialized - Write the basic parameters now:
			   max_write_par_uchar(parameter.hMyself, parameter.usCQTask, 8, parameter.ucCPULocation);
			   max_write_par_uchar(parameter.hMyself, parameter.usCQTask, 9, 0);		// channel: always 0
			   max_write_par_uchar(parameter.hMyself, parameter.usCQTask, 10, 1); 	// interface type: always RS-232
				// set size of CQmax internal data buffer
			   max_write_par_ulong(parameter.hMyself, parameter.usCQTask, 40, 512);
			   max_write_par_ulong(parameter.hMyself, parameter.usCQTask, 48, 512);

				// write the application specific parameters now:
			   max_write_par_ulong(parameter.hMyself, parameter.usCQTask, 24, parameter.ulBaudrate);
			   max_write_par_ulong(parameter.hMyself, parameter.usCQTask, 34, parameter.ulBaudrate);
			   max_write_par_uchar(parameter.hMyself, parameter.usCQTask, 28, parameter.ucBitParChar);
			   max_write_par_uchar(parameter.hMyself, parameter.usCQTask, 29, parameter.ucNoOfStopBits);
			   max_write_par_uchar(parameter.hMyself, parameter.usCQTask, 30, parameter.ucParity);
			   max_write_par_ushort(parameter.hMyself, parameter.usCQTask, 318, (USHORT)parameter.ucProtocol);

  				// call CQmax start procedure
	        	parameter.Error = max_call_proc(parameter.hMyself, parameter.usCQTask, 2);
				if (parameter.Error == ERR_OK)	    
			  	{
	        		max_read_par_uchar(parameter.hMyself, parameter.usCQTask, 0, &ucState);
	        		if (ucState == 3)
					{	// CQmax signals an error. Reason can be read from CQmax-task parameter 1
						parameter.Error = ERR_ILLEGAL_COMMAND;		
					}
					else
				  	{	// enable reception
					   usOutsize = sizeof(usCtrl);
					   usInsize  = 0;
						usCtrl 	 = 3;	// clear receive buffer and anable receiver
			        	parameter.Error = max_call_func(parameter.hMyself, parameter.usCQTask, 15,
												&usOutsize, &usCtrl, &usInsize, &usIndata);
						// activate the task to poll the receive state permanently
						if (parameter.Error == ERR_OK)	    
					  	{
							parameter.Error = max_wakeup_ni_task(parameter.hMyself, parameter.rcTDT.usTaskNo, 1);
						}
					}
				}
			}
	  	}
  	}
	max_exit_proc();
}


/****************************************************************************/
// This procedure transmits some bytes
void MAXEXPORT Transmit(void)
{
   USHORT      usInsize, usOutsize, usIndata, usOutdata, usCtrl;
   struct
   {
       USHORT  usState;
       ULONG   ulBuffer;
   } rcState;

	max_entry_proc();

	// first check if transmission is enabled via CQ function 6
  	usOutsize = 0;
   usInsize  = sizeof(rcState);
   parameter.Error = max_call_func(parameter.hMyself, parameter.usCQTask, 6, 
											&usOutsize, &usOutdata, &usInsize, &rcState);
	if (parameter.Error == ERR_OK)
	{
		if (rcState.usState & 0x8000)
		{	// if transmission is stopped, enable it now via CQ function 5
		   usOutsize = sizeof(usCtrl);
    		usInsize  = 0;
			usCtrl	 = 3; 	// enable transmission, clear transmit buffer
        	parameter.Error = max_call_func(parameter.hMyself, parameter.usCQTask, 5, 
									&usOutsize, &usCtrl, &usInsize, &usIndata);
			// transmit remains enabled after this call
		}
	}

	// write transmit data to CQmax via CQ function 7
   usOutsize = parameter.usTmtLength;
   usInsize  = 0;
   parameter.Error = max_call_func(parameter.hMyself, parameter.usCQTask, 7, 
											&usOutsize, parameter.acTmtData, &usInsize, &usIndata);
	max_exit_proc();
}


/****************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
	// the PDT-flags determine the task-type and
	// information how the data and parameter range are allocated
	
	//set flags for task type
	rcPDT.rcHead.usFlags         = MAX_NI_TASK +	MAXPDT_PARAMETER_IN_PROG;
	
	// interrupt source
	rcPDT.rcHead.usInterrupt     = 0;
	
	// the program number identifies the program uniquely
	rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;
	
	// number of global task procedures in the PDT
	rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;
	
	// the following lines are just for documentation and information purpose
	rcPDT.rcHead.ucType          = 1;   // always !
	rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
	rcPDT.rcHead.cVersion        = VERSION;
	rcPDT.rcHead.cRevision       = REVISION;
	rcPDT.rcHead.ucCPU           = MAX_CPU486;
	rcPDT.rcHead.ucCoProc        = _NOCOPROC;
	rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
	rcPDT.rcHead.ucProgType      = 0;   // not used
	rcPDT.rcHead.ulRes1          = 0;
	rcPDT.rcHead.ulRes2          = 0;
	rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet
	
	// address and size of the tasks data range
	// only when MAXPDT_DATA_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulDataAdr       = 0;
	rcPDT.rcHead.ulDataSize      = 0;
	
	// address and size of the tasks parameter range
	// only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
	rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);
	
	// initialize global procedure addresses in PDT
	rcPDT.main_proc        = MainProc;
	rcPDT.auto_init        = AutoInit;
	rcPDT.start        	  = Start;
	rcPDT.transmit         = Transmit;

	// transfer PDT information to the OsX
	max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
