/****************************************************************************
*	CALLBACK.C	
*
*	PURPOSE
*		Sample Real Time program
* 		The program show handling of X-DIO-40/i callback-functions. The 
*		callback-function will be called at pos. edge of DIN-1. At each call the
*		LED will be toggled.
*	INSTALL-PARAMETER
*		Command for SORCUS INS-File: MAXINST file="CALLBACK.EXE" no=900E autoinit
*	SETTINGS
*		To use this program, you have to:
*			- install the MDD for X-DIO-40/i
*	COPYRIGHT
*		(C)2003 SORCUS Computer GmbH
*	HISTORY
*		- 2003-10-21 (JS) 1.A: First Version
*
****************************************************************************/

#pragma option -r-

// _RTLIB_ must be defined in every real time program
#define _RTLIB_

// including header files
#include "max_lib.h"
#include "x_error.h"
#include "x_mdd.h"
#include "dos.h"
#include "stdlib.h"

// program version
#define VERSION         	'1'
#define REVISION        	'A'
#define PROGRAM_NUMBER		0x900E

// status of program
#define PROG_INIT			0x1
#define PROG_START		0x2
#define PROG_RUN			0x3
#define PROG_STOP			0x4

// PDT (Program Descriptor Table) data structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
   void* MAXEXPORT start;
   void* MAXEXPORT stop;
} rcPDT;

// parameter range of the task
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;
	MAX_ERROR    Error;				// error of program
	USHORT		 usStatus;			// status of program
	USHORT       usSlot;				// slot of module
	USHORT		 usLayer;			// layer of module
   MAXMODHND    hMyself;        	// handle for the CPU
	MAXMDDHND    hMdd;            // handle for the MDD          
	MAXCHLHND    hDin;				// handle to DIN channel
  	MAXCHLHND    hLed;				// handle to DIN channel
	MAX_OS_INFO  rcOsX;				// OsX
	USHORT       usCounter;
} rcParameter;

// Prototypes
void MAXEXPORT MainProc (void);
void MAXEXPORT AutoInit (void);
void MAXEXPORT Start (void);
void MAXEXPORT Stop (void);


MAX_CALLBACK MyCallbackFunction(MAXCHLHND handle, ULONG param, ULONG size, void* pData)
{
	UCHAR ucLed;

	rcParameter.usCounter++;
	if (rcParameter.Error == ERR_OK)
	{
		// toggle LED
		rcParameter.Error = max_read_channel_uchar(rcParameter.hLed, &ucLed);
		if (ucLed != 0)
		{
			rcParameter.Error = max_write_channel_uchar(rcParameter.hLed, 0);
		}
		else
		{
			rcParameter.Error = max_write_channel_uchar(rcParameter.hLed, 1);
		}
	}
}



/*************************************************************************
*			AutoInit
*************************************************************************/
void MAXEXPORT AutoInit(void)
{
	max_entry_proc();

   // init library
	rcParameter.Error = max_init_lib(0);
   if (rcParameter.Error == ERR_OK) 
	{
      // get a handle for the X-MAX-1
      rcParameter.Error = max_connect_cpu (0, MAX_MYSELF, 0, &rcParameter.rcOsX, &rcParameter.hMyself);
   }

	// set slot and layer of X-DIO-40
	rcParameter.usSlot = 6;
	rcParameter.usLayer = 0;

	// status
	rcParameter.usStatus = PROG_INIT;

	rcParameter.usCounter = 0;

   max_exit_proc();
};



/*************************************************************************
*			Start
*************************************************************************/
void MAXEXPORT Start(void)
{
   USHORT         i;
   CPS_XDIO40_A   rcDin, rcDout, rcDio, rcLed;

   max_entry_proc();

	// check error
	if (rcParameter.Error == ERR_OK)
	{
		// get a handle for the MDD
		rcParameter.Error = max_load_mdd (rcParameter.hMyself, rcParameter.usSlot, rcParameter.usLayer, 0, MDD_X_DIO_40, NULL, &rcParameter.hMdd);
	   if (rcParameter.Error == ERR_OK) 
		{
			// fill in CPS for LED channel
			rcLed.usDevice      = DEVICE_LED;
			rcLed.usIndexFirst  = 0;
			rcLed.usIndexLast   = 0;
			rcLed.usReadMode    = IO_MODE_DIRECT;
			rcLed.usWriteMode   = IO_MODE_DIRECT;
			rcLed.usFlags       = 0;
			rcLed.usMode        = 0;
			// open channel
		   rcParameter.Error = max_open_channel (rcParameter.hMdd, sizeof (rcLed), &rcLed, NULL, 0, &rcParameter.hLed);

			// fill in CPS for DIN channel 0 ... 3
			rcDin.usDevice      = DEVICE_DIN;
			rcDin.usIndexFirst  = 0;
			rcDin.usIndexLast   = 3;
			rcDin.usReadMode    = IO_MODE_DIRECT;
			rcDin.usWriteMode   = 0;
			rcDin.usFlags       = _CP_EXCLUSIVE |
										 _CP_SYNC_CALLBACK;		// callback-function will be called directly
			rcDin.usMode        = 0;
			for (i=0; i<3; i++)
			{
				rcDin.rcCallbackEvents.ausPosEdge[i] = 0;
				rcDin.rcCallbackEvents.ausNegEdge[i] = 0;
			}
			rcDin.rcCallbackEvents.ausPosEdge[0] = 0x2;	// callback-function has to be called at
																		// pos. edge of DIN-1
			// open channel with callback-function
		   rcParameter.Error = max_open_channel (rcParameter.hMdd, sizeof (rcDin), &rcDin, MyCallbackFunction, 0, &rcParameter.hDin);

	     	if (rcParameter.Error == ERR_OK)
		  	{
				// activate task
				rcParameter.Error = max_wakeup_ni_task(rcParameter.hMyself, rcParameter.rcTDT.usTaskNo, 1);
		  	}
		}
	}

	// status
	rcParameter.usStatus = PROG_START;

   max_exit_proc();
}



/*************************************************************************
*			MainProc
*************************************************************************/
void MAXEXPORT MainProc(void)
{
   UCHAR ucLed, ucOutData;
	MAX_ERROR Error;

   max_entry_proc();

	// status
	rcParameter.usStatus = PROG_RUN;

   max_exit_proc();
}



/*************************************************************************
*			Stop
*************************************************************************/
void MAXEXPORT Stop(void)
{
   max_entry_proc();

	// deactivate task
   rcParameter.Error = max_sleep_task (rcParameter.hMyself, rcParameter.rcTDT.usTaskNo);

	// close channels
	rcParameter.Error = max_close_channel(&rcParameter.hDin);
	rcParameter.Error = max_close_channel(&rcParameter.hLed);

	// status
	rcParameter.usStatus = PROG_STOP;

   max_exit_proc();
}




/*************************************************************************
*
*************************************************************************/
void main ()
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_NI_TASK + MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = 0;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = PROGRAM_NUMBER;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&rcParameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(rcParameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;
   rcPDT.start            = Start;
   rcPDT.stop             = Stop;
   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
